Public Class Spritz3D
    Private mobjGraphics As DX9ToolsR4.DX9Graphics
    Private mobjForm As Form
    Private mobjSprite As DX9ToolsR4.Sprites.SpriteBlt
    Private mobjKB As DirectInput.Device
    Private mobjObjects() As Object
    Private mobjTextures() As Direct3D.Texture
    Private mblnRunning, mblnGameLoaded, mblnOutsideControl As Boolean
    Private mobjMap As Map
    Private mobjVisibleSectors(1) As SectorIndex
    Private mobjGamePlayArea, mobjVisibleGamePlayArea As Rectangle
    Private mobjSectorSize As Size

    Public Event Idle()

    Public ReadOnly Property TextureCount() As Integer
        Get
            Return mobjTextures.Length
        End Get
    End Property

#Region " Add Object overloads ... "
    Public Function AddObject(ByVal Sprite As Interfaces.ISprite, Optional ByVal LayerIndex As Integer = 0) As Integer
        Dim Sector As SectorIndex

        ' allocate room for sprite
        If mobjObjects Is Nothing Then
            ReDim mobjObjects(0)
        Else
            ReDim Preserve mobjObjects(mobjObjects.Length)
        End If
        mobjObjects(mobjObjects.Length - 1) = Sprite

        ' determin what sector to add sprite to and add it
        Sector = GetSectorFromLocation(Helpers.Conversion.Vector2ToPoint(Sprite.Translation))
        mobjMap.GetLayer(LayerIndex).GetSector(Sector).AddSprite(mobjObjects.Length - 1)

        ' return index that sprite was inserted at
        Return mobjObjects.Length - 1
    End Function
#End Region

    Public Function GetObject(ByVal Index As Integer) As Object
        Return mobjObjects(Index)
    End Function

#Region " GetSectorFromLocation overloads ... "
    Public Function GetSectorFromLocation(ByVal Location As Point) As SectorIndex
        Dim Tmp As SectorIndex

        ' check if out of bounds
        If Not General.InBounds(Location, mobjVisibleGamePlayArea.Size) Then
            Throw New ArgumentOutOfRangeException("Location", Location, "Location needs to be within the bounds of the VisibleGamePlaySize property.")
        End If

        ' calculate sector from location given
        Tmp.Column = ((mobjVisibleGamePlayArea.X + Location.X) \ mobjSectorSize.Width)
        Tmp.Row = ((mobjVisibleGamePlayArea.Y + Location.Y) \ mobjSectorSize.Height)

        ' return sector index
        Return Tmp
    End Function
#End Region

    Public Function LoadTexture(ByVal Filename As String) As Integer
        If mobjTextures Is Nothing Then
            ReDim mobjTextures(0)
        Else
            ReDim Preserve mobjTextures(mobjTextures.Length)
        End If

        mobjTextures(mobjTextures.Length - 1) = Direct3D.TextureLoader.FromFile(mobjGraphics.Device, Filename)
        Return mobjTextures.Length - 1
    End Function

    Private Sub InitGraphics(ByVal Ctrl As Control)
        If mblnOutsideControl Then
            ' do nothing
        Else
            '
            ' Init form
            '
            If TypeOf Ctrl Is Form Then
                With CType(Ctrl, Form)
                    .MaximizeBox = False
                    .MinimizeBox = False
                    .ControlBox = False
                    .Text = ""
                    .Icon = Nothing
                    .FormBorderStyle = FormBorderStyle.None
                    .WindowState = FormWindowState.Normal
                    .StartPosition = FormStartPosition.CenterScreen
                    .Size = New Size(640, 480)
                    .TopMost = True
                End With
                mobjForm.Show()
            End If
        End If

        ' init graphics
        mobjGraphics = New DX9ToolsR4.DX9Graphics(Ctrl)
    End Sub

    Private Sub OneTimeInit()
        ' create map
        mobjSectorSize = New Size(64, 64)
        mobjMap = New Spritz3DR1.Map(Me, 1, mobjSectorSize, New Size(1, 1))

        ' calc visible sectors
        CalculateVisibleSectors()

        ' init sptrite
        mobjSprite = New DX9ToolsR4.Sprites.SpriteBlt(mobjGraphics.Device)

        ' init input
        ' InitInput()

        ' setup misc varibles
        mblnGameLoaded = False
        mblnRunning = False
    End Sub


    Friend Sub CalculateVisibleSectors()
        mobjVisibleSectors(0) = New SectorIndex(0, 0)
        mobjVisibleSectors(1) = New SectorIndex(0, 0)
    End Sub

    Public Sub New(ByVal Ctrl As Control)
        MyBase.new()

        mblnOutsideControl = True
        InitGraphics(Ctrl)
        OneTimeInit()
    End Sub

    Public Sub New()
        MyBase.new()

        mblnOutsideControl = False
        mobjForm = New Form()
        InitGraphics(mobjForm)

        OneTimeInit()
    End Sub

    Public Sub LoadGameFile(ByVal Filename As String)
        'Dim intDataBlockPosition, idx, intLayerCount, intTextureCount, intSpriteCount, intTmp As Integer
        'Dim objFile As Grouper.GrouperReader
        'Dim strTmp As String
        'Dim objTexture As Direct3D.Texture
        'Dim objMapSize, objSectorSize As Size



        '' open map file
        'objFile = New Grouper.GrouperReader()
        'objFile.Open(Filename)

        '' read header
        'intDataBlockPosition = 0
        'objFile.BeginDataBlock(intDataBlockPosition)
        'objFile.ReadDataBlock(intLayerCount)
        'objFile.ReadDataBlock(intTextureCount)
        'objFile.ReadDataBlock(objMapSize.Width)
        'objFile.ReadDataBlock(objMapSize.Height)
        'objFile.EndDataBlock()

        '' create map
        'objSectorSize = New Size(64, 64)
        'mobjMap = New Spritz3DR1.Map(intLayerCount, objSectorSize, objMapSize)

        '' cal number of sectors visible at any one time
        'mobjGamePlayArea = General.ToRectangle(objMapSize)
        'CalculateVisibleSectors()

        '' load textures
        'For idx = 0 To intTextureCount - 1

        '    ' get tmp filename
        '    strTmp = IO.Path.GetTempFileName

        '    ' begin reading texture data block and save texture to disk
        '    intDataBlockPosition += 1
        '    objFile.ReadFile(intDataBlockPosition, strTmp)

        '    ' load texture
        '    objTexture = Direct3D.TextureLoader.FromFile(mobjGraphics.Device, strTmp)
        '    ' add texture to list
        '    ReDim Preserve mobjTextures(mobjTextures.Length + 1)
        '    mobjTextures(mobjTextures.Length) = objTexture

        '    ' delete texture file
        '    IO.File.Delete(strTmp)

        'Next


        '' allocate room for sprites
        'ReDim mobjObjects(intSpriteCount)

        '' begin reading sprite data block
        'intDataBlockPosition += 1
        'objFile.BeginDataBlock(intDataBlockPosition)

        '' load sprites
        'For idx = 0 To intSpriteCount - 1
        '    ' create new sprite
        '    mobjObjects(idx) = New Helpers.SpriteObject()

        '    ' Read Texture Index
        '    With mobjObjects(idx)
        '        objFile.ReadDataBlock(.TextureIndex)
        '        objFile.ReadDataBlock(.Scaling.X)
        '        objFile.ReadDataBlock(.Scaling.Y)
        '        objFile.ReadDataBlock(.SourceRectangle.X)
        '        objFile.ReadDataBlock(.SourceRectangle.Y)
        '        objFile.ReadDataBlock(.SourceRectangle.Width)
        '        objFile.ReadDataBlock(.SourceRectangle.Height)
        '        objFile.ReadDataBlock(.Translation.X)
        '        objFile.ReadDataBlock(.Translation.Y)
        '        objFile.ReadDataBlock(intTmp)
        '        .Color = Color.FromArgb(intTmp)
        '        objFile.ReadDataBlock(.Layer)
        '    End With
        'Next

        '' done reading sprites
        'objFile.EndDataBlock()

        '' populate map with sprites
        'For idx = mobjObjects.GetLowerBound(0) To mobjObjects.GetUpperBound(0)
        '    With mobjMap.GetLayer(mobjObjects(idx).Layer)
        '        With .GetSector(New Spritz3DR1.SectorIndex(CInt(mobjObjects(idx).Translation.X) \ objSectorSize.Width, _
        '                                        CInt(mobjObjects(idx).Translation.Y) \ objSectorSize.Height))
        '            .AddSprite(idx)
        '        End With
        '    End With
        'Next


        '' load game file
        'mblnGameLoaded = True
    End Sub

    Public Sub RunGame()
        mblnRunning = True
        While mblnRunning
            DrawGraphics()
            GetInput()
            RaiseEvent Idle()
        End While
        mblnRunning = False
    End Sub

    Public Sub StopGame()
        mblnRunning = False
    End Sub

#Region " Rendering section ..."
    Private Sub DrawGraphics()
        Dim idx, idy As Integer

        mobjGraphics.Clear()
        mobjSprite.Sprite.Begin()

        ' render layers back to front
        For idx = 0 To mobjMap.LayerCount - 1
            RenderLayer(idx)
        Next

        mobjSprite.Sprite.End()
        mobjGraphics.Present()
    End Sub

    Private Sub RenderLayer(ByVal Index As Integer)
        Dim Sector As Spritz3DR1.SectorIndex

        For Sector.Column = mobjVisibleSectors(0).Column To mobjVisibleSectors(1).Column
            For Sector.Row = mobjVisibleSectors(0).Row To mobjVisibleSectors(1).Row
                RenderSector(mobjMap.GetLayer(Index), Sector)
            Next
        Next
    End Sub

    Private Sub RenderSector(ByVal Layer As Spritz3DR1.MapLayer, ByVal Sector As Spritz3DR1.SectorIndex)
        Dim idx As Integer

        With Layer.GetSector(Sector)
            With .GetSprites
                .Reset()
                While .MoveNext
                    DrawSprite(CType(.Current, Interfaces.ISprite))
                End While
            End With
        End With
    End Sub

    Public Sub DrawSprite(ByVal Sprite As Interfaces.ISprite)
        With Sprite
            If Not General.InBounds(Sprite.TextureIndex, 0, mobjTextures.Length) Then
                Throw New ArgumentOutOfRangeException("Sprite.TextureIndex", Sprite.TextureIndex, "Index to texture is invalid.")
            End If
            Try
                mobjSprite.Sprite.Draw(mobjTextures(.TextureIndex), .SourceRectangle, _
                                       .Scaling, .RotationCenter, .Rotation, .Translation, .Color)
            Catch
            End Try
        End With
    End Sub
#End Region

    Private Sub GetInput()

    End Sub

    Private Sub InitInput()
        mobjKB = New DirectInput.Device(DirectInput.SystemGuid.Keyboard)
        mobjKB.SetCooperativeLevel(mobjForm, DirectInput.CooperativeLevelFlags.Exclusive Or _
                                             DirectInput.CooperativeLevelFlags.Foreground)
        mobjKB.Acquire()
    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()
        CleanUp()
    End Sub

    Private Sub CleanUp()
        Try
            mobjKB.Unacquire()
            mobjKB.Dispose()
            mobjKB = Nothing
            mobjSprite = Nothing
        Catch
        End Try
        Array.Clear(mobjTextures, 0, mobjObjects.Length)
        mobjTextures = Nothing
        mobjGraphics = Nothing
        Array.Clear(mobjObjects, 0, mobjObjects.Length)
        mobjObjects = Nothing
    End Sub

    Public ReadOnly Property Sprite() As DX9ToolsR4.Sprites.SpriteBlt
        Get
            Return mobjSprite
        End Get
    End Property

    Public ReadOnly Property Graphics() As DX9ToolsR4.DX9Graphics
        Get
            Return mobjGraphics
        End Get
    End Property

    Public Property GamePlayArea() As Rectangle
        Get
            Return mobjGamePlayArea
        End Get
        Set(ByVal Value As Rectangle)
            mobjGamePlayArea = Value
        End Set
    End Property

    Public ReadOnly Property TheMap() As Map
        Get
            Return mobjMap
        End Get
    End Property

    Public ReadOnly Property GameIsRunning() As Boolean
        Get
            Return mblnRunning
        End Get
    End Property

    Public Property VisibleGamePlayLocation() As Point
        Get
            Return mobjVisibleGamePlayArea.Location
        End Get
        Set(ByVal Value As Point)
            mobjVisibleGamePlayArea.Location = Value
            CalculateVisibleSectors()
        End Set
    End Property

    Public ReadOnly Property VisibleGamePlaySize() As Size
        Get
            Return mobjVisibleGamePlayArea.Size
        End Get
    End Property
End Class