Imports System
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Windows.Forms

Namespace vbAccelerator.Components.Menu


  Public Class IconMenuItem
    Inherits MenuItem

    Private m_tag As String = ""
    Private m_font As Font = Nothing
    Private m_iconIndex As Integer = -1

    Private iconWidth As Integer = 24
    Private iconToTextSeparatorWidth As Integer = 7
    Private textToEdgeSeparatorWidth As Integer = 16


    Public Property Font() As Font
      Get
        Return m_font
      End Get
      Set(ByVal Value As Font)
        m_font = Value
      End Set
    End Property

    Public Property Tag() As String
      Get
        Return m_tag
      End Get
      Set(ByVal Value As String)
        m_tag = Value
      End Set
    End Property

    Public Property IconIndex() As Integer
      Get
        Return m_iconIndex
      End Get
      Set(ByVal Value As Integer)
        m_iconIndex = Value
      End Set
    End Property

    Private Sub DrawCaption(ByVal gfx As Graphics, ByVal text As String, ByVal brush As Brush, _
    ByVal layoutRect As RectangleF, ByVal showAccelerator As Boolean, ByVal shortCut As Boolean, _
    ByVal isEnabled As Boolean)
      Dim sf As System.Drawing.StringFormat = New System.Drawing.StringFormat()
      sf.FormatFlags = StringFormatFlags.NoWrap
      sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Hide
      sf.LineAlignment = StringAlignment.Center
      If (showAccelerator) Then
        sf.HotkeyPrefix = System.Drawing.Text.HotkeyPrefix.Show
      End If
      If (shortCut) Then
        Dim Size As SizeF = gfx.MeasureString(text, m_font)
        layoutRect.X = layoutRect.Width - Size.Width - textToEdgeSeparatorWidth
        sf.Alignment = StringAlignment.Near
      Else
        sf.Alignment = StringAlignment.Near
      End If
      If (isEnabled) Then
        gfx.DrawString(text, m_font, brush, layoutRect, sf)
      Else
        Dim disabledBrush As Brush = SystemBrushes.ControlDark
        gfx.DrawString(text, m_font, disabledBrush, layoutRect, sf)
        'ControlPaint.DrawStringDisabled(gfx, text, m_font,
        '	Color.FromKnownColor(KnownColor.ControlDark), layoutRect, sf)
      End If
    End Sub

    Private Sub DrawIcon(ByVal gfx As Graphics, ByVal rect As Rectangle, _
    ByVal isSelected As Boolean, ByVal isChecked As Boolean, _
    ByVal isRadioChecked As Boolean, ByVal isEnabled As Boolean)

      Dim iconImageList As System.Windows.Forms.ImageList = Nothing

      Dim m As IconMainMenu = CType(Me.GetMainMenu(), IconMainMenu)
      If (Not m Is Nothing) Then
        iconImageList = m.IconImageList
      Else
        Dim c As IconContextMenu = CType(Me.GetContextMenu(), IconContextMenu)
        If (Not c Is Nothing) Then
          iconImageList = c.IconImageList
        End If
      End If

      Dim hasIcon As Boolean = False
      If (Not iconImageList Is Nothing) Then
        If ((m_iconIndex >= 0) And (m_iconIndex < iconImageList.Images.Count)) Then
          hasIcon = True

                    Dim x As Integer = rect.Left + (iconWidth - iconImageList.ImageSize.Width) \ 2
                    Dim y As Integer = rect.Top + (rect.Height - iconImageList.ImageSize.Height) \ 2

          If ((isSelected) And (isEnabled)) Then
            ' draw icon as a shadow here
            ControlPaint.DrawImageDisabled(gfx, _
             iconImageList.Images(m_iconIndex), _
             x, y, Color.FromKnownColor(KnownColor.Control))
            x -= 1
            y -= 1
          End If
          If (isEnabled) Then
            iconImageList.Draw(gfx, x, y, _
        iconImageList.ImageSize.Width, iconImageList.ImageSize.Height, m_iconIndex)
          Else
            ControlPaint.DrawImageDisabled(gfx, _
             iconImageList.Images(m_iconIndex), _
             x, y, Color.FromKnownColor(KnownColor.Control))
          End If
        End If
      End If

      If (isChecked) Then

        Dim checkRect As Rectangle = rect
        checkRect.Width = iconWidth
        checkRect.Inflate(-1, -1)
        checkRect.Height -= 1
        Dim pPen As Pen = SystemPens.Highlight
        gfx.DrawRectangle(pPen, checkRect)
        If (Not hasIcon) Then
                    Dim x As Integer = rect.Left + (iconWidth - SystemInformation.MenuCheckSize.Width) \ 2
                    Dim y As Integer = rect.Top + (rect.Height - SystemInformation.MenuCheckSize.Width) \ 2

          ' Fill the background of the check item with an alpha-blended
          ' highlight colour:
          Dim alpha As Integer = 24
          If (isSelected) Then
            alpha = 128
          End If

          Dim br As Brush = New SolidBrush(Color.FromArgb(alpha, Color.FromKnownColor(KnownColor.Highlight)))
          gfx.FillRectangle(br, checkRect)
          br.Dispose()

          '// create a new bitmap to draw the menu glyph onto:
          Dim bm As Bitmap = New Bitmap(SystemInformation.MenuCheckSize.Width, SystemInformation.MenuCheckSize.Height)
          Dim gr As Graphics = Graphics.FromImage(bm)
          If (isRadioChecked) Then
            ' draw radio option						
            ControlPaint.DrawMenuGlyph(gr, 0, 0, _
             bm.Width, bm.Height, _
             MenuGlyph.Bullet)
          Else
            ' draw check mark
            ControlPaint.DrawMenuGlyph(gr, 0, 0, _
             bm.Width, bm.Height, _
             MenuGlyph.Checkmark)
          End If
          Dim outRect As Rectangle = New Rectangle(x, y, bm.Width, bm.Height)
          '// Draw the menu glyph transparently:
          Dim imgAttr As ImageAttributes = New ImageAttributes()
          imgAttr.SetColorKey(bm.GetPixel(0, 0), bm.GetPixel(0, 0), ColorAdjustType.Default)
          gfx.DrawImage(bm, outRect, 0, 0, bm.Width, bm.Height, GraphicsUnit.Pixel, imgAttr)
          imgAttr.Dispose()
          bm.Dispose()
          gr.Dispose()
        End If
      End If

    End Sub

    Protected Overrides Sub OnDrawItem(ByVal e As System.Windows.Forms.DrawItemEventArgs)
      If (Me.Parent Is Me.GetMainMenu()) Then
        Dim bBrush As Brush
        bBrush = SystemBrushes.Control
        e.Graphics.FillRectangle(bBrush, e.Bounds)
        If ( _
         ((e.State And DrawItemState.Selected) = DrawItemState.Selected) Or _
         ((e.State And DrawItemState.HotLight) = DrawItemState.HotLight) _
         ) Then
          Dim highlightRect As Rectangle = e.Bounds
          highlightRect.X += 1
          highlightRect.Y += 1
          highlightRect.Width -= 3

          Dim pPen As Pen = SystemPens.Highlight
          If ((e.State And DrawItemState.HotLight) = DrawItemState.HotLight) Then
            highlightRect.Height -= 2
            bBrush = New SolidBrush(HighlightBrushColor)
            e.Graphics.FillRectangle(bBrush, highlightRect)
            bBrush.Dispose()

            e.Graphics.DrawRectangle(pPen, highlightRect)
          Else
            highlightRect.X -= 1
            highlightRect.Height -= 1
            bBrush = SystemBrushes.Control
            e.Graphics.FillRectangle(bBrush, highlightRect)

            highlightRect.Height -= 1
            highlightRect.Width -= 1
            pPen = SystemPens.ControlDark
            e.Graphics.DrawLine(pPen, highlightRect.X, highlightRect.Y + highlightRect.Height, highlightRect.X, highlightRect.Y)
            e.Graphics.DrawLine(pPen, highlightRect.X, highlightRect.Y, highlightRect.X + highlightRect.Width, highlightRect.Y)
            e.Graphics.DrawLine(pPen, highlightRect.X + highlightRect.Width, highlightRect.Y, highlightRect.X + highlightRect.Width, highlightRect.Y + highlightRect.Height)

            '//pen = SystemPens.ControlDark
            pPen = New Pen(Color.FromArgb(128, Color.Black))
            e.Graphics.DrawLine(pPen, _
             highlightRect.X + highlightRect.Width + 1, _
             highlightRect.Y + 4, _
             highlightRect.X + highlightRect.Width + 1, _
             highlightRect.Y + highlightRect.Height)
            pPen.Dispose()
            '//pen = SystemPens.ControlDarkDark
            pPen = New Pen(Color.FromArgb(64, Color.Black))
            e.Graphics.DrawLine(pPen, _
             highlightRect.X + highlightRect.Width + 2, _
             highlightRect.Y + 4, _
             highlightRect.X + highlightRect.Width + 2, _
             highlightRect.Y + highlightRect.Height)
            pPen.Dispose()
            pPen = New Pen(Color.FromArgb(32, Color.Black))
            e.Graphics.DrawLine(pPen, _
             highlightRect.X + highlightRect.Width + 3, _
             highlightRect.Y + 5, _
             highlightRect.X + highlightRect.Width + 3, _
             highlightRect.Y + highlightRect.Height)
            pPen.Dispose()
          End If

        End If
        bBrush = SystemBrushes.WindowText
        Dim layoutRect As RectangleF = New RectangleF(e.Bounds.X + 1, e.Bounds.Y + 1, e.Bounds.Width - 2, e.Bounds.Height)
        layoutRect.X += 4
        DrawCaption(e.Graphics, Me.Text, bBrush, layoutRect, _
          ((e.State And DrawItemState.NoAccelerator) <> DrawItemState.NoAccelerator), False, Me.Enabled)
      Else
        Dim iconBarBrush As Brush = SystemBrushes.Control
        e.Graphics.FillRectangle(iconBarBrush, e.Bounds.X, e.Bounds.Y, _
          iconWidth, e.Bounds.Height)

        Dim rectOut As Rectangle = e.Bounds

        If (Me.Text.Equals("-")) Then
          rectOut.X += iconWidth
          Dim bBrush As Brush
          'Original Code
          'If (System.Environment.OSVersion.Version.Major > 4) Then
          '  bBrush = SystemBrushes.Window
          'Else
          '  bBrush = SystemBrushes.Menu
          'End If
          ''Modified by Camalot Designs 04/11/03

          Dim m As IconMainMenu = CType(Me.GetMainMenu(), IconMainMenu)
          If (Not m Is Nothing) Then
            If (m.MenuStyle = m.msMenuStyle.Windows_Clasic) Then
              bBrush = SystemBrushes.Menu
            Else
              bBrush = SystemBrushes.Window
            End If
          Else
            Dim c As IconContextMenu = CType(Me.GetContextMenu(), IconContextMenu)
            If (Not c Is Nothing) Then
              If (c.MenuStyle = c.msMenuStyle.Windows_Clasic) Then
                bBrush = SystemBrushes.Menu
              Else
                bBrush = SystemBrushes.Window
              End If
            End If
          End If

          e.Graphics.FillRectangle(bBrush, rectOut)

          Dim pPen As Pen = SystemPens.ControlDark
          e.Graphics.DrawLine(ppen, _
           iconWidth + iconToTextSeparatorWidth, e.Bounds.Y + 1, _
           e.Bounds.Width, e.Bounds.Y + 1)
        Else
            Dim rectIn As Rectangle = rectOut
            Dim bBrush As Brush
            rectIn.Height -= 1
            If ( _
                 ((e.State And DrawItemState.Selected) = DrawItemState.Selected) And _
                 (Me.Enabled)) Then
              'Original Code
              'If (System.Environment.OSVersion.Version.Major > 4) Then
              '  bBrush = SystemBrushes.Window
              'Else
              '  bBrush = SystemBrushes.Menu
              'End If
              ''Modified by Camalot Designs 04/11/03
            Dim m As IconMainMenu = CType(Me.GetMainMenu(), IconMainMenu)
            If (Not m Is Nothing) Then
              If (m.MenuStyle = m.msMenuStyle.Windows_Clasic) Then
                bBrush = SystemBrushes.Menu
              Else
                bBrush = SystemBrushes.Window
              End If
            Else
              Dim c As IconContextMenu = CType(Me.GetContextMenu(), IconContextMenu)
              If (Not c Is Nothing) Then
                If (c.MenuStyle = c.msMenuStyle.Windows_Clasic) Then
                  bBrush = SystemBrushes.Menu
                Else
                  bBrush = SystemBrushes.Window
                End If
              End If
            End If
            e.Graphics.FillRectangle(bBrush, rectIn)
            bBrush = New SolidBrush(Color.FromArgb(64, Color.FromKnownColor(KnownColor.Highlight)))
            e.Graphics.FillRectangle(bBrush, rectIn)
            bBrush.Dispose()

            Dim pPen As Pen = SystemPens.Highlight
            Dim highlightRect As Rectangle = rectIn
            highlightRect.Width -= 1
            e.Graphics.DrawRectangle(pPen, highlightRect)
          Else
              rectOut.X += iconWidth
              rectOut.Width -= iconWidth
              'Original Code
              'If (System.Environment.OSVersion.Version.Major > 4) Then
              '  bBrush = SystemBrushes.Window
              'Else
              '  bBrush = SystemBrushes.menu
              'End If
              ''Modified by Camalot Designs 04/11/03
            Dim m As IconMainMenu = CType(Me.GetMainMenu(), IconMainMenu)
            If (Not m Is Nothing) Then
              If (m.MenuStyle = m.msMenuStyle.Windows_Clasic) Then
                bBrush = SystemBrushes.Menu
              Else
                bBrush = SystemBrushes.Window
              End If
            Else
              Dim c As IconContextMenu = CType(Me.GetContextMenu(), IconContextMenu)
              If (Not c Is Nothing) Then
                If (c.MenuStyle = c.msMenuStyle.Windows_Clasic) Then
                  bBrush = SystemBrushes.Menu
                Else
                  bBrush = SystemBrushes.Window
                End If
              End If
            End If
            e.Graphics.FillRectangle(bBrush, rectOut)
            End If

            DrawIcon( _
             e.Graphics, e.Bounds, _
             ((e.State And DrawItemState.Selected) = DrawItemState.Selected), _
             Me.Checked, Me.RadioCheck, Me.Enabled)

            bBrush = SystemBrushes.WindowText
            Dim layoutRect As RectangleF = New RectangleF(e.Bounds.X + iconWidth + iconToTextSeparatorWidth, e.Bounds.Y, e.Bounds.Width, e.Bounds.Height)
            DrawCaption(e.Graphics, Me.Text, bBrush, layoutRect, _
             ((e.State And DrawItemState.NoAccelerator) <> DrawItemState.NoAccelerator), _
             False, Me.Enabled)

            If (Me.ShowShortcut) Then
              If (Me.Shortcut <> shortCut.None) Then
                Dim shortCut As String = ShortcutToString(Me.Shortcut)
                DrawCaption(e.Graphics, shortCut, bBrush, layoutRect, False, True, Me.Enabled)
              End If
            End If
          End If
        End If
    End Sub

    Protected Overrides Sub OnMeasureItem(ByVal e As System.Windows.Forms.MeasureItemEventArgs)

      If (Me.Parent Is Me.GetMainMenu()) Then
        Dim fontSize As SizeF = e.Graphics.MeasureString(Me.Text.Replace("&", ""), Me.Font)
        e.ItemWidth = CType(fontSize.Width + 3, Integer)
        e.ItemHeight = CType(fontSize.Height, Integer)
      Else
        Dim sSize As Size = New Size(iconWidth + iconToTextSeparatorWidth + textToEdgeSeparatorWidth, 3)
        If (Not Me.Text.Equals("-")) Then
          Dim toMeasure As String = Me.Text
          If (Me.ShowShortcut) Then
            If (Me.Shortcut <> Shortcut.None) Then
              toMeasure += " " & ShortcutToString(Me.Shortcut)
            End If
          End If
          Dim fontSize As SizeF = e.Graphics.MeasureString(toMeasure, Me.Font)
          sSize.Width += CType(fontSize.Width, Integer)
          sSize.Height = Math.Max(iconWidth, CType(fontSize.Height, Integer))
        End If
        e.ItemHeight = sSize.Height
        e.ItemWidth = sSize.Width
      End If
    End Sub

    Private Function HighlightBrushColor() As Color
      ''//return Color.FromArgb(192, Color.FromKnownColor(KnownColor.Highlight))
      Dim cH As Color = Color.FromKnownColor(KnownColor.Highlight)
      Dim cM As Color

      'Original Code
      'If (System.Environment.OSVersion.Version.Major > 4) Then
      '  cM = Color.FromKnownColor(KnownColor.Window)
      'Else
      '  cM = Color.FromKnownColor(KnownColor.Menu)
      'End If

      ''Modified by Camalot Designs 04/11/03
      Dim m As IconMainMenu = CType(Me.GetMainMenu(), IconMainMenu)
      If (Not m Is Nothing) Then
        If (m.MenuStyle = m.msMenuStyle.Windows_Clasic) Then
          cM = Color.FromKnownColor(KnownColor.Menu)
        Else
          cM = Color.FromKnownColor(KnownColor.Window)
        End If
      Else
        Dim c As IconContextMenu = CType(Me.GetContextMenu(), IconContextMenu)
        If (Not c Is Nothing) Then
          If (c.MenuStyle = c.msMenuStyle.Windows_Clasic) Then
            cM = Color.FromKnownColor(KnownColor.Menu)
          Else
            cM = Color.FromKnownColor(KnownColor.Window)
          End If
        End If
      End If

            Return Color.FromArgb( _
             Math.Max(0, Math.Min(255, cH.R + (cM.R - cH.R) \ 2)), _
             Math.Max(0, Math.Min(255, cH.G + (cM.G - cH.G) \ 2)), _
             Math.Max(0, Math.Min(255, cH.B + (cM.B - cH.B) \ 2)))
    End Function

    Private Function ShortcutToString(ByVal s As Shortcut) As String
      Dim ret As String = s.ToString()
      ret = ret.Replace("Ctrl", "Ctrl+")
      ret = ret.Replace("Alt", "Alt+")
      ret = ret.Replace("Shift", "Shift+")
      Return ret
    End Function

    Public Sub New()
      MyBase.New()
      Me.OwnerDraw = True
      m_font = SystemInformation.MenuFont
    End Sub
  End Class
  Public Class IconContextMenu
    Inherits ContextMenu
    Private imgList As System.Windows.Forms.ImageList

    Public Enum msMenuStyle
      Windows_XP
      Windows_Clasic
    End Enum

    Private MnuStyle As msMenuStyle = MenuStyle.Windows_XP
    Public Property MenuStyle() As msMenuStyle
      Get
        Return MnuStyle
      End Get
      Set(ByVal Value As msMenuStyle)
        MnuStyle = Value
      End Set
    End Property
    Public Property IconImageList() As System.Windows.Forms.ImageList
      Get
        Return imgList
      End Get
      Set(ByVal Value As System.Windows.Forms.ImageList)
        imgList = Value
      End Set
    End Property

    Public Sub New()
      MyBase.new()
    End Sub

    Public Sub New(ByVal items As IconMenuItem())
      MyBase.new(items)
    End Sub
  End Class

  Public Class IconMainMenu
    Inherits MainMenu

    Private imgList As System.Windows.Forms.ImageList
    Public Enum msMenuStyle
      Windows_XP
      Windows_Clasic
    End Enum

    Private MnuStyle As msMenuStyle = MenuStyle.Windows_XP
    Public Property MenuStyle() As msMenuStyle
      Get
        Return MnuStyle
      End Get
      Set(ByVal Value As msMenuStyle)
        MnuStyle = Value
      End Set
    End Property

    Public Property IconImageList() As System.Windows.Forms.ImageList
      Get
        Return imgList
      End Get
      Set(ByVal Value As System.Windows.Forms.ImageList)
        imgList = Value
      End Set
    End Property

    Public Sub New()
      MyBase.new()
    End Sub

    Public Sub New(ByVal items As IconMenuItem())
      MyBase.new(items)
    End Sub
  End Class
end namespace
