Public Class RulerControl
    Inherits System.Windows.Forms.UserControl

    Private mobjCellSize As Size = New Size(8, 21)
    Private mblnShowTicks As Boolean = True
    Private mblnShowNumbers As Boolean = True
    Private mobjOffset As Point = New Point(0, 0)
    Private mintNumberIndex As Integer = 5
    Private mintBaseNumber As Integer = 0

    Public Const DefaultNumberIndex As Integer = 5
    Public Const DefaultCellW As Integer = 8
    Public Const DefaultCellH As Integer = 21

    Public Event BeforeCellSizeChange(ByVal OldValue As Size, ByRef NewValue As Size, ByRef Cancel As Boolean)
    Public Event BeforeShowTicksChange(ByVal OldValue As Boolean, ByRef NewValue As Boolean, ByRef Cancel As Boolean)
    Public Event BeforeOffsetChange(ByVal OldValue As Point, ByRef NewValue As Point, ByRef Cancel As Boolean)
    Public Event BeforeShowNumbersChange(ByVal OldValue As Boolean, ByRef NewValue As Boolean, ByRef Cancel As Boolean)
    Public Event BeforeNumberIndexChange(ByVal OldValue As Integer, ByRef NewValue As Integer, ByRef Cancel As Boolean)
    Public Event BeforeBaseNumberChange(ByVal OldValue As Integer, ByRef NewValue As Integer, ByRef Cancel As Boolean)


#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'UserControl overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        '
        'CellControl
        '
        Me.Name = "CellControl"
        Me.Size = New System.Drawing.Size(280, 24)

    End Sub

#End Region


    Private Sub RulerControl_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        Dim idx, idy, TopPosition As Integer
        Dim objBackBrush, objTextBrush As Brush
        Dim objGridColor As Pen

        Try
            objBackBrush = New SolidBrush(Me.BackColor)
            objGridColor = New Pen(Me.ForeColor)

            e.Graphics.Clear(Me.BackColor)

            e.Graphics.DrawRectangle(objGridColor, 0, 0, Me.Width - 1, Me.Height - 1)

            ' draw ticks
            If mblnShowTicks Then
                ' draw vert lines
                For idx = mobjOffset.X To Me.Width Step mobjCellSize.Width
                    e.Graphics.DrawLine(objGridColor, idx, 1, idx, Me.Height - 2)
                Next
            End If

            ' remove center of ticks(lines)
            e.Graphics.FillRectangle(objBackBrush, 1, 5, Me.Width - 2, Me.Height - 11)

            ' draw numbers
            If mblnShowNumbers Then
                objTextBrush = New SolidBrush(Me.ForeColor)
                idx = CInt(e.Graphics.MeasureString("0", Me.Font).Height) \ 2
                TopPosition = CInt((Me.Height \ 2) - idx)

                idy = mintBaseNumber
                For idx = mobjOffset.X To Me.Width Step mintNumberIndex * mobjCellSize.Width
                    e.Graphics.DrawString(CStr((idy \ mobjCellSize.Width) + mintBaseNumber), _
                                          Me.Font, objTextBrush, idx - 1, TopPosition)
                    idy += mintNumberIndex * mobjCellSize.Width
                Next
            End If

        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try
    End Sub

    Public Property CellSize() As Size
        Get
            Return mobjCellSize
        End Get
        Set(ByVal Value As Size)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeCellSizeChange(mobjCellSize, Value, blnCancel)
            If blnCancel Then Exit Property

            mobjCellSize = Value
            RulerControl_Resize(Nothing, Nothing)
            Me.Refresh()
        End Set
    End Property

    Public Property ShowTicks() As Boolean
        Get
            Return mblnShowTicks
        End Get
        Set(ByVal Value As Boolean)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeShowTicksChange(mblnShowTicks, Value, blnCancel)
            If blnCancel Then Exit Property
            mblnShowTicks = Value
            Me.Refresh()

        End Set
    End Property

    Public Property Offset() As Point
        Get
            Return mobjOffset
        End Get
        Set(ByVal Value As Point)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeOffsetChange(mobjOffset, Value, blnCancel)
            If blnCancel Then Exit Property

            mobjOffset = Value
            Me.Refresh()
        End Set
    End Property

    Public Property NumberIndex() As Integer
        Get
            Return mintNumberIndex
        End Get
        Set(ByVal Value As Integer)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeBaseNumberChange(mintNumberIndex, Value, blnCancel)
            If blnCancel Then Exit Property
            mintNumberIndex = CInt(IIf(Value < 1, 1, Value))
            Me.Refresh()
        End Set
    End Property

    Private Sub RulerControl_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        Me.Height = mobjCellSize.Height
        Me.Refresh()
    End Sub

    Public Property ShowNumbers() As Boolean
        Get
            Return mblnShowNumbers
        End Get
        Set(ByVal Value As Boolean)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeShowNumbersChange(mblnShowNumbers, Value, blnCancel)
            If blnCancel Then Exit Property

            mblnShowNumbers = Value
            Me.Refresh()
        End Set
    End Property

    Public Property BaseNumber() As Integer
        Get
            Return mintBaseNumber
        End Get
        Set(ByVal Value As Integer)
            Dim blnCancel As Boolean = False

            RaiseEvent BeforeBaseNumberChange(mintBaseNumber, Value, blnCancel)
            If blnCancel Then Exit Property

            mintBaseNumber = Value
            Me.Refresh()
        End Set
    End Property
End Class
