'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Saturday, December 21, 2002
'=============================================
Namespace Commands

#Region " Intefaces ... "
    Public Interface ICommand
        Property Tag() As Object
    End Interface

    Public Interface IMouseDrag
        Inherits ICommand
        Enum MouseEventType
            Down = 0
            Drag = 1
            Release = 2
        End Enum

        Delegate Sub IMouseDragCallback(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs, ByVal ev As MouseEventType)

        Property ReferenceControl() As Control
        Property MouseButton() As MouseButtons
        Property Dragging() As IMouseDragCallback
    End Interface
#End Region

    Namespace GeneralCommands
        Public Class MouseDragCommand
            Implements IMouseDrag

            Private WithEvents mobjCtrl As Control
            Private menuButton As MouseButtons = MouseButtons.Left
            Private mobjDrag As IMouseDrag.IMouseDragCallback
            Private mobjTag As Object

            Public Property MouseButton() As System.Windows.Forms.MouseButtons Implements IMouseDrag.MouseButton
                Get
                    Return menuButton
                End Get
                Set(ByVal Value As System.Windows.Forms.MouseButtons)
                    menuButton = Value
                End Set
            End Property

            Public Property ReferenceControl() As System.Windows.Forms.Control Implements IMouseDrag.ReferenceControl
                Get
                    Return mobjCtrl
                End Get
                Set(ByVal Value As System.Windows.Forms.Control)
                    mobjCtrl = Value
                End Set
            End Property

            Private Sub mobjCtrl_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles mobjCtrl.MouseDown
                Try
                    If e.Button = menuButton Then mobjDrag.Invoke(Me, e, IMouseDrag.MouseEventType.Down)
                Catch
                    'ERR: do nothing
                End Try
            End Sub

            Private Sub mobjCtrl_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles mobjCtrl.MouseMove
                Try
                    If e.Button = menuButton Then mobjDrag.Invoke(Me, e, IMouseDrag.MouseEventType.Drag)
                Catch
                    'ERR: do nothing
                End Try
            End Sub

            Private Sub mobjCtrl_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles mobjCtrl.MouseUp
                Try
                    If e.Button = menuButton Then mobjDrag.Invoke(Me, e, IMouseDrag.MouseEventType.Release)
                Catch
                    'ERR: do nothing
                End Try
            End Sub

            Public Property Dragging() As IMouseDrag.IMouseDragCallback Implements IMouseDrag.Dragging
                Get
                    Return mobjDrag
                End Get
                Set(ByVal Value As IMouseDrag.IMouseDragCallback)
                    mobjDrag = Value
                End Set
            End Property

            Protected Overrides Sub Finalize()
                MyBase.Finalize()
                mobjDrag = Nothing
                mobjCtrl = Nothing
            End Sub

            Public Sub New(Optional ByVal Ctrl As Control = Nothing, _
                           Optional ByVal Button As MouseButtons = MouseButtons.Left, _
                           Optional ByVal DragCallback As IMouseDrag.IMouseDragCallback = Nothing)
                MyBase.New()
                mobjCtrl = Ctrl
                mobjDrag = DragCallback
                menuButton = Button
            End Sub

            Public Property Tag() As Object Implements IMouseDrag.Tag
                Get
                    Return mobjTag
                End Get
                Set(ByVal Value As Object)
                    mobjTag = Value
                End Set
            End Property
        End Class
    End Namespace

    Public Class CommandStack
        Private mobjCommandStack As Collections.ArrayList


        Public Function Pop() As ICommand
            Dim objTmp As ICommand

            objTmp = CType(mobjCommandStack.Item(mobjCommandStack.Count), ICommand)
            mobjCommandStack.Remove(objTmp)
            Return objTmp
        End Function

        Public Function Peek() As ICommand
            Return CType(mobjCommandStack.Item(mobjCommandStack.Count), ICommand)
        End Function

        Public Sub Push(ByVal Command As ICommand)
            mobjCommandStack.Add(Command)
        End Sub

        Public Sub New()
            MyBase.new()
            mobjCommandStack = New Collections.ArrayList()
        End Sub

        Public Sub Clear()
            mobjCommandStack.Clear()
        End Sub

        Public ReadOnly Property Count() As Integer
            Get
                Return mobjCommandStack.Count
            End Get
        End Property

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            mobjCommandStack.Clear()
            mobjCommandStack = Nothing
        End Sub
    End Class
End Namespace