'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Saturday, December 21, 2002
'=============================================

Namespace Properties
    Public Enum PropertyType
        [Custom] = 0
        [Long] = 1
        [Integer] = 2
        [Byte] = 3
        [Boolean] = 4
        [Single] = 5
        [Short] = 6
        [Double] = 7
        [Date] = 8
        [Char] = 9
        [Decimal] = 10
        [String] = 11
        [Object] = 12
        [Point] = 13
        [PointF] = 14
        [Size] = 15
        [SizeF] = 16
        [Rectangle] = 17
        [RectangleF] = 18
        [Color] = 19
        [Brush] = 20
        [Pen] = 21
    End Enum

    Public Interface IProperty
        Property Name() As String
        Property Parent() As PropertiesCollection
        ReadOnly Property Index() As Integer
        Property Children() As PropertiesCollection
        ReadOnly Property Type() As PropertyType
    End Interface

#Region " Property interface types "
    Public Interface IPropertyLong
        Inherits IProperty
        Property Value() As Long
    End Interface

    Public Interface IPropertyInteger
        Inherits IProperty
        Property Value() As Integer
    End Interface

    Public Interface IPropertyByte
        Inherits IProperty
        Property Value() As Byte
    End Interface

    Public Interface IPropertyBoolean
        Inherits IProperty
        Property Value() As Boolean
    End Interface

    Public Interface IPropertySingle
        Inherits IProperty
        Property Value() As Single
    End Interface

    Public Interface IPropertyShort
        Inherits IProperty
        Property Value() As Short
    End Interface

    Public Interface IPropertyDouble
        Inherits IProperty
        Property Value() As Double
    End Interface

    Public Interface IPropertyDate
        Inherits IProperty
        Property Value() As Date
    End Interface

    Public Interface IPropertyChar
        Inherits IProperty
        Property Value() As Char
    End Interface

    Public Interface IPropertyDecimal
        Inherits IProperty
        Property Value() As Decimal
    End Interface

    Public Interface IPropertyString
        Inherits IProperty
        Property Value() As String
    End Interface

    Public Interface IPropertyObject
        Inherits IProperty
        Property Value() As Object
    End Interface

    Public Interface IPropertyPoint
        Inherits IProperty
        Property Value() As Point
    End Interface

    Public Interface IPropertyPointF
        Inherits IProperty
        Property Value() As PointF
    End Interface

    Public Interface IPropertySize
        Inherits IProperty
        Property Value() As Size
    End Interface

    Public Interface IPropertySizeF
        Inherits IProperty
        Property Value() As SizeF
    End Interface

    Public Interface IPropertyRectangle
        Inherits IProperty
        Property Value() As Rectangle
    End Interface

    Public Interface IPropertyRectangleF
        Inherits IProperty
        Property Value() As RectangleF
    End Interface

    Public Interface IPropertyColor
        Inherits IProperty
        Property Value() As Color
    End Interface

    Public Interface IPropertyBrush
        Inherits IProperty
        Property Value() As Brush
    End Interface

    Public Interface IPropertyPen
        Inherits IProperty
        Property Value() As Pen
    End Interface
#End Region

#Region " Property Objects ... "
    Public MustInherit Class BaseProperty
        Implements IProperty
        Private mobjChildren, mobjParent As PropertiesCollection
        Private mstrName As String

        Public Property Children() As CBX.Libs.CommonCommandsR3.Properties.PropertiesCollection Implements CBX.Libs.CommonCommandsR3.Properties.IProperty.Children
            Get
                Return mobjChildren
            End Get
            Set(ByVal Value As CBX.Libs.CommonCommandsR3.Properties.PropertiesCollection)
                mobjChildren = Value
            End Set
        End Property

        Public ReadOnly Property Index() As Integer Implements CBX.Libs.CommonCommandsR3.Properties.IProperty.Index
            Get
                Return mobjParent.IndexOf(Me)
            End Get
        End Property

        Public Property Name() As String Implements CBX.Libs.CommonCommandsR3.Properties.IProperty.Name
            Get
                Return mstrName
            End Get
            Set(ByVal Value As String)
                mstrName = Value
            End Set
        End Property

        Public Property Parent() As CBX.Libs.CommonCommandsR3.Properties.PropertiesCollection Implements CBX.Libs.CommonCommandsR3.Properties.IProperty.Parent
            Get
                Return mobjParent
            End Get
            Set(ByVal Value As CBX.Libs.CommonCommandsR3.Properties.PropertiesCollection)
                mobjParent = Value
            End Set
        End Property

        Public MustOverride ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType Implements CBX.Libs.CommonCommandsR3.Properties.IProperty.Type

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Name As String)
            mstrName = Name
        End Sub
    End Class

    Public Class PropertyLong
        Inherits BaseProperty
        Implements IPropertyLong
        Private mlngValue As Long

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Long
            End Get
        End Property

        Public Property Value() As Long Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyLong.Value
            Get
                Return mlngValue
            End Get
            Set(ByVal Value As Long)
                mlngValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Long)
            mlngValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Long)
            MyBase.new(Name)
            mlngValue = Value
        End Sub
    End Class

    Public Class PropertyInteger
        Inherits BaseProperty
        Implements IPropertyInteger
        Private mintValue As Integer

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Integer
            End Get
        End Property

        Public Property Value() As Integer Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyInteger.Value
            Get
                Return mintValue
            End Get
            Set(ByVal Value As Integer)
                mintValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Integer)
            mintValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Integer)
            MyBase.new(Name)
            mintValue = Value
        End Sub
    End Class

    Public Class PropertyByte
        Inherits BaseProperty
        Implements IPropertyByte
        Private mbytValue As Byte


        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Byte
            End Get
        End Property

        Public Property Value() As Byte Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyByte.Value
            Get
                Return mbytValue
            End Get
            Set(ByVal Value As Byte)
                mbytValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Byte)
            mbytValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Byte)
            MyBase.new(Name)
            mbytValue = Value
        End Sub
    End Class

    Public Class PropertyBoolean
        Inherits BaseProperty
        Implements IPropertyBoolean
        Private mblnValue As Boolean

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Boolean
            End Get
        End Property

        Public Property Value() As Boolean Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyBoolean.Value
            Get
                Return mblnValue
            End Get
            Set(ByVal Value As Boolean)
                mblnValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Boolean)
            mblnValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Boolean)
            MyBase.new(Name)
            mblnValue = Value
        End Sub
    End Class

    Public Class PropertySingle
        Inherits BaseProperty
        Implements IPropertySingle
        Private msngValue As Single

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Single
            End Get
        End Property

        Public Property Value() As Single Implements CBX.Libs.CommonCommandsR3.Properties.IPropertySingle.Value
            Get
                Return msngValue
            End Get
            Set(ByVal Value As Single)
                msngValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Single)
            msngValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Single)
            MyBase.new(Name)
            msngValue = Value
        End Sub
    End Class

    Public Class PropertyShort
        Inherits BaseProperty
        Implements IPropertyShort
        Private mshtValue As Short

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Short
            End Get
        End Property

        Public Property Value() As Short Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyShort.Value
            Get
                Return mshtValue
            End Get
            Set(ByVal Value As Short)
                mshtValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Short)
            mshtValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Short)
            MyBase.new(Name)
            mshtValue = Value
        End Sub
    End Class

    Public Class PropertyDouble
        Inherits BaseProperty
        Implements IPropertyDouble
        Private mdblValue As Double

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Double
            End Get
        End Property

        Public Property Value() As Double Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyDouble.Value
            Get
                Return mdblValue
            End Get
            Set(ByVal Value As Double)
                mdblValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Double)
            mdblValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Double)
            MyBase.new(Name)
            mdblValue = Value
        End Sub
    End Class

    Public Class PropertyDate
        Inherits BaseProperty
        Implements IPropertyDate
        Private mdteValue As Date

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Date
            End Get
        End Property

        Public Property Value() As Date Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyDate.Value
            Get
                Return mdteValue
            End Get
            Set(ByVal Value As Date)
                mdteValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Date)
            mdteValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Date)
            MyBase.new(Name)
            mdteValue = Value
        End Sub
    End Class

    Public Class PropertyChar
        Inherits BaseProperty
        Implements IPropertyChar
        Private mchrValue As Char

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Char
            End Get
        End Property

        Public Property Value() As Char Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyChar.Value
            Get
                Return mchrValue
            End Get
            Set(ByVal Value As Char)
                mchrValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Char)
            mchrValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Char)
            MyBase.new(Name)
            mchrValue = Value
        End Sub
    End Class

    Public Class PropertyDecimal
        Inherits BaseProperty
        Implements IPropertyDecimal
        Private mdecValue As Decimal

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Decimal
            End Get
        End Property

        Public Property Value() As Decimal Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyDecimal.Value
            Get
                Return mdecValue
            End Get
            Set(ByVal Value As Decimal)
                mdecValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Decimal)
            mdecValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Decimal)
            MyBase.new(Name)
            mdecValue = Value
        End Sub
    End Class

    Public Class PropertyString
        Inherits BaseProperty
        Implements IPropertyString
        Private mstrValue As String

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.String
            End Get
        End Property

        Public Property Value() As String Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyString.Value
            Get
                Return mstrValue
            End Get
            Set(ByVal Value As String)
                mstrValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As String)
            mstrValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As String)
            MyBase.new(Name)
            mstrValue = Value
        End Sub
    End Class

    Public Class PropertyObject
        Inherits BaseProperty
        Implements IPropertyObject
        Private mobjValue As Object

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Object
            End Get
        End Property

        Public Property Value() As Object Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyObject.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Object)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Object)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Object)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyPoint
        Inherits BaseProperty
        Implements IPropertyPoint
        Private mobjValue As Point

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Point
            End Get
        End Property

        Public Property Value() As Point Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyPoint.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Point)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Point)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Point)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyPointF
        Inherits BaseProperty
        Implements IPropertyPointF
        Private mobjValue As PointF

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.PointF
            End Get
        End Property

        Public Property Value() As PointF Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyPointF.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As PointF)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As PointF)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As PointF)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertySize
        Inherits BaseProperty
        Implements IPropertySize
        Private mobjValue As Size

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Size
            End Get
        End Property

        Public Property Value() As Size Implements CBX.Libs.CommonCommandsR3.Properties.IPropertySize.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Size)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Size)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Size)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertySizeF
        Inherits BaseProperty
        Implements IPropertySizeF
        Private mobjValue As SizeF

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.SizeF
            End Get
        End Property

        Public Property Value() As SizeF Implements CBX.Libs.CommonCommandsR3.Properties.IPropertySizeF.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As SizeF)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As SizeF)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As SizeF)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyRectangle
        Inherits BaseProperty
        Implements IPropertyRectangle
        Private mobjValue As Rectangle

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Rectangle
            End Get
        End Property

        Public Property Value() As Rectangle Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyRectangle.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Rectangle)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Rectangle)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Rectangle)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyRectangleF
        Inherits BaseProperty
        Implements IPropertyRectangleF
        Private mobjValue As RectangleF

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.RectangleF
            End Get
        End Property

        Public Property Value() As RectangleF Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyRectangleF.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As RectangleF)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As RectangleF)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As RectangleF)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyColor
        Inherits BaseProperty
        Implements IPropertyColor
        Private mobjValue As Color

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Color
            End Get
        End Property

        Public Property Value() As Color Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyColor.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Color)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Color)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Color)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyBrush
        Inherits BaseProperty
        Implements IPropertyBrush
        Private mobjValue As Brush

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Brush
            End Get
        End Property

        Public Property Value() As Brush Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyBrush.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Brush)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Brush)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Brush)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

    Public Class PropertyPen
        Inherits BaseProperty
        Implements IPropertyPen
        Private mobjValue As Pen

        Public Overrides ReadOnly Property Type() As CBX.Libs.CommonCommandsR3.Properties.PropertyType
            Get
                Return PropertyType.Pen
            End Get
        End Property

        Public Property Value() As Pen Implements CBX.Libs.CommonCommandsR3.Properties.IPropertyPen.Value
            Get
                Return mobjValue
            End Get
            Set(ByVal Value As Pen)
                mobjValue = Value
            End Set
        End Property

        Public Sub New()
            MyBase.new()
        End Sub

        Public Sub New(ByVal Value As Pen)
            mobjValue = Value
        End Sub

        Public Sub New(ByVal Name As String, ByVal Value As Pen)
            MyBase.new(Name)
            mobjValue = Value
        End Sub
    End Class

#End Region

    Public Class PropertiesCollection
        Implements IEnumerable, IDisposable

        Private mobjItems As System.Collections.ArrayList

        Public Sub New()
            mobjItems = New System.Collections.ArrayList()
        End Sub

        Public Sub SwapItem(ByVal Index1 As Integer, ByVal Index2 As Integer)
            Dim intFirst, intLast As Integer
            Dim objFirst, objLast As Object

            ' do err chech
            If Index1 < 0 Or Index1 > mobjItems.Count - 1 Then
                Throw New ArgumentOutOfRangeException("Index1")
            End If
            If Index2 < 0 Or Index2 > mobjItems.Count - 1 Then
                Throw New ArgumentOutOfRangeException("Index2")
            End If
            If Index1 = Index2 Then Exit Sub

            intFirst = CType(IIf(Index1 < Index2, Index1, Index2), Integer)
            intLast = CType(IIf(Index1 > Index2, Index1, Index2), Integer)

            ' store items
            objFirst = mobjItems(intFirst)
            objLast = mobjItems(intLast)

            ' remove from list
            mobjItems.RemoveAt(intLast)
            mobjItems.RemoveAt(intFirst)
            ' reinsert
            mobjItems.Insert(intFirst, objFirst)
            mobjItems.Insert(intLast, objLast)
        End Sub

        Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
            Return New PropertiesCollectionEnumerator(Me)
        End Function

        Public Sub Dispose() Implements System.IDisposable.Dispose
            GC.SuppressFinalize(Me)
        End Sub

        Public Function Add(ByVal Item As IProperty) As Integer
            Return mobjItems.Add(Item)
        End Function

        Public Sub Clear()
            mobjItems.Clear()
        End Sub

        Public ReadOnly Property Count() As Integer
            Get
                Return mobjItems.Count
            End Get
        End Property

        Public Function IndexOf(ByVal Item As IProperty) As Integer
            Return mobjItems.IndexOf(Item)
        End Function

        Public Sub Insert(ByVal Index As Integer, ByVal Item As IProperty)
            mobjItems.Insert(Index, Item)
        End Sub

        Default Public Property Item(ByVal Index As Integer) As IProperty
            Get
                Return CType(mobjItems.Item(Index), IProperty)
            End Get
            Set(ByVal Value As IProperty)
                mobjItems(Index) = Value
            End Set
        End Property

        Public Sub Remove(ByVal Item As IProperty)
            mobjItems.Remove(Item)
        End Sub

        Public Sub Remove(ByVal Index As Integer)
            mobjItems.RemoveAt(Index)
        End Sub

        Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
            mobjItems.RemoveRange(Index, Count)
        End Sub

        Class PropertiesCollectionEnumerator
            Implements IEnumerator

            Private mobjParent As PropertiesCollection
            Private CurrentEntry As IProperty
            Private mintCurrentIndex As Integer = 0

            ' Create a reference back to the object instance that 
            ' owns this Enumberator.
            Public Sub New(ByVal xParent As PropertiesCollection)
                mobjParent = xParent
            End Sub

            Public ReadOnly Property Current() As Object Implements System.Collections.IEnumerator.Current
                Get
                    Return CurrentEntry
                End Get
            End Property

            Public Function MoveNext() As Boolean Implements System.Collections.IEnumerator.MoveNext
                ' If we've itterated until we've reached the last one,
                ' then stop!
                If mintCurrentIndex < mobjParent.Count Then
                    CurrentEntry = mobjParent.Item(mintCurrentIndex)
                    mintCurrentIndex += 1
                    Return True
                Else
                    mobjParent.Dispose()
                    Return False
                End If
            End Function

            Public Sub Reset() Implements System.Collections.IEnumerator.Reset
                mintCurrentIndex = 0
            End Sub
        End Class

    End Class
End Namespace