'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 27, 2003
'=============================================

''' <summary>
''' Contais a number of collection objects.
''' </summary>
Namespace Collections


    ''' <summary>
    ''' Provides a simplified collection class for DirectDrawLayer types.
    ''' </summary>
    Public Class DirectDrawLayerCollection
        Private mobjItems() As DirectDrawLayer
        Private mintActualCount As Integer = 0

        Private Function FirstFreeIndex() As Integer
            Dim idx As Integer

            For idx = 0 To mobjItems.Length - 1
                If mobjItems(idx) Is Nothing Then Return idx
            Next

            Return -1
        End Function

        ''' <summary>
        ''' Adds a DirectDrawLayer type to the collection.
        ''' </summary>
        ''' <param name="Item">A reference to a DirectDrawLayer type.</param>
        ''' <returns>Returns the index that the item was added to.</returns>
        ''' <exception cref="ArgumentNullException">
        '''     Will throw a ArgumentNullException Exception if 'Item' is nothing.
        ''' </exception>
        Public Function Add(ByVal Item As DirectDrawLayer) As Integer
            Dim FreeIndex As Integer

            If Item Is Nothing Then Throw New ArgumentNullException("Item")

            If mobjItems Is Nothing Then
                ReDim mobjItems(0)
                FreeIndex = 0
            Else
                FreeIndex = FirstFreeIndex()
                If FreeIndex = -1 Then
                    ReDim Preserve mobjItems(mobjItems.Length)
                    FreeIndex = mobjItems.Length - 1
                End If
            End If
            mintActualCount += 1
            mobjItems(FreeIndex) = Item
            Return FreeIndex
        End Function

        ''' <summary>
        ''' Removes all items.
        ''' </summary>
        Public Sub Clear()
            Erase mobjItems
            mobjItems = Nothing
            mintActualCount = 0
        End Sub

        ''' <summary>
        ''' Gets/Sets a item in the collection.
        ''' </summary>
        ''' <param name="Index">The index of the item to be retrieved.</param>
        ''' <returns>Returns a reference to a DirectDrawLayer type.</returns>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Default Public Property Item(ByVal Index As Integer) As DirectDrawLayer
            Get
                If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
                If Index < 0 Then Throw New ArgumentOutOfRangeException()
                If Index > mobjItems.Length - 1 Then Throw New ArgumentOutOfRangeException()
                Return mobjItems(Index)
            End Get
            Set(ByVal Value As DirectDrawLayer)
                If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
                If Index < 0 Then Throw New ArgumentOutOfRangeException()
                If Index > mobjItems.Length - 1 Then Throw New ArgumentOutOfRangeException()

                If mobjItems(Index) Is Nothing Then
                    If Not Value Is Nothing Then mintActualCount += 1
                Else
                    If Value Is Nothing Then mintActualCount -= 1
                End If
                mobjItems(Index) = Value
            End Set
        End Property

        ''' <summary>
        ''' Returns the number of items in the collection.
        ''' </summary>
        ''' <returns>Returns the number of items in the collection.</returns>
        Public ReadOnly Property Count() As Integer
            Get
                If Not mobjItems Is Nothing Then
                    Return mobjItems.Length
                Else
                    Return 0
                End If
            End Get
        End Property

        ''' <summary>
        ''' Returns the actual number of items in the collection.
        ''' </summary>
        ''' <returns>Returns the actual number of items in the collection.</returns>
        Public ReadOnly Property ActualCount() As Integer
            Get
                Return mintActualCount
            End Get
        End Property

#Region " Remove Overloads ... "
        ''' <summary>
        ''' Removes an item.
        ''' </summary>
        ''' <param name="Index">The index of the item to be removed.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Public Sub Remove(ByVal Index As Integer)
            If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
            If Index < 0 Then Throw New ArgumentOutOfRangeException()
            If Index > mobjItems.Length - 1 Then Throw New ArgumentOutOfRangeException()
            mobjItems(Index) = Nothing
            mintActualCount -= 1
        End Sub

        ''' <summary>
        ''' Removes a number of items.
        ''' </summary>
        ''' <param name="Indexs">The index(s) of the items to be removed.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Public Sub Remove(ByVal Indexs() As Integer)
            Dim idx As Integer

            If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
            For idx = Indexs.GetLowerBound(0) To Indexs.GetUpperBound(0)
                If Indexs(idx) < 0 Then Throw New ArgumentOutOfRangeException()
                If Indexs(idx) > mobjItems.Length - 1 Then Throw New ArgumentOutOfRangeException()
                mobjItems(Indexs(idx)) = Nothing
                mintActualCount -= 1
            Next
        End Sub

        ''' <summary>
        ''' Removes a number of items.
        ''' </summary>
        ''' <param name="Index">The index of the first item to be removed.</param>
        ''' <param name="Count">The number of items to removed.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        ''' <exception cref="ArgumentException">
        '''     Will throw a standard ArgumentException Exception if the index and count do not denote a valid range of elements in the collection.
        ''' </exception>
        Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
            Dim idx As Integer

            If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
            If Index < 0 Then Throw New ArgumentOutOfRangeException()
            If Index > mobjItems.Length - 1 Then Throw New ArgumentOutOfRangeException()

            If Index + Count > mobjItems.Length - 1 Then Throw New ArgumentException()
            For idx = Index To Index + Count
                mobjItems(idx) = Nothing
                mintActualCount -= 1
            Next
        End Sub

        ''' <summary>
        ''' Removes an item.
        ''' </summary>
        ''' <param name="Item">A reference to a Item previously added to the collection.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Public Sub Remove(ByVal Item As DirectDrawLayer)
            Dim index As Integer

            If mobjItems Is Nothing Then Throw New ArgumentOutOfRangeException()
            index = mobjItems.IndexOf(mobjItems, Item)
            If index = -1 Then Throw New ArgumentOutOfRangeException()
            mobjItems(index) = Nothing
            mintActualCount -= 1
        End Sub
#End Region
    End Class
End Namespace