'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Sunday, December 29, 2002
'=============================================

''' <summary>
''' The DX9Graphics class is used to handle Direct3D related functions.
''' </summary>
Public Class DX9Graphics
    ''' <summary>
    ''' A reference to the Direct3D.Device type.
    ''' </summary>
    Protected mobjD3D As Direct3D.Device
    ''' <summary>
    ''' Provides access to the PresentParameters.
    ''' </summary>
    Protected mobjPresent As New Direct3D.PresentParameters()
    ''' <summary>
    ''' Provides access to the default clear flags.
    ''' </summary>
    Protected menuClearFlags As Direct3D.ClearFlags
    ' Protected mobjCamera As DX9ToolsR4.Camera


    ''' <summary>
    ''' Provides access to the PresentParameters.
    ''' </summary>
    ''' <returns>Returns a refrence to a Direct3D.PresentParameters type stored internally.</returns>
    Public Property PresentParameters() As Direct3D.PresentParameters
        Get
            Return mobjPresent
        End Get
        Set(ByVal Value As Direct3D.PresentParameters)
            If Not mobjD3D Is Nothing Then
                Throw New DX9ToolsR4.Exceptions.DX9ToolsCanNotSetPresentParameters()
            End If
            mobjPresent = Value
        End Set
    End Property

    ''' <summary>
    ''' Provides access to the default clear flags.
    ''' </summary>
    ''' <returns>Returns a Direct3D.ClearFlags type.</returns>
    Public Property ClearFlags() As Direct3D.ClearFlags
        Get
            Return menuClearFlags
        End Get
        Set(ByVal Value As Direct3D.ClearFlags)
            menuClearFlags = Value
        End Set
    End Property

#Region " Clear Methods ... "
    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    Public Sub Clear()
        mobjD3D.Clear(menuClearFlags, Color.Black, 1, 0)
    End Sub

    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    Public Sub Clear(ByVal Color As Color)
        mobjD3D.Clear(menuClearFlags, Color, 1, 0)
    End Sub

    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    Public Sub Clear(ByVal Color As Integer)
        mobjD3D.Clear(menuClearFlags, Color, 1, 0)
    End Sub

    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub Clear(ByVal ZDepth As Single, ByVal Stencil As Integer)
        mobjD3D.Clear(menuClearFlags, Color.Black, ZDepth, Stencil)
    End Sub

    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub Clear(ByVal Color As Color, ByVal ZDepth As Single, ByVal Stencil As Integer)
        mobjD3D.Clear(menuClearFlags, Color, ZDepth, Stencil)
    End Sub

    ''' <summary>
    ''' Clears the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub Clear(ByVal Color As Integer, ByVal ZDepth As Single, ByVal Stencil As Integer)
        mobjD3D.Clear(menuClearFlags, Color, ZDepth, Stencil)
    End Sub
#End Region

#Region " BeginScene overloads ... "
    ''' <summary>
    ''' Begins the scene.
    ''' </summary>
    Public Sub BeginScene(Optional ByVal DoClear As Boolean = True)
        If DoClear Then Me.Clear()
        mobjD3D.BeginScene()
    End Sub

    ''' <summary>
    ''' Clears the scene then begins the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    Public Sub BeginScene(ByVal Color As Color)
        Me.Clear(Color)
        mobjD3D.BeginScene()
    End Sub

    ''' <summary>
    ''' Clears the scene then begins the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    Public Sub BeginScene(ByVal Color As Integer)
        Me.Clear(Color)
        mobjD3D.BeginScene()
    End Sub

    ''' <summary>
    ''' Clears the scene then begins the scene.
    ''' </summary>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub BeginScene(ByVal ZDepth As Single, ByVal Stencil As Integer)
        Me.Clear(ZDepth, Stencil)
        mobjD3D.BeginScene()
    End Sub

    ''' <summary>
    ''' Clears the scene then begins the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub BeginScene(ByVal Color As Color, ByVal ZDepth As Single, ByVal Stencil As Integer)
        Me.Clear(Color, ZDepth, Stencil)
        mobjD3D.BeginScene()
    End Sub

    ''' <summary>
    ''' Clears the scene then begins the scene.
    ''' </summary>
    ''' <param name="Color">The color to use to clear the scene.</param>
    ''' <param name="ZDepth">New z value that this method stores in the depth buffer. This parameter can be in the range of 0.0 through 1.0 (for z-based or w-based depth buffers). A value of 0.0 represents the nearest distance to the viewer; a value of 1.0 represents the farthest distance.</param>
    ''' <param name="Stencil">Integer value to store in each stencil-buffer entry. This parameter can be in the range of 0 through 2n-1, where n is the bit depth of the stencil buffer.</param>
    Public Sub BeginScene(ByVal Color As Integer, ByVal ZDepth As Single, ByVal Stencil As Integer)
        Me.Clear(Color, ZDepth, Stencil)
        mobjD3D.BeginScene()
    End Sub
#End Region

#Region " End Scene Overload ... "
    ''' <summary>
    ''' Ends the scene.
    ''' </summary>
    Public Sub EndScene(Optional ByVal DoPresent As Boolean = True)
        mobjD3D.EndScene()
        If DoPresent Then mobjD3D.Present()
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub EndScene(ByVal RectPresent As Rectangle, ByVal OverrideWindowHandle As IntPtr, ByVal SourceRectangle As Boolean)
        mobjD3D.EndScene()
        mobjD3D.Present(RectPresent, OverrideWindowHandle, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    Public Sub EndScene(ByVal OverrideWindowHandle As IntPtr)
        mobjD3D.EndScene()
        mobjD3D.Present(OverrideWindowHandle)
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="SourceRectangle">A Rectangle  that contains the source rectangle. If the rectangle exceeds the source surface, it is clipped to the source surface. This parameter can be used only if the swap chain was created with SwapEffect.Copy.</param>
    ''' <param name="DestRectangle">A Rectangle  that contains the destination rectangle. This parameter can be used only if the swap chain was created with SwapEffect.Copy.</param>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    Public Sub EndScene(ByVal SourceRectangle As Rectangle, ByVal DestRectangle As Rectangle, ByVal OverrideWindowHandle As IntPtr)
        mobjD3D.EndScene()
        mobjD3D.Present(SourceRectangle, DestRectangle, OverrideWindowHandle)
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub EndScene(ByVal RectPresent As Rectangle, ByVal SourceRectangle As Boolean)
        mobjD3D.EndScene()
        mobjD3D.Present(RectPresent, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="OverrideWindow">Pointer to a destination window whose client area is taken as the target for this presentation. If this parameter is omitted, PresentParameters.DeviceWindow is taken.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub EndScene(ByVal RectPresent As Rectangle, ByVal OverrideWindow As Control, ByVal SourceRectangle As Boolean)
        mobjD3D.EndScene()
        mobjD3D.Present(RectPresent, OverrideWindow, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Ends the scene and calls present.
    ''' </summary>
    ''' <param name="OverrideWindow">Pointer to a destination window whose client area is taken as the target for this presentation. If this parameter is omitted, PresentParameters.DeviceWindow is taken.</param>
    Public Sub EndScene(ByVal OverrideWindow As Control)
        mobjD3D.EndScene()
        mobjD3D.Present(OverrideWindow)
    End Sub
#End Region

#Region " Present overloads ... "
    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    Public Sub Present()
        mobjD3D.Present()
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub Present(ByVal RectPresent As Rectangle, ByVal OverrideWindowHandle As IntPtr, ByVal SourceRectangle As Boolean)
        mobjD3D.Present(RectPresent, OverrideWindowHandle, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    Public Sub Present(ByVal OverrideWindowHandle As IntPtr)
        mobjD3D.Present(OverrideWindowHandle)
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="SourceRectangle">A Rectangle  that contains the source rectangle. If the rectangle exceeds the source surface, it is clipped to the source surface. This parameter can be used only if the swap chain was created with SwapEffect.Copy.</param>
    ''' <param name="DestRectangle">A Rectangle  that contains the destination rectangle. This parameter can be used only if the swap chain was created with SwapEffect.Copy.</param>
    ''' <param name="OverrideWindowHandle">Destination window whose client area is taken as the target for this presentation. If this parameter is not used, PresentParameters.DeviceWindow is taken.</param>
    Public Sub Present(ByVal SourceRectangle As Rectangle, ByVal DestRectangle As Rectangle, ByVal OverrideWindowHandle As IntPtr)
        mobjD3D.Present(SourceRectangle, DestRectangle, OverrideWindowHandle)
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub Present(ByVal RectPresent As Rectangle, ByVal SourceRectangle As Boolean)
        mobjD3D.Present(RectPresent, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="RectPresent">The Rectangle  to present.</param>
    ''' <param name="OverrideWindow">Pointer to a destination window whose client area is taken as the target for this presentation. If this parameter is omitted, PresentParameters.DeviceWindow is taken.</param>
    ''' <param name="SourceRectangle">Value that is set to true if a Rectangle  is being passed to the rectPresent parameter.</param>
    Public Sub Present(ByVal RectPresent As Rectangle, ByVal OverrideWindow As Control, ByVal SourceRectangle As Boolean)
        mobjD3D.Present(RectPresent, OverrideWindow, SourceRectangle)
    End Sub

    ''' <summary>
    ''' Presents the scene.
    ''' </summary>
    ''' <param name="OverrideWindow">Pointer to a destination window whose client area is taken as the target for this presentation. If this parameter is omitted, PresentParameters.DeviceWindow is taken.</param>
    Public Sub Present(ByVal OverrideWindow As Control)
        mobjD3D.Present(OverrideWindow)
    End Sub
#End Region

    ''' <summary>
    ''' Provides a reference to the Direct3D.Device type stored internally.
    ''' </summary>
    ''' <returns>Returns a reference to the Direct3D.Device type stored internally.</returns>
    Public ReadOnly Property Device() As Direct3D.Device
        Get
            Return mobjD3D
        End Get
    End Property

#Region " Constructor overloads ... "

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Target">Reference to the control that the DX9Graphics class will render to.</param>
    ''' <param name="Adapter">The index of the display adapter to use.</param>
    ''' <exception cref="Exception">
    '''     Will throw a standard Exception if the mode is not availible.
    ''' </exception>
    Public Sub New(ByVal Target As Control, Optional ByVal Adapter As Integer = 0)

#If Trace = True Then
         TEnter("CBX.Libs.DX9ToolsR4.DX9Graphics.New")
        TDataDump(Adapter.ToString, "Adapter", "CBX.Libs.DX9ToolsR4.DX9Graphics.New")

#End If

        Try

            mobjPresent.Windowed = True
            mobjPresent.SwapEffect = Direct3D.SwapEffect.Discard
            mobjPresent.BackBufferCount = 1
            mobjPresent.BackBufferFormat = Direct3D.Manager.Adapters(Adapter).CurrentDisplayMode.Format
            mobjPresent.BackBufferWidth = Target.ClientSize.Width()
            mobjPresent.BackBufferHeight = Target.ClientSize.Height()

            OneTimeInit(Target, Adapter)
        Catch ex As Exception
            Throw New Exceptions.DX9ToolsBaseError("Could not initialise graphics.")
        End Try

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.DX9Graphics.New")
#End If
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Target">Reference to the control that the DX9Graphics class will render to.</param>
    ''' <param name="Width">The width of the display mode to use.</param>
    ''' <param name="Height">The height of the display mode to use.</param>
    ''' <param name="Depth">The display format of the display mode to use.</param>
    ''' <param name="Adapter">The index of the display adapter to use.</param>
    ''' <exception cref="DX9ToolsBaseError">
    '''     Will throw a DX9ToolsBaseError Exception if there was a problem during initilization.
    ''' </exception>
    ''' <exception cref="DX9ToolsModeNotAvailible">
    '''     Will throw a DX9ToolsModeNotAvailible Exception if the mode is not availible.
    ''' </exception>
    Public Sub New(ByVal Target As Control, _
                   ByVal Width As Integer, _
                   ByVal Height As Integer, _
                   ByVal Depth As Direct3D.Format, _
                   Optional ByVal Adapter As Integer = 0)
#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.DX9Graphics.New")
        TDataDump(Width.ToString, "Width", "CBX.Libs.DX9ToolsR4.DX9Graphics.New")
        TDataDump(Height.ToString, "Height", "CBX.Libs.DX9ToolsR4.DX9Graphics.New")
        TDataDump(Depth.ToString, "Depth", "CBX.Libs.DX9ToolsR4.DX9Graphics.New")
        TDataDump(Adapter.ToString, "Adapter", "CBX.Libs.DX9ToolsR4.DX9Graphics.New")
#End If

        Try
            If Not GeneralHelpers.ModeAvailible(Adapter, Width, Height, Depth) Then
                Throw New Exceptions.DX9ToolsModeNotAvailible("Could not verify that the selected " _
                                                            + "resolution is compatable with the hardware", _
                                                            Exceptions.DX9TErrorCode.ModeNotAvailible)
            End If

            mobjPresent.BackBufferWidth = Width
            mobjPresent.BackBufferHeight = Height
            mobjPresent.BackBufferCount = 1
            mobjPresent.SwapEffect = Direct3D.SwapEffect.Copy
            mobjPresent.PresentationInterval = Direct3D.PresentInterval.Immediate
            mobjPresent.BackBufferFormat = Depth

            OneTimeInit(Target, Adapter)
        Catch err As Exception
            Throw New Exceptions.DX9ToolsBaseError("Could not initialize graphics.", err)
        End Try

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.DX9Graphics.New")
#End If
    End Sub

    Private Sub OneTimeInit(ByVal Ctrl As Control, Optional ByVal Adapter As Integer = 0)
#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.DX9Graphics.OneTimeInit")
#End If

        Dim objAdapters As Direct3D.AdapterInformation
        Dim DevCaps As Direct3D.Caps
        Dim DevFlags As Direct3D.CreateFlags
        Dim objCamera As Camera


        objAdapters = Direct3D.Manager.Adapters(Adapter)


        'specify depth buffer
        If Direct3D.Manager.CheckDepthStencilMatch(0, Direct3D.DeviceType.Hardware, _
                                          mobjPresent.BackBufferFormat, _
                                          mobjPresent.BackBufferFormat, _
                                          Direct3D.DepthFormat.D16) Then
            'support exists.
            mobjPresent.AutoDepthStencilFormat = Direct3D.DepthFormat.D16
            mobjPresent.EnableAutoDepthStencil = True

            #If Trace = True Then
            TInfo("Adapter supports depth buffer.", "CBX.Libs.DX9ToolsR4.DX9Graphics.OneTimeInit")
#End If

        Else
            'support does not exist!
            Throw New Exceptions.DX9ToolsBaseError("Direct3D Adapter does not support 16bit depth buffers. This sample requires a depth buffer with at least 16bits of precision.")
        End If

        ' check device capabilities
        DevCaps = Direct3D.Manager.GetDeviceCaps(Adapter, Direct3D.DeviceType.Hardware)


        ' take advantage of hw tnl where available
        If DevCaps.DeviceCaps.SupportsHardwareTransformAndLight Then
            'If DevCaps.DeviceCaps.SupportsPureDevice Then
            '    DevFlags = Direct3D.CreateFlags.HardwareVertexProcessing Or Direct3D.CreateFlags.PureDevice
            ' Else
            DevFlags = Direct3D.CreateFlags.HardwareVertexProcessing
            'End If
        Else
            DevFlags = Direct3D.CreateFlags.SoftwareVertexProcessing
        End If


#If Trace = True Then
        TDataDump(DevFlags.ToString, "DevFlags", "CBX.Libs.DX9ToolsR4.DX9Graphics.OneTimeInit")
#End If



        mobjD3D = New Direct3D.Device(objAdapters.Adapter, Direct3D.DeviceType.Hardware, Ctrl, _
                                      DevFlags, mobjPresent)

        menuClearFlags = Direct3D.ClearFlags.Target Or Direct3D.ClearFlags.ZBuffer

        ' setup device
        mobjD3D.RenderState.ZBufferEnable = True
        mobjD3D.RenderState.Lighting = False
        mobjD3D.RenderState.AlphaBlendEnable = True
        mobjD3D.RenderState.SourceBlend = Direct3D.Blend.SourceAlpha
        mobjD3D.RenderState.DestinationBlend = Direct3D.Blend.InvSourceAlpha
        mobjD3D.SamplerState(0).MinFilter = Direct3D.TextureFilter.Linear
        mobjD3D.SamplerState(0).MagFilter = Direct3D.TextureFilter.Linear
        mobjD3D.RenderState.Ambient = Color.FromArgb(128, 128, 128)

        '
        ' Setup camera
        '
        objCamera = New DX9ToolsR4.Camera(mobjD3D)
        With objCamera
            '.Position = New Vector3(0, 0, -100)
            '.LookAt = New Vector3(0, 0, 0)
            '.Orientation = New Vector3(0, 1, 0)
            '.FieldOfView = Direct3D.Geometry.DegreeToRadian(90)
            '.AspectRatio = 1.0
            '.NearPlane = 1.0
            '.FarPlane = 1000
            .UpdateVP()
        End With

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.DX9Graphics.OneTimeInit")
#End If
    End Sub
#End Region

    ''' <summary>
    ''' 
    ''' </summary>
    Protected Overrides Sub Finalize()
#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.DX9Graphics.Finalize")
#End If

        MyBase.Finalize()

        CleanUp()

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.DX9Graphics.Finalize")
#End If
    End Sub

    Private Sub CleanUp()
#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.DX9Graphics.CleanUp")
#End If

        ' mobjCamera = Nothing
        If Not mobjD3D Is Nothing Then
            mobjD3D.Dispose()
            mobjD3D = Nothing
        End If

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.DX9Graphics.CleanUp")
#End If
    End Sub

    '''' <summary>
    '''' Returns a reference to a internal camera object. 
    '''' </summary>
    '''' <returns>Returns a reference to a internal camera object.</returns>
    'Public Property Camera() As DX9ToolsR4.Camera
    '    Get
    '        Return mobjCamera
    '    End Get
    '    Set(ByVal Value As DX9ToolsR4.Camera)
    '        mobjCamera = Value
    '    End Set
    'End Property
End Class