'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 26, 2003
'=============================================


''' <summary>
''' This namespace contains GUI (Graphical User Interface) related objects, methods etc.
''' </summary>
Namespace GUI

    ''' <summary>
    ''' Provides a base class for a GUI that uses a indexed texture.
    ''' </summary>
    Public Class GUIIndexed
        Implements IGUIIndexed

        Private mobjParent As IGUI
        Private mobjChildren As GUICollection
        Private mobjVerts(3) As Direct3D.CustomVertex.TransformedColoredTextured
        Private mintTextureIndex(8) As Integer
        Private mobjRTCallback As RequestTexture

        ''' <summary>
        ''' Adds a child GUI to this GUI.
        ''' </summary>
        ''' <param name="Item">A object that implements a IGUI interface.</param>
        ''' <returns>Returns the index that the item was added to.</returns>       
        Public Function AddChild(ByVal Item As IGUI) As Integer
            Return mobjChildren.Add(Item)
        End Function

#Region " Remove Overloads ... "
        ''' <summary>
        ''' Removes an item.
        ''' </summary>
        ''' <param name="Index">The index of the item to be removed.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Public Sub Remove(ByVal Index As Integer)
            mobjChildren.Remove(Index)
        End Sub

        ''' <summary>
        ''' Removes a number of items.
        ''' </summary>
        ''' <param name="Index">The index of the first item to be removed.</param>
        ''' <param name="Count">The number of items to removed.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        ''' <exception cref="ArgumentException">
        '''     Will throw a standard ArgumentException Exception if the index and count do not denote a valid range of elements in the collection.
        ''' </exception>
        Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
            mobjChildren.Remove(Index, Count)
        End Sub

        ''' <summary>
        ''' Removes an item.
        ''' </summary>
        ''' <param name="Item">A reference to a Item previously added to the gui.</param>
        ''' <exception cref="ArgumentOutOfRangeException">
        '''     Will throw a standard ArgumentOutOfRangeException Exception if the index is out of range.
        ''' </exception>
        Public Sub Remove(ByVal G As IGUI)
            mobjChildren.Remove(G)
        End Sub
#End Region

#Region " IGUIIndexed implementation ... "
        ''' <summary>
        ''' Gets/Sets the value of a child gui.
        ''' </summary>
        ''' <param name="Index">The index of the child.</param>
        ''' <returns>Returns the gui at the given index.</returns>
        Public Property Child(ByVal Index As Integer) As CBX.Libs.DX9ToolsR4.GUI.IGUI Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Child
            Get
                Return mobjChildren(Index)
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.GUI.IGUI)
                mobjChildren(Index) = Value
            End Set
        End Property

        ''' <summary>
        ''' The number of children the gui contains.
        ''' </summary>
        ''' <returns>Returns the number of child objects.</returns>
        Public ReadOnly Property ChildCount() As Integer Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.ChildCount
            Get
                Return mobjChildren.Count
            End Get
        End Property

        ''' <summary>
        ''' Returns a reference to the parent gui.
        ''' </summary>
        ''' <returns>Returns a reference to the parent gui.</returns>
        Public Property Parent() As CBX.Libs.DX9ToolsR4.GUI.IGUI Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Parent
            Get
                Return mobjParent
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.GUI.IGUI)
                mobjParent = Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the height of the gui.
        ''' </summary>
        ''' <returns>Returns the height of the gui.</returns>
        Public Property Height() As Single Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Height
            Get
                Return mobjVerts(VertexIndex.BottomLeft).Y - mobjVerts(VertexIndex.TopLeft).Y
            End Get
            Set(ByVal Value As Single)
                mobjVerts(VertexIndex.BottomLeft).Y = mobjVerts(VertexIndex.TopLeft).Y + Value
                mobjVerts(VertexIndex.BottomRight).Y = mobjVerts(VertexIndex.TopRight).Y + Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the x-coordinate of a gui's left edge in pixels.
        ''' </summary>
        ''' <returns>Returns the x-coordinate of a gui's left edge in pixels.</returns>
        Public Property Left() As Single Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Left
            Get
                Return mobjVerts(VertexIndex.TopLeft).X
            End Get
            Set(ByVal Value As Single)
                mobjVerts(VertexIndex.TopLeft).X = Value
                mobjVerts(VertexIndex.BottomLeft).X = Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the y-coordinate of a gui's top edge in pixels.
        ''' </summary>
        ''' <returns>Returns the y-coordinate of a gui's top edge in pixels.</returns>               
        Public Property Top() As Single Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Top
            Get
                Return mobjVerts(VertexIndex.TopLeft).Y
            End Get
            Set(ByVal Value As Single)
                mobjVerts(VertexIndex.TopLeft).Y = Value
                mobjVerts(VertexIndex.TopRight).Y = Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or sets the width of the gui.
        ''' </summary>
        ''' <returns>Returns the width of the gui.</returns>
        Public Property Width() As Single Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Width
            Get
                Return mobjVerts(VertexIndex.TopRight).X - mobjVerts(VertexIndex.TopLeft).X
            End Get
            Set(ByVal Value As Single)
                mobjVerts(VertexIndex.TopRight).X = mobjVerts(VertexIndex.TopLeft).X + Value
                mobjVerts(VertexIndex.BottomRight).X = mobjVerts(VertexIndex.BottomLeft).X + Value
            End Set
        End Property

        ''' <summary>
        ''' Provides access to the vertexes for the gui.
        ''' </summary>
        ''' <param name="Index">The Index if the vertex that is to be returned.</param>
        ''' <returns>Returns the vertexes for the gui.</returns>
        Public ReadOnly Property Vertexes(ByVal Index As VertexIndex) As Microsoft.DirectX.Direct3D.CustomVertex.TransformedColoredTextured Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Vertexes
            Get
                Return mobjVerts(Index)
            End Get
        End Property

        ''' <summary>
        ''' Returns all of the gui's vertexes.
        ''' </summary>
        ''' <returns>Returns the vertexes for the gui.</returns>
        Public ReadOnly Property Vertexes() As Microsoft.DirectX.Direct3D.CustomVertex.TransformedColoredTextured() Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Vertexes
            Get
                Return mobjVerts
            End Get
        End Property

        ''' <summary>
        ''' Used to render the gui.
        ''' </summary>
        ''' <param name="Device">A reference to a Direct3D.Device type.</param>
        ''' <param name="RenderChildren">If true then all chindren need to be rendered also.</param>
        ''' <returns></returns>
        Public Overridable Sub Render(ByVal Device As Microsoft.DirectX.Direct3D.Device, _
                   Optional ByVal RenderChildren As Boolean = True) Implements CBX.Libs.DX9ToolsR4.GUI.IGUI.Render
            Dim idx As Integer

            For idx = 0 To 7
                If mobjRTCallback Is Nothing Then
                    Device.SetTexture(idx, Nothing)
                Else
                    Device.SetTexture(idx, mobjRTCallback(mintTextureIndex(idx)))
                End If
            Next

            Device.DrawUserPrimitives(Direct3D.PrimitiveType.TriangleStrip, 2, mobjVerts)

            If RenderChildren Then
                For idx = 0 To mobjChildren.Count - 1
                    mobjChildren(idx).Render(Device, RenderChildren)
                Next
            End If
        End Sub

        ''' <summary>
        ''' Provides a callback method to for requesting textures.
        ''' </summary>
        ''' <returns>Returns a reference to the texture.</returns>
        Public Property RequestTextureCallback() As CBX.Libs.DX9ToolsR4.GUI.RequestTexture Implements CBX.Libs.DX9ToolsR4.GUI.IGUIIndexed.RequestTextureCallback
            Get
                Return mobjRTCallback
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.GUI.RequestTexture)
                mobjRTCallback = Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or Sets the texture index for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture index for a particular texture stage.</returns>
        Public Property TextureIndex(ByVal Stage As Integer) As Integer Implements CBX.Libs.DX9ToolsR4.GUI.IGUIIndexed.TextureIndex
            Get
                Return mintTextureIndex(Stage)
            End Get
            Set(ByVal Value As Integer)
                mintTextureIndex(Stage) = Value
            End Set
        End Property

        ''' <summary>
        ''' Gets or Sets the texture index for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture index for a particular texture stage.</returns>
        Public Property TextureIndex(ByVal Stage As TextureStage) As Integer Implements CBX.Libs.DX9ToolsR4.GUI.IGUIIndexed.TextureIndex
            Get
                Return mintTextureIndex(Stage)
            End Get
            Set(ByVal Value As Integer)
                mintTextureIndex(Stage) = Value
            End Set
        End Property
#End Region

#Region " New overloads ... "

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Left">Sets the value of the Left property.</param>
        ''' <param name="Top">Sets the value of the Top property.</param>
        ''' <param name="Width">Sets the value of the Width property.</param>
        ''' <param name="Height">Sets the value of the Height property.</param>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        ''' <param name="RHW">Sets the RHW of the vertexes.</param>
        Public Sub New(ByVal Left As Single, ByVal Top As Single, ByVal Width As Single, ByVal Height As Single, _
                       ByVal Color As Color, ByVal RHW As Single)
            mobjChildren = New GUICollection()
            mintTextureIndex(0) = -1
            mintTextureIndex(1) = -1
            mintTextureIndex(2) = -1
            mintTextureIndex(3) = -1
            mintTextureIndex(4) = -1
            mintTextureIndex(5) = -1
            mintTextureIndex(6) = -1
            mintTextureIndex(7) = -1
            ' TopLeft
            Me.SetVertexData(VertexIndex.TopLeft, _
                New Direct3D.CustomVertex.TransformedColoredTextured(Left, Top, 0, RHW, Color.ToArgb, 0, 0))
            ' Topright
            Me.SetVertexData(VertexIndex.TopRight, _
                New Direct3D.CustomVertex.TransformedColoredTextured(Left + Width, Top, 0, RHW, Color.ToArgb, 1, 0))
            ' bottomleft
            Me.SetVertexData(VertexIndex.BottomLeft, _
                New Direct3D.CustomVertex.TransformedColoredTextured(Left, Top + Height, 0, RHW, Color.ToArgb, 0, 1))
            ' bottomright
            Me.SetVertexData(VertexIndex.BottomRight, _
                New Direct3D.CustomVertex.TransformedColoredTextured(Left + Width, Top + Height, 0, RHW, Color.ToArgb, 1, 1))
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Bounds">The dimentions of the gui to be set.</param>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        ''' <param name="RHW">Sets the RHW of the vertexes.</param>
        Public Sub New(ByVal Bounds As Rectangle, _
                       ByVal Color As Color, ByVal RHW As Single)
            Me.New(Bounds.Left, Bounds.Top, Bounds.Width, Bounds.Height, Color, RHW)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Location"></param>
        ''' <param name="Size"></param>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        ''' <param name="RHW">Sets the RHW of the vertexes.</param>
        Public Sub New(ByVal Location As Point, ByVal Size As Size, _
               ByVal Color As Color, ByVal RHW As Single)
            Me.New(Location.X, Location.Y, Size.Width, Size.Height, Color, RHW)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Left">Sets the value of the Left property.</param>
        ''' <param name="Top">Sets the value of the Top property.</param>
        ''' <param name="Width">Sets the value of the Width property.</param>
        ''' <param name="Height">Sets the value of the Height property.</param>
        Public Sub New(ByVal Left As Single, ByVal Top As Single, ByVal Width As Single, ByVal Height As Single)
            Me.New(Left, Top, Width, Height, Color.White, 0)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Bounds">The dimentions of the gui to be set.</param>
        Public Sub New(ByVal Bounds As Rectangle)
            Me.New(Bounds.Left, Bounds.Top, Bounds.Width, Bounds.Height, Color.White, 0)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            Me.New(0, 0, 100, 25, Color.White, 0)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Left">Sets the value of the Left property.</param>
        ''' <param name="Top">Sets the value of the Top property.</param>
        ''' <param name="Width">Sets the value of the Width property.</param>
        ''' <param name="Height">Sets the value of the Height property.</param>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        Public Sub New(ByVal Left As Single, ByVal Top As Single, _
                       ByVal Width As Single, ByVal Height As Single, ByVal Color As Color)
            Me.New(Left, Top, Width, Height, Color, 0)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Bounds">The dimentions of the gui to be set.</param>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        Public Sub New(ByVal Bounds As Rectangle, ByVal Color As Color)
            Me.New(Bounds.Left, Bounds.Top, Bounds.Width, Bounds.Height, Color, 0)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Color">Sets the color of the vertexes.</param>
        Public Sub New(ByVal Color As Color)
            Me.New(0, 0, 100, 25, Color, 0)
        End Sub
#End Region

#Region " SetVertexData overloads ... "
        ''' <summary>
        ''' Sets the vertex data directly.
        ''' </summary>
        ''' <param name="Index">The index of the vertex to set.</param>
        ''' <param name="V">The new vertex to use.</param>
        Public Sub SetVertexData(ByVal Index As VertexIndex, ByVal V As Direct3D.CustomVertex.TransformedColoredTextured)
            mobjVerts(Index) = V
        End Sub

        ''' <summary>
        ''' Sets the vertex texture coordanants directly.
        ''' </summary>
        ''' <param name="Index">The index of the vertex to set.</param>
        ''' <param name="TextureCords">The texture coordanants to set.</param>
        Public Sub SetVertexData(ByVal Index As VertexIndex, ByVal TextureCords As Vector2)
            mobjVerts(Index).Tu = TextureCords.X
            mobjVerts(Index).Tv = TextureCords.Y
        End Sub

        ''' <summary>
        ''' Sets the vertex texture coordanants directly.
        ''' </summary>
        ''' <param name="Index">The index of the vertex to set.</param>
        ''' <param name="Tu">The tU texture coordanants to set.</param>
        ''' <param name="Tv">The tV texture coordanants to set.</param>
        Public Sub SetVertexData(ByVal Index As VertexIndex, ByVal Tu As Single, ByVal Tv As Single)
            mobjVerts(Index).Tu = Tu
            mobjVerts(Index).Tv = Tv
        End Sub

        ''' <summary>
        ''' Sets the RHW for all the verts.
        ''' </summary>
        ''' <param name="RHW">The value of the RHW to set.</param>
        Public Sub SetVertexData(ByVal RHW As Single)
            mobjVerts(VertexIndex.TopLeft).Rhw = RHW
            mobjVerts(VertexIndex.TopRight).Rhw = RHW
            mobjVerts(VertexIndex.BottomLeft).Rhw = RHW
            mobjVerts(VertexIndex.BottomRight).Rhw = RHW
        End Sub

        ''' <summary>
        ''' Sets the RHW for all the verts.
        ''' </summary>
        ''' <param name="Index">The index of the vertex to be set.</param>
        ''' <param name="RHW">The value of the RHW to set.</param>
        Public Sub SetVertexData(ByVal Index As VertexIndex, ByVal RHW As Single)
            mobjVerts(Index).Rhw = RHW
        End Sub

        ''' <summary>
        ''' Sets the color of all the vertexes.
        ''' </summary>
        ''' <param name="Color">The color to set the vertexes to.</param>
        Public Sub SetVertexData(ByVal Color As Color)
            mobjVerts(VertexIndex.TopLeft).Color = Color.ToArgb
            mobjVerts(VertexIndex.TopRight).Color = Color.ToArgb
            mobjVerts(VertexIndex.BottomLeft).Color = Color.ToArgb
            mobjVerts(VertexIndex.BottomRight).Color = Color.ToArgb
        End Sub

        ''' <summary>
        ''' Sets the color of all the vertexes.
        ''' </summary>
        ''' <param name="Index">The index of the vertex to be set.</param>
        ''' <param name="Color">The color to set the vertexes to.</param>
        Public Sub SetVertexData(ByVal Index As VertexIndex, ByVal Color As Color)
            mobjVerts(Index).Color = Color.ToArgb
        End Sub

        ''' <summary>
        ''' Sets the dimentions of the gui.
        ''' </summary>
        ''' <param name="Bounds">The dimentions of the gui to be set.</param>
        Public Sub SetVertexData(ByVal Bounds As Rectangle)
            Me.Left = Bounds.Left
            Me.Top = Bounds.Top
            Me.Width = Bounds.Width
            Me.Height = Bounds.Height
        End Sub

        ''' <summary>
        ''' Sets the location of the gui.
        ''' </summary>
        ''' <param name="Location">The location of the gui to set.</param>
        Public Sub SetVertexData(ByVal Location As Point)
            Me.Left = Location.X
            Me.Top = Location.Y
        End Sub

        ''' <summary>
        ''' Sets the size of the gui.
        ''' </summary>
        ''' <param name="Size">The size of the gui to set.</param>
        Public Sub SetVertexData(ByVal Size As Size)
            Me.Width = Size.Width
            Me.Height = Size.Height
        End Sub

        ''' <summary>
        ''' Sets the dimentions of the gui.
        ''' </summary>
        ''' <param name="Location">The location of the gui to set.</param>
        ''' <param name="Size">The size of the gui to set.</param>
        Public Sub SetVertexData(ByVal Location As Point, ByVal Size As Size)
            Me.Left = Location.X
            Me.Top = Location.Y
            Me.Width = Size.Width
            Me.Height = Size.Height
        End Sub
#End Region
    End Class
End Namespace