'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 22, 2003
'=============================================

''' <summary>
''' Contains types relates to animation.
''' </summary>
Namespace Animation

    ''' <summary>
    ''' Provides an interface for a animation key.
    ''' </summary>
    Public Interface IKey
        ''' <summary>
        ''' Represents a mesure of time.
        ''' </summary>
        ''' <returns>Returns a time value.</returns>
        Property Time() As Long
    End Interface

    ''' <summary>
    ''' Provides an interface for a MatrixKey
    ''' </summary>
    Public Interface IKeyMatrix
        Inherits IKey
        ''' <summary>
        ''' Gets sets a D3D matrix type.
        ''' </summary>
        ''' <returns>Returns a D3D matrix type.</returns>
        Property Matrix() As Matrix
    End Interface

    ''' <summary>
    ''' Provides a interface for a PRS (Position, Rotation, Scale) key.
    ''' </summary>
    Public Interface IKeyPRS3
        Inherits IKey
        ''' <summary>
        ''' Gets/Sets the position value.
        ''' </summary>
        ''' <returns>Returns the position value.</returns>
        Property Position() As Vector3
        ''' <summary>
        ''' Gets/Sets the Rotation value.
        ''' </summary>
        ''' <returns>Returns the Rotation value.</returns>
        Property Rotation() As Vector3
        ''' <summary>
        ''' Gets/Sets the Scale value.
        ''' </summary>
        ''' <returns>Returns the Scale value.</returns>
        Property Scale() As Vector3
    End Interface

    ''' <summary>
    ''' Provides a base interface for a Vector3 key.
    ''' </summary>
    Public Interface IKeyVector3
        Inherits IKey
        ''' <summary>
        ''' Gets/Sets the value of the Vector3 Type.
        ''' </summary>
        ''' <returns>Returns the value of the Vector3 Type.</returns>
        Property Value() As Vector3
    End Interface

    ''' <summary>
    ''' Provides a interface for a Position key. 
    ''' </summary>
    Public Interface IKeyPosition3
        Inherits IKeyVector3
    End Interface

    ''' <summary>
    ''' Provides a interface for a Rotation key. 
    ''' </summary>
    Public Interface IKeyRotation3
        Inherits IKeyVector3
    End Interface

    ''' <summary>
    ''' Provides a interface for a Scale key. 
    ''' </summary>
    Public Interface IKeyScale3
        Inherits IKeyVector3
    End Interface

    ''' <summary>
    ''' Provides a interface for a Direct3D TextureState.
    ''' </summary>
    Public Interface IKeyTextureState
        Inherits IKey
        ''' <summary>
        ''' Gets/Sets the texture state.
        ''' </summary>
        ''' <returns>Returns the texture state.</returns>
        Property State() As Direct3D.TextureState
    End Interface

    ''' <summary>
    ''' Provides a interface for a sprite.
    ''' </summary>
    Public Interface IKeySprite
        Inherits IKey
        ''' <summary>
        ''' Gets/Sets the Scale value for the sprite.
        ''' </summary>
        ''' <returns>Returns the Scale value for the sprite.</returns>
        Property Scale() As Vector2
        ''' <summary>
        ''' Gets/Sets the Translation value for the sprite.
        ''' </summary>
        ''' <returns>Returns the Translation value for the sprite.</returns>
        Property Translation() As Vector2
        ''' <summary>
        ''' Gets/Sets the RotationCenter value for the sprite.
        ''' </summary>
        ''' <returns>Returns the RotationCenter value for the sprite.</returns>
        Property RotationCenter() As Vector2
        ''' <summary>
        ''' Gets/Sets the Rotation value for the sprite.
        ''' </summary>
        ''' <returns>Returns the Rotation value for the sprite.</returns>
        Property Rotation() As Single
        ''' <summary>
        ''' Gets/Sets the SourceRectangle value for the sprite.
        ''' </summary>
        ''' <returns>Returns the SourceRectangle value for the sprite.</returns>
        Property SourceRectangle() As Rectangle
        ''' <summary>
        ''' Gets/Sets the Color value for the sprite.
        ''' </summary>
        ''' <returns>Returns the Color value for the sprite.</returns>
        Property Color() As Color
    End Interface

    ''' <summary>
    ''' Privides a interface for a animation track.
    ''' </summary>
    Public Interface ITrack
        ''' <summary>
        ''' Adds a track.
        ''' </summary>
        ''' <param name="Key">A object type that implements the IKey interface.</param>
        ''' <returns>Returns the index that the track was added to.</returns>
        Function Add(ByVal Key As IKey) As Integer
        ''' <summary>
        ''' Gets/Sets a key for the track.
        ''' </summary>
        ''' <param name="Index">The index of the key.</param>
        ''' <returns>Returns a reference to a object type that implements the IKey interface.</returns>
        Default Property Item(ByVal Index As Integer) As IKey
        ''' <summary>
        ''' Returns the number of keys this track contains.
        ''' </summary>
        ''' <returns>Returns the number of keys this track contains.</returns>
        ReadOnly Property Count() As Integer
        ''' <summary>
        ''' Removes a key.
        ''' </summary>
        ''' <param name="index">The index of the key to be removed.</param>
        Sub Remove(ByVal index As Integer)
        ''' <summary>
        ''' Removes a key.
        ''' </summary>
        ''' <param name="indexs">The index(s) of the keys to be removed.</param>
        Sub Remove(ByVal indexs() As Integer)
        ''' <summary>
        ''' Removes a key.
        ''' </summary>
        ''' <param name="index">The index of the key to be removed.</param>
        ''' <param name="Count">The number of keys to be removed starting at index.</param>
        Sub Remove(ByVal index As Integer, ByVal Count As Integer)
        ''' <summary>
        ''' Removes a key.
        ''' </summary>
        ''' <param name="Item">A reference to a object type that implements the IKey interface and that has already been added to the track.</param>
        Sub Remove(ByVal Item As IKey)
    End Interface
End Namespace