'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 11, 2003
'=============================================

''' <summary>
''' This namespace contains GUI (Graphical User Interface) related objects, methods etc.
''' </summary>
Namespace GUI

    ''' <summary>
    ''' Provides a delegate for requesting a texture.
    ''' </summary>
    ''' <param name="Index">The index of the texture.</param>
    ''' <returns>Returns a reference to a Direct3D.Texture type.</returns>
    Public Delegate Function RequestTexture(ByVal Index As Integer) As Direct3D.Texture

    ''' <summary>
    ''' Provides an interface for a GUI.
    ''' </summary>
    Public Interface IGUI
        ''' <summary>
        ''' The number of children the gui contains.
        ''' </summary>
        ''' <returns>Returns the number of child gui's.</returns>
        ReadOnly Property ChildCount() As Integer
        ''' <summary>
        ''' Gets/Sets the value of a child gui.
        ''' </summary>
        ''' <param name="Index">The index of the child gui.</param>
        ''' <returns>Return the gui at the given index.</returns>
        Property Child(ByVal Index As Integer) As IGUI
        ''' <summary>
        ''' Returns a reference to the parent gui.
        ''' </summary>
        ''' <returns>Returns a reference to the parent gui.</returns>
        Property Parent() As IGUI
        ''' <summary>
        ''' Gets or sets the x-coordinate of a gui's left edge in pixels.
        ''' </summary>
        ''' <returns>Returns the x-coordinate of a gui's left edge in pixels.</returns>
        Property Left() As Single
        ''' <summary>
        ''' Gets or sets the y-coordinate of a gui's top edge in pixels.
        ''' </summary>
        ''' <returns>Returns the y-coordinate of a gui's top edge in pixels.</returns>               
        Property Top() As Single
        ''' <summary>
        ''' Gets or sets the width of the gui.
        ''' </summary>
        ''' <returns>Returns the width of the gui.</returns>
        Property Width() As Single
        ''' <summary>
        ''' Gets or sets the height of the gui.
        ''' </summary>
        ''' <returns>Returns the height of the gui.</returns>
        Property Height() As Single
        ''' <summary>
        ''' Provides access to the vertexes for the gui.
        ''' </summary>
        ''' <param name="Index">The Index if the vertex that is to be returned.</param>
        ''' <returns>Returns the vertexes for the gui.</returns>
        ReadOnly Property Vertexes(ByVal Index As VertexIndex) As Direct3D.CustomVertex.TransformedColoredTextured
        ''' <summary>
        ''' Returns all of the gui's vertexes.
        ''' </summary>
        ''' <returns>Returns the vertexes for the gui.</returns>
        ReadOnly Property Vertexes() As Direct3D.CustomVertex.TransformedColoredTextured()
        ''' <summary>
        ''' Used to render the gui.
        ''' </summary>
        ''' <param name="Device">A reference to a Direct3D.Device type.</param>
        ''' <param name="RenderChildren">If true then all chindren need to be rendered also.</param>
        ''' <returns></returns>
        Sub Render(ByVal Device As Direct3D.Device, Optional ByVal RenderChildren As Boolean = True)
    End Interface

    ''' <summary>
    ''' Provides an interface for a GUI that uses a indexed texture.
    ''' </summary>
    Public Interface IGUIIndexed
        Inherits IGUI

        ''' <summary>
        ''' Gets or Sets the texture index for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture index for a particular texture stage.</returns>
        Property TextureIndex(ByVal Stage As TextureStage) As Integer
        ''' <summary>
        ''' Gets or Sets the texture index for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture index for a particular texture stage.</returns>
        Property TextureIndex(ByVal Stage As Integer) As Integer
        ''' <summary>
        ''' Provides a callback method to for requesting textures.
        ''' </summary>
        ''' <returns>Returns a reference to the texture.</returns>
        Property RequestTextureCallback() As RequestTexture
    End Interface

    ''' <summary>
    ''' Provides an interface for a GUI that uses a texture.
    ''' </summary>
    Public Interface IGUITextured
        Inherits IGUI

        ''' <summary>
        ''' Gets or Sets the texture for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture for a particular texture stage.</returns>
        Property Texture(ByVal Stage As TextureStage) As Direct3D.Texture
        ''' <summary>
        ''' Gets or Sets the texture for a particular texture stage.
        ''' </summary>
        ''' <param name="Stage">The index of the texture state.</param>
        ''' <returns>Returns the texture for a particular texture stage.</returns>
        Property Texture(ByVal Stage As Integer) As Direct3D.Texture
    End Interface
End Namespace

