'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: August 6, 2003
'=============================================

''' <summary>
''' Provides a easy way to handle the keyboard state data. And provides KeyDown and KeyUp events.
''' </summary>
Public Class KBEvents
    Dim mobjCurrentState, mobjOldState As DirectInput.KeyboardState

    ''' <summary>
    ''' Raised when a key is pressed.
    ''' </summary>
    Public Event KeyDown(ByVal Keys() As DirectInput.Key)
    ''' <summary>
    ''' Raised when a key is released.
    ''' </summary>
    Public Event KeyUp(ByVal Keys() As DirectInput.Key)


    ''' <summary>
    ''' Updates the state of the CurrentState property.
    ''' </summary>
    ''' <param name="State">The state to asign to the CurrentState property.</param>
    Public Sub Update(ByVal State As DirectInput.KeyboardState)
        mobjCurrentState = State

        ' perform events
        DoKeyDown()
        DoKeyUp()

        mobjOldState = mobjCurrentState
    End Sub

    Private Sub DoKeyDown()
        Dim Keys(), Key As DirectInput.Key
        Dim idx As Integer


        For idx = 0 To 255
            Key = CType(idx, DirectInput.Key)
            If mobjCurrentState(Key) Then
                If Keys Is Nothing Then
                    ReDim Preserve Keys(0)
                Else
                    ReDim Preserve Keys(Keys.Length)
                End If

                Keys(Keys.Length - 1) = Key
            End If
        Next

        If Not Keys Is Nothing Then RaiseEvent KeyDown(Keys)
    End Sub

    Private Sub DoKeyUp()
        Dim Keys(), Key As DirectInput.Key
        Dim idx As Integer



        For idx = 0 To 255
            Key = CType(idx, DirectInput.Key)
            If mobjOldState(Key) = True And mobjCurrentState(Key) = False Then
                If Keys Is Nothing Then
                    ReDim Preserve Keys(0)
                Else
                    ReDim Preserve Keys(Keys.Length)
                End If

                Keys(Keys.Length - 1) = Key
            End If
        Next

        If Not Keys Is Nothing Then RaiseEvent KeyUp(Keys)
    End Sub

    ''' <summary>
    ''' Represents the current state of the keyboard keys.
    ''' </summary>
    ''' <param name="Key">A DirectInput.Key type representing a key on the keyboard.</param>
    ''' <returns>Returns true is the key is pressed.</returns>
    Public ReadOnly Property CurrentState(ByVal Key As DirectInput.Key) As Boolean
        Get
            Return mobjCurrentState(Key)
        End Get
    End Property

    ''' <summary>
    ''' Represents the current state of the keyboard keys.
    ''' </summary>
    ''' <returns>Returns the current state of the keyboard.</returns>
    Public ReadOnly Property CurrentState() As DirectInput.KeyboardState
        Get
            Return mobjCurrentState
        End Get
    End Property

    ''' <summary>
    ''' Represents the previous state of the keyboard keys.
    ''' </summary>
    ''' <param name="Key">A DirectInput.Key type representing a key on the keyboard.</param>
    ''' <returns>Returns true is the key is pressed.</returns>
    Public ReadOnly Property OldState(ByVal Key As DirectInput.Key) As Boolean
        Get
            Return mobjOldState(Key)
        End Get
    End Property

     ''' <summary>
    ''' Represents the previous state of the keyboard keys.
    ''' </summary>
    ''' <returns>Returns the previous state of the keyboard.</returns>
    Public ReadOnly Property OldState() As DirectInput.KeyboardState
        Get
            Return mobjOldState
        End Get
    End Property
End Class