'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: March 21, 2003
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry

''' <summary>
''' The Sprites namespace contains types that are specific to working with sprites.
''' </summary>
Namespace Sprites

    ''' <summary>
    ''' The SpriteBlt class provides a easy way to render sprites to the screen.
    ''' </summary>
    ''' <Remarks> 
    ''' This class is more intended to be used in rapid prototyping.
    ''' </Remarks>
    Public Class SpriteBlt

        Private mobjDevice As Direct3D.Device
        Private mobjSprite As Direct3D.Sprite

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.new()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Device">A reference to a Direct3D.Device object that this class will store and use.</param>
        Public Sub New(ByVal Device As Direct3D.Device)
            If Device Is Nothing Then
                Throw New ArgumentNullException("Device")
            End If

            mobjSprite = New Direct3D.Sprite(Device)
        End Sub

        ''' <summary>
        ''' Returns a reference to a Direct3D.Sprite object that is stored internally.
        ''' </summary>
        ''' <returns>Returns a reference to a Direct3D.Sprite object.</returns>
        Public ReadOnly Property Sprite() As Direct3D.Sprite
            Get
                Return mobjSprite
            End Get
        End Property

#Region " Draw overloads ... "
        'TODO: add more draw overloads :(

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Color As Color)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Color As Integer)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single, _
                        ByVal Color As Color)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single, _
                        ByVal Color As Integer)
            mobjSprite.Draw(Texture, SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Color)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Integer)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single, _
                        ByVal Color As Color)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle, _
                        ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, ByVal Rotation As Single, _
                        ByVal Color As Integer)
            mobjSprite.Draw(Texture, SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color.White)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Color As Color)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Color As Integer)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, _
        '                ByVal RotationCenter As Vector2, ByVal Rotation As Single)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
        '                    Rotation, Translation, Color.White)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal RotationCenter As Vector2, _
        '                ByVal Rotation As Single, ByVal Color As Color)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
        '                    Rotation, Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal RotationCenter As Vector2, _
        '                ByVal Rotation As Single, ByVal Color As Integer)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
        '                    Rotation, Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '           ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color.White)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Color)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Integer)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
        '                    DegreeToRadian(0), Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, _
        '                ByVal RotationCenter As Vector2, ByVal Rotation As Single)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
        '                    Rotation, Translation, Color.White)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, _
        '                ByVal Rotation As Single, ByVal Color As Color)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
        '                    Rotation, Translation, Color)
        'End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        '''' <param name="Translation">The location on the screen to draw the sprite.</param>
        '''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        '''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        '''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        '''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        'Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, _
        '                ByVal Rotation As Single, ByVal Color As Integer)
        '    mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
        '                    Rotation, Translation, Color)
        'End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Color As Color)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Color As Integer)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, _
                        ByVal RotationCenter As Vector2, ByVal Rotation As Single)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal RotationCenter As Vector2, _
                        ByVal Rotation As Single, ByVal Color As Color)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal RotationCenter As Vector2, _
                        ByVal Rotation As Single, ByVal Color As Integer)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, New Vector2(1, 1), RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Color)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal Color As Integer)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, New Vector2(0, 0), _
                            DegreeToRadian(0), Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, _
                        ByVal RotationCenter As Vector2, ByVal Rotation As Single)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color.White)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, _
                        ByVal Rotation As Single, ByVal Color As Color)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        ''' <param name="Translation">The location on the screen to draw the sprite.</param>
        ''' <param name="Scale">The scaling to use when drawing the sprite.</param>
        ''' <param name="RotationCenter">The axis from which the sprite will rotate around.</param>
        ''' <param name="Rotation">The rotation value in radians to rotate the sprite.</param>
        ''' <param name="Color">Specifies what color the sprite will be drawn with. For example if the color RGB was (255, 0, 0) then the sprite would be drawn with a red hue.</param>
        Public Sub Draw(ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle, ByVal Translation As Vector2, ByVal Scale As Vector2, ByVal RotationCenter As Vector2, _
                        ByVal Rotation As Single, ByVal Color As Integer)
            mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Scale, RotationCenter, _
                            Rotation, Translation, Color)
        End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Sprite">A reference to a object that implements the ISpriteTransform interface.</param>
        ''' <param name="Texture">Reference to a source texture that the SpriteBlt class will draw with.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        Public Sub Draw(ByVal Sprite As ISpriteTransform, ByVal Texture As Direct3D.Texture, ByVal SourceRectangle As Rectangle)
            If Sprite.UseTransformMatrix Then
                mobjSprite.DrawTransform(Texture, SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
            Else
                mobjSprite.Draw(Texture, SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
                                                    Sprite.Rotation, Sprite.Translation, Sprite.Color)
            End If
        End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="Sprite">A reference to a object that implements the ISpriteTransform interface.</param>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        '''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        '''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        'Public Sub Draw(ByVal Sprite As ISpriteTransform, _
        '                ByVal TextureCollection As DX9ToolsR4.TextureCollection, ByVal Index As Integer, _
        '                ByVal SourceRectangle As Rectangle)

        '    If Sprite.UseTransformMatrix Then
        '        mobjSprite.DrawTransform(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
        '    Else
        '        mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
        '                                            Sprite.Rotation, Sprite.Translation, Sprite.Color)
        '    End If
        'End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Sprite">A reference to a object that implements the ISpriteTransform interface.</param>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        ''' <param name="Index">A index inside of the TextureCollection to retrieve the tecture from.</param>
        ''' <param name="SourceRectangle">A source rectangle determining what part of the texture to draw.</param>
        Public Sub Draw(ByVal Sprite As ISpriteTransform, _
                        ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection, ByVal Index As Integer, _
                        ByVal SourceRectangle As Rectangle)

            If Sprite.UseTransformMatrix Then
                mobjSprite.DrawTransform(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
            Else
                mobjSprite.Draw(CType(TextureCollection(Index), Direct3D.Texture), SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
                                                    Sprite.Rotation, Sprite.Translation, Sprite.Color)
            End If
        End Sub

        '''' <summary>
        '''' The Draw method is used to draw draw the sprite on the screen.
        '''' </summary>
        '''' <param name="Sprite">A reference to a object that implements the ISpriteTransformTextured interface.</param>
        '''' <param name="TextureCollection">A reference to a DX9ToolsR4.TextureCollection object.</param>
        'Public Sub Draw(ByVal Sprite As ISpriteTransformTextured, _
        '                ByVal TextureCollection As DX9ToolsR4.TextureCollection)
        '    If Sprite.UseTransformMatrix Then

        '        If Sprite.UseTextureIndex Then
        '            mobjSprite.DrawTransform(CType(TextureCollection(Sprite.TextureIndex), Direct3D.Texture), Sprite.SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
        '        Else
        '            mobjSprite.DrawTransform(Sprite.Texture, Sprite.SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
        '        End If

        '    Else
        '        If Sprite.UseTextureIndex Then
        '            mobjSprite.Draw(CType(TextureCollection(Sprite.TextureIndex), Direct3D.Texture), Sprite.SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
        '                                                Sprite.Rotation, Sprite.Translation, Sprite.Color)
        '        Else
        '            mobjSprite.Draw(Sprite.Texture, Sprite.SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
        '                                                Sprite.Rotation, Sprite.Translation, Sprite.Color)
        '        End If
        '    End If
        'End Sub

        ''' <summary>
        ''' The Draw method is used to draw draw the sprite on the screen.
        ''' </summary>
        ''' <param name="Sprite">A reference to a object that implements the ISpriteTransformTextured interface.</param>
        ''' <param name="TextureCollection">A reference to a DX9ToolsR4.SimpleTextureCollection object.</param>
        Public Sub Draw(ByVal Sprite As ISpriteTransformTextured, _
                        ByVal TextureCollection As DX9ToolsR4.Collections.SimpleTextureCollection)
            If Sprite.UseTransformMatrix Then

                If Sprite.UseTextureIndex Then
                    mobjSprite.DrawTransform(CType(TextureCollection(Sprite.TextureIndex), Direct3D.Texture), Sprite.SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
                Else
                    mobjSprite.DrawTransform(Sprite.Texture, Sprite.SourceRectangle, Sprite.TransformMatrix, Sprite.Color)
                End If

            Else
                If Sprite.UseTextureIndex Then
                    mobjSprite.Draw(CType(TextureCollection(Sprite.TextureIndex), Direct3D.Texture), Sprite.SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
                                                        Sprite.Rotation, Sprite.Translation, Sprite.Color)
                Else
                    mobjSprite.Draw(Sprite.Texture, Sprite.SourceRectangle, Sprite.Scaling, Sprite.RotationCenter, _
                                                        Sprite.Rotation, Sprite.Translation, Sprite.Color)
                End If
            End If
        End Sub
#End Region

        ''' <summary>
        ''' Forces an object to free resources before the object is destroyed by the Garbage Collector.
        ''' </summary>
        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            mobjSprite.Dispose()
            mobjSprite = Nothing
            mobjDevice = Nothing
        End Sub

        ''' <summary>
        ''' Returns/sets a reference to a Direct3D.Device object that is stored internally.
        ''' </summary>
        ''' <returns>Returns/Sets a reference to a Direct3D.Device object.</returns>
        Public Property Device() As Direct3D.Device
            Get
                Return mobjDevice
            End Get
            Set(ByVal Value As Direct3D.Device)
                mobjDevice = Value
            End Set
        End Property
    End Class
End Namespace