'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: December 11, 2003
'=============================================

Public Class OpenSaveTileTracking
    Public Enum TileTrackingFileType
        TSE = 0
        TXT = 1
    End Enum

    Private Structure TileSetFile
        Public File As String
        Public ID As Integer
    End Structure

    Private Structure MapFileEntry
        Public TempFilename, Filename As String
        Public ID As Integer
    End Structure

    Public Shared Function LoadAsXML(ByVal Filename As String) As TileTrackingCollection
        Dim F As Xml.XmlDocument
        Dim objCollection As TileTrackingCollection
        Dim NodeX As Xml.XmlNode
        Dim idx, idy As Integer
        Dim TextureFiles() As MapFileEntry



        F = New Xml.XmlDocument()
        F.Load(Filename)

        If F.DocumentElement.Name.ToLower = "xmltilesetv1" Then
            If F.DocumentElement.HasChildNodes Then
                ' evrythings a ok
            Else
                Throw New Exception("No XML data to read!")
            End If
        Else
            Throw New Exception("Root element in XML document is not equal to 'XMLTileSetV1'.")
        End If

        ' read files first
        For Each NodeX In F.DocumentElement.ChildNodes
            Select Case NodeX.Name.ToLower
                Case "files"
                    ReadTextureFiles(NodeX, TextureFiles)
            End Select
        Next

        ' now read tiles
        For Each NodeX In F.DocumentElement.ChildNodes
            Select Case NodeX.Name.ToLower
                Case "tiles"
                    objCollection = ReadMapTiles(NodeX)
                    If objCollection Is Nothing Then
                        ' ERR: no tiles read. Should we raise an error msg???
                    End If
            End Select
        Next

        If Not objCollection Is Nothing Then
            ' assign filename properties
            For idx = 0 To TextureFiles.Length - 1

                For idy = 0 To objCollection.Count - 1
                    With objCollection(idy)
                        If CInt(.Filename) = TextureFiles(idx).ID Then .Filename = TextureFiles(idx).TempFilename
                    End With
                Next
            Next
        End If


        F = Nothing

        Return objCollection
    End Function

    Private Shared Function ReadMapTiles(ByVal e As Xml.XmlNode) As TileTrackingCollection
        Dim TexID As Integer
        Dim NodeX, NodeTile As Xml.XmlNode
        Dim objCollection As New TileTrackingCollection()
        Dim Tile As TileTrackingItem

        For Each NodeX In e.ChildNodes
            Select Case NodeX.Name.ToLower
                Case "tile"
                    Tile = New TileTrackingItem()

                    For Each NodeTile In NodeX.ChildNodes
                        Select Case NodeTile.Name.ToLower
                            Case "column"
                                Tile.Column = Xml.XmlConvert.ToInt32(NodeTile.InnerText)

                            Case "destination"
                                Tile.Destination = GetRectEntry(NodeTile)

                            Case "layer"
                                Tile.Layer = Xml.XmlConvert.ToInt32(NodeTile.InnerText)

                            Case "row"
                                Tile.Row = Xml.XmlConvert.ToInt32(NodeTile.InnerText)

                            Case "source"
                                Tile.Source = GetRectEntry(NodeTile)

                            Case "textureid"
                                Tile.Filename = NodeTile.InnerText
                        End Select
                    Next

                    objCollection.Add(Tile)
            End Select
        Next

        Return objCollection
    End Function

    Private Shared Sub ReadTextureFiles(ByVal e As Xml.XmlNode, ByRef TextureFiles() As MapFileEntry)
        Dim Textures() As MapFileEntry
        Dim NodeX, NodeFile As Xml.XmlNode
        Dim idx As Integer


        For Each NodeX In e.ChildNodes
            Select Case NodeX.Name.ToLower
                Case "file"

                    If Textures Is Nothing Then
                        ReDim Textures(0)
                    Else
                        ReDim Preserve Textures(Textures.Length)
                    End If

                    For Each NodeFile In NodeX.ChildNodes
                        Select Case NodeFile.Name.ToLower
                            Case "id"
                                Textures(Textures.Length - 1).ID = Xml.XmlConvert.ToInt32(NodeFile.InnerText)

                            Case "originalstoredfilename"
                                Textures(Textures.Length - 1).Filename = NodeFile.InnerText

                            Case "storedfile"
                                Dim ff As IO.BinaryWriter

                                Textures(Textures.Length - 1).TempFilename = IO.Path.GetTempFileName
                                ff = New IO.BinaryWriter(IO.File.Create(Textures(Textures.Length - 1).TempFilename))
                                ff.Write(Convert.FromBase64String(NodeFile.InnerText))
                                ff.Close()
                                ff = Nothing
                        End Select
                    Next
            End Select
        Next

        TextureFiles = Textures
        '' load textures if there were textures loaded
        'If Not Textures Is Nothing Then
        '    Dim idy As Integer = 0

        '    ' ensure that textures are added according to there id
        '    idx = 0
        '    idy = 0

        '    While idx <= Textures.Length - 1
        '        For idy = 0 To Textures.Length - 1
        '            If Textures(idy).ID = idx Then
        '                ' gobjTextures.Add(Direct3D.TextureLoader.FromFile(gobjGraphics.Device, Textures(idy).TempFilename))
        '                idx += 1
        '            End If
        '        Next
        '    End While
        'End If
    End Sub

    Private Shared Function GetRectEntry(ByVal e As Xml.XmlNode) As Rectangle
        Dim R As New Rectangle()
        Dim NodeX As Xml.XmlNode

        For Each NodeX In e.ChildNodes
            Select Case NodeX.Name.ToLower
                Case "left"
                    R.X = Xml.XmlConvert.ToInt32(NodeX.InnerText)

                Case "top"
                    R.Y = Xml.XmlConvert.ToInt32(NodeX.InnerText)

                Case "width"
                    R.Width = Xml.XmlConvert.ToInt32(NodeX.InnerText)

                Case "height"
                    R.Height = Xml.XmlConvert.ToInt32(NodeX.InnerText)

            End Select
        Next

        Return R
    End Function

    Public Shared Sub SaveAsXML(ByVal Filename As String, ByVal TrackedTiles As TileTrackingCollection)
        Dim f As Xml.XmlDocument
        Dim Item As TileTrackingItem
        Dim idx As Integer
        Dim Files() As TileSetFile

        If TrackedTiles Is Nothing Then
            Throw New Exception("There are no texture files or there were no tracked tiles!")
            Exit Sub
        End If

        Files = BuidFileList(TrackedTiles)
        If Files Is Nothing Then
            Throw New Exception("There are no texture files or there were no tracked tiles!")
            Exit Sub
        End If

        f = New Xml.XmlDocument()

        Dim xmldecl As Xml.XmlDeclaration
        xmldecl = f.CreateXmlDeclaration("1.0", Nothing, Nothing)

        'Add the new node to the document.
        Dim root As Xml.XmlElement = f.DocumentElement
        f.InsertBefore(xmldecl, root)

        Dim RootNode As Xml.XmlNode = f.CreateNode(Xml.XmlNodeType.Element, "XMLTileSetV1", "")
        f.AppendChild(RootNode)


        Dim FilesNode As Xml.XmlNode = f.CreateNode(Xml.XmlNodeType.Element, "Files", "")
        Dim TilesNode As Xml.XmlNode = f.CreateNode(Xml.XmlNodeType.Element, "Tiles", "")

        RootNode.AppendChild(FilesNode)
        RootNode.AppendChild(TilesNode)


        Dim FileNode As Xml.XmlNode
        For idx = 0 To Files.Length - 1
            FileNode = f.CreateNode(Xml.XmlNodeType.Element, "File", "")


            Dim ID As Xml.XmlNode

            ID = f.CreateNode(Xml.XmlNodeType.Element, "ID", "")
            ID.InnerText = Files(idx).ID.ToString

            Dim ff As IO.BinaryReader
            Dim Btys() As Byte
            ff = New IO.BinaryReader(IO.File.OpenRead(Files(idx).File))
            Btys = ff.ReadBytes(CInt(ff.BaseStream.Length))
            ff.Close()
            ff = Nothing

            Dim StoredFile As Xml.XmlNode

            StoredFile = f.CreateNode(Xml.XmlNodeType.Element, "StoredFile", "")
            StoredFile.InnerText = Convert.ToBase64String(Btys)

            Dim OriginalStoredFileName As Xml.XmlNode

            OriginalStoredFileName = f.CreateNode(Xml.XmlNodeType.Element, "OriginalStoredFileName", "")
            OriginalStoredFileName.InnerText = IO.Path.GetFileName(Files(idx).File)

            FileNode.AppendChild(ID)
            FileNode.AppendChild(StoredFile)
            FileNode.AppendChild(OriginalStoredFileName)

            FilesNode.AppendChild(FileNode)
        Next

        ' save tiles
        Dim TileNode As Xml.XmlNode
        For Each Item In TrackedTiles
            TileNode = f.CreateNode(Xml.XmlNodeType.Element, "Tile", "")

            Dim Column As Xml.XmlNode

            Column = f.CreateNode(Xml.XmlNodeType.Element, "Column", "")
            Column.InnerText = Item.Column.ToString

            Dim Destination, L, T, W, H As Xml.XmlNode

            Destination = f.CreateNode(Xml.XmlNodeType.Element, "Destination", "")
            L = f.CreateNode(Xml.XmlNodeType.Element, "Left", "")
            T = f.CreateNode(Xml.XmlNodeType.Element, "Top", "")
            W = f.CreateNode(Xml.XmlNodeType.Element, "Width", "")
            H = f.CreateNode(Xml.XmlNodeType.Element, "Height", "")

            L.InnerText = Item.Destination.Left.ToString
            T.InnerText = Item.Destination.Top.ToString
            W.InnerText = Item.Destination.Width.ToString
            H.InnerText = Item.Destination.Height.ToString

            Destination.AppendChild(L)
            Destination.AppendChild(T)
            Destination.AppendChild(W)
            Destination.AppendChild(H)


            Dim Layer As Xml.XmlNode

            Layer = f.CreateNode(Xml.XmlNodeType.Element, "Layer", "")
            Layer.InnerText = Item.Layer.ToString

            Dim Row As Xml.XmlNode

            Row = f.CreateNode(Xml.XmlNodeType.Element, "Row", "")
            Row.InnerText = Item.Row.ToString

            Dim Source As Xml.XmlNode

            Source = f.CreateNode(Xml.XmlNodeType.Element, "Source", "")
            L = f.CreateNode(Xml.XmlNodeType.Element, "Left", "")
            T = f.CreateNode(Xml.XmlNodeType.Element, "Top", "")
            W = f.CreateNode(Xml.XmlNodeType.Element, "Width", "")
            H = f.CreateNode(Xml.XmlNodeType.Element, "Height", "")

            L.InnerText = Item.Source.Left.ToString
            T.InnerText = Item.Source.Top.ToString
            W.InnerText = Item.Source.Width.ToString
            H.InnerText = Item.Source.Height.ToString

            Source.AppendChild(L)
            Source.AppendChild(T)
            Source.AppendChild(W)
            Source.AppendChild(H)

            Dim TextureID As Xml.XmlNode

            TextureID = f.CreateNode(Xml.XmlNodeType.Element, "TextureID", "")
            TextureID.InnerText = GetID(Files, Item.Filename).ToString

            TileNode.AppendChild(Column)
            TileNode.AppendChild(Destination)
            TileNode.AppendChild(Layer)
            TileNode.AppendChild(Row)
            TileNode.AppendChild(Source)
            TileNode.AppendChild(TextureID)

            TilesNode.AppendChild(TileNode)
        Next

        ' finnaly save xml data to file
        f.Save(Filename)
        f = Nothing
    End Sub

    Private Shared Function GetID(ByVal FileList() As TileSetFile, ByVal File As String) As Integer
        Dim Item As TileSetFile

        For Each Item In FileList
            If Item.File.ToUpper = File.ToUpper Then Return Item.ID
        Next

        Return -1
    End Function

    Private Shared Function GetFileFromID(ByVal FileList() As TileSetFile, ByVal ID As Integer) As String
        Dim Item As TileSetFile

        For Each Item In FileList
            If Item.ID = ID Then Return Item.File
        Next

        Return Nothing
    End Function

    Private Shared Function BuidFileList(ByVal TrackedTiles As CBX.Games.EasyG.Extentions.TileTrackingCollection) As TileSetFile()
        Dim idx As Integer
        Dim Item As EasyG.Extentions.TileTrackingItem
        Dim Files() As TileSetFile

        For Each Item In TrackedTiles
            If Files Is Nothing Then
                ReDim Files(0)
            Else
                For idx = 0 To Files.Length - 1
                    If Files(idx).File.ToUpper = Item.Filename.ToUpper Then GoTo continueforloop
                Next

                ReDim Preserve Files(Files.Length)
            End If

            Files(Files.Length - 1).File = Item.Filename
            Files(Files.Length - 1).ID = Files.Length - 1

ContinueForLoop:
        Next

        Return Files
    End Function

    Public Shared Function ReadFile(ByVal Filename As String, ByVal FileType As TileTrackingFileType) As TileTrackingCollection
        Dim strParts(), strTmp As String
        Dim objFile As IO.StreamReader
        Dim objGrouper As Grouper.GrouperReader
        Dim intCount, idx, intTmp As Integer
        Dim objItem As Extentions.TileTrackingItem
        Dim objRect As Rectangle
        Dim objCollection As TileTrackingCollection

        Try
            objCollection = New TileTrackingCollection()

            Select Case FileType
                Case TileTrackingFileType.TSE
                    objGrouper = New Grouper.GrouperReader()
                    objGrouper.Open(Filename)
                    objGrouper.BeginDataBlock(0)
                    objGrouper.ReadDataBlock(intCount)
                    objGrouper.EndDataBlock()

                    objGrouper.BeginDataBlock(1)
                    For idx = 0 To intCount - 1
                        objItem = New TileTrackingItem()

                        objGrouper.ReadDataBlock(intTmp)
                        objGrouper.ReadDataBlock(objItem.Filename, intTmp)
                        objGrouper.ReadDataBlock(objItem.Layer)
                        objGrouper.ReadDataBlock(objItem.Source.Left)
                        objGrouper.ReadDataBlock(objItem.Source.Top)
                        objGrouper.ReadDataBlock(objItem.Source.Width)
                        objGrouper.ReadDataBlock(objItem.Source.Height)
                        objGrouper.ReadDataBlock(objItem.Destination.Left)
                        objGrouper.ReadDataBlock(objItem.Destination.Top)
                        objGrouper.ReadDataBlock(objItem.Destination.Width)
                        objGrouper.ReadDataBlock(objItem.Destination.Height)
                        objGrouper.ReadDataBlock(objItem.Row)
                        objGrouper.ReadDataBlock(objItem.Column)

                        objCollection.Add(objItem)
                    Next

                    objGrouper.EndDataBlock()


                    objGrouper.Close()
                    objGrouper = Nothing

                Case TileTrackingFileType.TXT

                    objFile = New IO.StreamReader(New IO.FileStream(Filename, IO.FileMode.Open, IO.FileAccess.Read, IO.FileShare.None))

                    ' read number of entries
                    strTmp = objFile.ReadLine()
                    intCount = CInt(strTmp.Trim)

                    objItem = New Extentions.TileTrackingItem()
                    For idx = 0 To intCount - 1
                        ' read filename
                        strTmp = objFile.ReadLine()
                        objItem.Filename = strTmp.Trim

                        ' read layer
                        strTmp = objFile.ReadLine()
                        objItem.Layer = CInt(strTmp.Trim)

                        ' read source
                        strTmp = objFile.ReadLine()
                        strParts = strTmp.Split(","c)
                        objRect.X = CInt(strParts(0))
                        objRect.Y = CInt(strParts(1))
                        objRect.Width = CInt(strParts(2))
                        objRect.Height = CInt(strParts(3))
                        objItem.Source = General.ToRectangle(objRect)

                        ' read destinatinn
                        strTmp = objFile.ReadLine()
                        strParts = strTmp.Split(","c)
                        objRect.X = CInt(strParts(0))
                        objRect.Y = CInt(strParts(1))
                        objRect.Width = CInt(strParts(2))
                        objRect.Height = CInt(strParts(3))
                        objItem.Destination = General.ToRectangle(objRect)

                        ' read row
                        strTmp = objFile.ReadLine()
                        objItem.Row = CInt(strTmp.Trim)

                        ' read column
                        strTmp = objFile.ReadLine()
                        objItem.Column = CInt(strTmp.Trim)

                        objCollection.Add(objItem)
                        objItem = New Extentions.TileTrackingItem()
                    Next

                    objFile.Close()
                    objFile = Nothing

                Case Else
                    ' do noting
                    'TODO: do something
            End Select

        Catch ex As Exception
            Try
                If Not objCollection Is Nothing Then
                    objCollection.Clear()
                    objCollection.Dispose()
                    objCollection = Nothing
                End If

            Catch clearcol As Exception
                ' ERR: do somthing
            End Try
        End Try

        Return objCollection
    End Function


    Public Shared Sub WriteFile(ByVal Filename As String, ByVal FileType As TileTrackingFileType, ByVal Collection As TileTrackingCollection)
        Dim objFile As IO.StreamWriter
        Dim objGrouper As Grouper.GrouperWriter
        Dim idx As Integer

        Try
            If Collection Is Nothing Then Exit Sub

            Select Case FileType
                Case TileTrackingFileType.TSE
                    objGrouper = New Grouper.GrouperWriter()
                    objGrouper.Create(Filename)
                    objGrouper.BeginDataBlock()
                    objGrouper.WriteData(Collection.Count)
                    objGrouper.EndDataBlock()

                    objGrouper.BeginDataBlock()
                    For idx = 0 To Collection.Count - 1

                        With CType(Collection(idx), TileTrackingItem)
                            objGrouper.WriteData(.Filename.Length)
                            objGrouper.WriteData(.Filename)
                            objGrouper.WriteData(.Layer)
                            objGrouper.WriteData(.Source.Left)
                            objGrouper.WriteData(.Source.Top)
                            objGrouper.WriteData(.Source.Width)
                            objGrouper.WriteData(.Source.Height)
                            objGrouper.WriteData(.Destination.Left)
                            objGrouper.WriteData(.Destination.Top)
                            objGrouper.WriteData(.Destination.Width)
                            objGrouper.WriteData(.Destination.Height)
                            objGrouper.WriteData(.Row)
                            objGrouper.WriteData(.Column)
                        End With

                    Next
                    objGrouper.EndDataBlock()


                    objGrouper.Close()
                    objGrouper = Nothing

                Case TileTrackingFileType.TXT

                    objFile = New IO.StreamWriter(New IO.FileStream(Filename, IO.FileMode.Open, IO.FileAccess.Write, IO.FileShare.None))

                    ' write number of entries
                    objFile.WriteLine(Collection.Count)

                    For idx = 0 To Collection.Count - 1

                        With CType(Collection(idx), TileTrackingItem)
                            ' write filename
                            objFile.WriteLine(.Filename)

                            ' write layer
                            objFile.WriteLine(.Layer)

                            ' write source
                            objFile.WriteLine(.Source.X.ToString & "," & _
                                              .Source.Y.ToString & "," & _
                                              .Source.Width.ToString & "," & _
                                              .Source.Height.ToString)

                            ' write destinatinn
                            objFile.WriteLine(.Destination.X.ToString & "," & _
                                              .Destination.Y.ToString & "," & _
                                              .Destination.Width.ToString & "," & _
                                              .Destination.Height.ToString)

                            ' write row
                            objFile.WriteLine(.Row)

                            ' write column
                            objFile.WriteLine(.Column)

                        End With
                    Next

                    objFile.Close()
                    objFile = Nothing

                Case Else
                    ' do noting
                    'TODO: do something
            End Select

        Catch ex As Exception
            ' ERR: do somthing
        End Try
    End Sub
End Class