

Public Class TileTrackingCollection
    Implements IEnumerable, IDisposable, ICloneable

    Private mobjItems As System.Collections.ArrayList

    Public Sub New()
        mobjItems = New System.Collections.ArrayList()
    End Sub

    Public Sub SwapItem(ByVal Index1 As Integer, ByVal Index2 As Integer)
        Dim intFirst, intLast As Integer
        Dim objFirst, objLast As Object

        ' do err chech
        If Index1 < 0 Or Index1 > mobjItems.Count - 1 Then
            Throw New ArgumentOutOfRangeException("Index1")
        End If
        If Index2 < 0 Or Index2 > mobjItems.Count - 1 Then
            Throw New ArgumentOutOfRangeException("Index2")
        End If
        If Index1 = Index2 Then Exit Sub

        intFirst = CType(IIf(Index1 < Index2, Index1, Index2), Integer)
        intLast = CType(IIf(Index1 > Index2, Index1, Index2), Integer)

        ' store items
        objFirst = mobjItems(intFirst)
        objLast = mobjItems(intLast)

        ' remove from list
        mobjItems.RemoveAt(intLast)
        mobjItems.RemoveAt(intFirst)
        ' reinsert
        mobjItems.Insert(intFirst, objFirst)
        mobjItems.Insert(intLast, objLast)
    End Sub

    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        Return New TileTrackingCollectionEnumerator(Me)
    End Function

    Public Sub Dispose() Implements System.IDisposable.Dispose
        GC.SuppressFinalize(Me)
    End Sub

    Public Function Add(ByVal Item As TileTrackingItem) As Integer
        Dim objTile As TileTrackingItem
        Dim idx As Integer


        For idx = 0 To mobjItems.Count - 1
            objTile = CType(mobjItems(idx), TileTrackingItem)
            ' check if it's on the same layer
            If objTile.Layer = Item.Layer Then

                ' check if tile has been placed here already
                If IsSameRect(objTile.Destination, Item.Destination) Then

                    ' if so check if it is the same filename
                    If objTile.Filename.Trim.ToUpper = Item.Filename.Trim.ToUpper Then

                        ' if same filename update source rect
                        objTile.Source = New Rectangle(Item.Source.Location, Item.Source.Size)
                        ' an identicle tile was found and updated so we exit
                        Return -2
                    Else
                        ' if different filename update source rect and filename
                        objTile.Filename = Item.Filename
                        objTile.Source = New Rectangle(Item.Source.Location, Item.Source.Size)
                        ' an identicle tile was found and updated so we exit
                        Return -3
                    End If
                End If
            End If
        Next

        ' seems to be no maching tiles found so add it
        Return mobjItems.Add(Item)
    End Function

    Public Sub Clear()
        mobjItems.Clear()
    End Sub

    Public ReadOnly Property Count() As Integer
        Get
            Return mobjItems.Count
        End Get
    End Property

    Public Function IndexOf(ByVal Item As TileTrackingItem) As Integer
        Return mobjItems.IndexOf(Item)
    End Function

    Public Sub Insert(ByVal Index As Integer, ByVal Item As TileTrackingItem)
        mobjItems.Insert(Index, Item)
    End Sub

    Default Public ReadOnly Property Item(ByVal Index As Integer) As TileTrackingItem
        Get
            Return CType(mobjItems.Item(Index), TileTrackingItem)
        End Get
    End Property

    Public Sub Remove(ByVal Item As TileTrackingItem)
        mobjItems.Remove(Item)
    End Sub

    Public Sub Remove(ByVal Index As Integer)
        mobjItems.RemoveAt(Index)
    End Sub

    Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
        mobjItems.RemoveRange(Index, Count)
    End Sub

    Class TileTrackingCollectionEnumerator
        Implements IEnumerator

        Private mobjParent As TileTrackingCollection
        Private CurrentEntry As TileTrackingItem
        Private mintCurrentIndex As Integer = 0

        ' Create a reference back to the object instance that 
        ' owns this Enumberator.
        Public Sub New(ByVal xParent As TileTrackingCollection)
            mobjParent = xParent
        End Sub

        Public ReadOnly Property Current() As Object Implements System.Collections.IEnumerator.Current
            Get
                Return CurrentEntry
            End Get
        End Property

        Public Function MoveNext() As Boolean Implements System.Collections.IEnumerator.MoveNext
            ' If we've itterated until we've reached the last one,
            ' then stop!
            If mintCurrentIndex < mobjParent.Count Then
                CurrentEntry = mobjParent.Item(mintCurrentIndex)
                mintCurrentIndex += 1
                Return True
            Else
                mobjParent.Dispose()
                Return False
            End If
        End Function

        Public Sub Reset() Implements System.Collections.IEnumerator.Reset
            mintCurrentIndex = 0
        End Sub
    End Class

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Dim Col As TileTrackingCollection

        Col = New TileTrackingCollection()
        With Me.GetEnumerator
            .Reset()
            While .MoveNext
                With CType(.Current, Extentions.TileTrackingItem)
                    Col.Add(CType(.Clone, Extentions.TileTrackingItem))
                End With
            End While
        End With

        Return Col
    End Function
End Class