'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: April 12, 2003
'=============================================

Imports System.Reflection

Public Module ReflectionHelpers
    ' A collection of helpers used for Reflection.  We care about asking two
    ' questions:
    '   1) Is a Member visible outside of the assembly?  (Protected and Public
    '       members are, Private and Friend members are not)
    '   2) Does a Member have Protected accessiblity?
    ' These questions are answered with the helpers below, overloaded for each
    ' type of reflection object.

    'Reflect on all members public or private on a type, except those which are inherited.
    Public Const MemberBindingFlags As BindingFlags = _
        BindingFlags.Instance Or _
        BindingFlags.Static Or _
        BindingFlags.Public Or _
        BindingFlags.NonPublic Or _
        BindingFlags.DeclaredOnly

    Public Function IsProtected(ByVal t As Type) As Boolean
        Return t.IsNestedFamily OrElse t.IsNestedFamORAssem
    End Function

    Public Function IsProtected(ByVal method As MethodBase) As Boolean
        Return method.IsFamily OrElse method.IsFamilyOrAssembly
    End Function

    Public Function IsProtected(ByVal field As FieldInfo) As Boolean
        Return field.IsFamily OrElse field.IsFamilyOrAssembly
    End Function

    Public Function IsProtected(ByVal evt As EventInfo) As Boolean
        'Accessibilty is determined by the accessibily of the hookups.
        Return IsProtected(evt.GetAddMethod(True))
    End Function

    Public Function IsProtected(ByVal prop As PropertyInfo) As Boolean
        'Accessibilty is determined by the accessibily of the accessors.
        If prop.CanRead Then
            Return IsProtected(prop.GetGetMethod(True))
        Else
            Return IsProtected(prop.GetSetMethod(True))
        End If
    End Function

    Public Function IsExportable(ByVal t As Type) As Boolean
        Return t.IsPublic OrElse t.IsNestedPublic OrElse IsProtected(t)
    End Function

    Public Function IsExportable(ByVal method As MethodBase) As Boolean
        Return method.IsPublic OrElse IsProtected(method)
    End Function

    Public Function IsExportable(ByVal field As FieldInfo) As Boolean
        Return field.IsPublic OrElse IsProtected(field)
    End Function

    Public Function IsExportable(ByVal evt As EventInfo) As Boolean
        'Accessibilty is determined by the accessibily of the hookups.
        Return IsExportable(evt.GetAddMethod(True))
    End Function

    Public Function IsExportable(ByVal prop As PropertyInfo) As Boolean
        'Accessibilty is determined by the accessibily of the accessors.
        If prop.CanRead Then
            Return IsExportable(prop.GetGetMethod(True))
        Else
            Return IsExportable(prop.GetSetMethod(True))
        End If
    End Function

    Public Function IsExportable(ByVal m As MemberInfo) As Boolean

        Select Case m.MemberType
            Case MemberTypes.Method, MemberTypes.Constructor
                Return IsExportable(CType(m, MethodBase))

            Case MemberTypes.Event
                Return IsExportable(CType(m, EventInfo))

            Case MemberTypes.Field
                Return IsExportable(CType(m, FieldInfo))

            Case MemberTypes.Property
                Return IsExportable(CType(m, PropertyInfo))

            Case MemberTypes.NestedType
                Return IsExportable(CType(m, Type))

        End Select

        Debug.Fail("Unexpected member type: ", m.GetType.ToString)
        Return False

    End Function

End Module