Option Strict On
Option Explicit On 
Imports System.io.File


<System.Runtime.InteropServices.ProgId("clsPAK_NET.clsPAK")> _
Public Class clsPAK

    ' private consts
    Private Const MAX_FILENAME_LEN As Short = 256
    Private Const UNIQUE_NAME As Short = &H0S

    ' private types
    Private Structure FileInPAK
        Public Named As String
        Public Position As Integer
        Public Length As Integer
    End Structure

    Private Structure PAKFileHeader
        Dim Ident As Integer
        Dim DirOfs As Integer
        Dim DirLen As Integer
    End Structure

    Private Const OFS_MAXPATHNAME As Short = 128
    Private Const MAX_PATH As Short = 260

    Private Const FILE_BEGIN As Short = 0
    ' The fallowing commented const are not needed
    'Private Const FILE_CURRENT = 1
    'Private Const FILE_END = 2
    '
    'Private Const OF_CANCEL = &H800
    'Private Const OF_CREATE = &H1000
    'Private Const OF_DELETE = &H200
    'Private Const OF_EXIST = &H4000
    'Private Const OF_PARSE = &H100
    'Private Const OF_PROMPT = &H2000
    'Private Const OF_READ = &H0
    'Private Const OF_READWRITE = &H2
    'Private Const OF_REOPEN = &H8000
    'Private Const OF_SHARE_COMPAT = &H0
    'Private Const OF_SHARE_DENY_NONE = &H40
    'Private Const OF_SHARE_DENY_READ = &H30
    'Private Const OF_SHARE_DENY_WRITE = &H20
    'Private Const OF_SHARE_EXCLUSIVE = &H10
    'Private Const OF_VERIFY = &H400
    Private Const OF_WRITE As Short = &H1S

    Private Structure OFSTRUCT
        Dim cBytes As Byte
        Dim fFixedDisk As Byte
        Dim nErrCode As Short
        Dim Reserved1 As Short
        Dim Reserved2 As Short
        <VBFixedArray(OFS_MAXPATHNAME)> Dim szPathName() As Byte

        'UPGRADE_TODO: "Initialize" must be called to initialize instances of this structure. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1026"'
        Public Sub Initialize()
            ReDim szPathName(OFS_MAXPATHNAME)
        End Sub
    End Structure

    ' Private vars
    Private mtypPakHeader As clsPAK.PAKFileHeader
    Private mtypPakFiles() As clsPAK.FileInPAK
    Private mlngNummtypPakFiles As Integer
    Private mstrPackedFile As String
    Public AutoSave As Boolean
    Private mobjFileIO As System.IO.FileStream

    ' private declares
    Private Declare Function GetTempFileNameA Lib "kernel32" (ByVal lpszPath As String, ByVal lpPrefixString As String, ByVal wUnique As Integer, ByVal lpTempFileName As String) As Integer
    Private Declare Function GetTempPathA Lib "kernel32" (ByVal nBufferLength As Integer, ByVal lpBuffer As String) As Integer

    'UPGRADE_WARNING: Structure OFSTRUCT may require marshalling attributes to be passed as an argument in this Declare statement. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1050"'
    Private Declare Function OpenFile Lib "kernel32" (ByVal lpFileName As String, ByRef lpReOpenBuff As clsPAK.OFSTRUCT, ByVal wStyle As Integer) As Integer
    Private Declare Function SetEndOfFile Lib "kernel32" (ByVal hFile As Integer) As Integer
    Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Integer) As Integer
    Private Declare Function DeleteFile Lib "kernel32" Alias "DeleteFileA" (ByVal lpFileName As String) As Integer
    Private Declare Function SetFilePointer Lib "kernel32" (ByVal hFile As Integer, ByVal lDistanceToMove As Integer, ByRef lpDistanceToMoveHigh As Integer, ByVal dwMoveMethod As Integer) As Integer
    'Private Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, ByVal hTemplateFile As Long) As Long
    'Private Declare Function ReadFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, lpNumberOfBytesRead As Long, lpOverlapped As OVERLAPPED) As Long
    'Private Declare Function WriteFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToWrite As Long, lpNumberOfBytesWritten As Long, lpOverlapped As OVERLAPPED) As Long

    Public Event FileAdded(ByVal SourceFile As String, ByVal StoredFile As String)
    Public Event FileRemoved(ByVal StoredFile As String)

    Public Event DirectoryAdded()


    Public Function OpenPAK(ByVal PAKFile As String) As Integer
        Dim idx As Integer


        If IsNothing(mobjFileIO) Then

            mobjFileIO = Open(PAKFile, IO.FileMode.Open, IO.FileAccess.ReadWrite, IO.FileShare.Read)
            If mobjFileIO.CanSeek = False Then
                ' error
                Return 2
            End If
        Else

            Return 1

        End If


        ' read PAK file header
        mobjFileIO.Seek(0, IO.SeekOrigin.Begin)

        FileGet(mlngFileNum, mtypPakHeader, 1)
        ' check for error
        If Err.Number Then
            OpenPAK = 4
            Exit Function
        End If

        ' check if header info is PACK
        If Me.LongToStr(mtypPakHeader.Ident) <> "PACK" Then
            If Me.ClosePAK = 0 Then OpenPAK = 5 Else OpenPAK = 6
            Exit Function
        End If


        ' calc num files in pak
        mlngNummtypPakFiles = Int(mtypPakHeader.DirLen / Len(mtypPakFiles(0)))
        ' if no files in pak exit
        If mlngNummtypPakFiles = 0 Then GoTo Skip

        ' allocate room for files
        ReDim mtypPakFiles(mlngNummtypPakFiles - 1)
        ' check for error
        If Err.Number Then
            OpenPAK = 7
            Exit Function
        End If

        ' +1 Because c++ byte count is 0-based
        ' read the file names contained in pak
        'UPGRADE_WARNING: Get was upgraded to FileGet and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
        FileGet(mlngFileNum, mtypPakFiles, mtypPakHeader.DirOfs + 1)
        ' check for error
        If Err.Number Then
            OpenPAK = 8
            Exit Function
        End If

        ' format file names read by adding Chr$(0) at end of filename
        For idx = 0 To mlngNummtypPakFiles - 1
            ' format file name
            mtypPakFiles(idx).Named = Trim(Left(mtypPakFiles(idx).Named, InStr(mtypPakFiles(idx).Named, Chr(0)) - 1))
        Next

Skip:
        ' to acknoledge OpenPAK was successfull set the
        ' name of the pak file opened
        mstrPackedFile = PAKFile
        ' record that the pak file has been opened
        mblnFileOpened = True
    End Function

    Public Function GetPAKIdent(ByVal PAKFile As String) As Integer
        Dim TmpNum As Integer
        Dim hPAK As clsPAK.PAKFileHeader

        On Error Resume Next

        If mblnFileOpened Then
            Err.Number = 55
            Exit Function
        End If

        ' open pak
        TmpNum = FreeFile()
        FileOpen(TmpNum, PAKFile, OpenMode.Binary, , OpenShare.LockRead)
        If Err.Number Then Exit Function

        ' read header
        'UPGRADE_WARNING: Get was upgraded to FileGet and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
        FileGet(mlngFileNum, hPAK)
        If Err.Number Then Exit Function

        ' close file
        FileClose(TmpNum)
        If Err.Number Then Exit Function

        ' return ident
        GetPAKIdent = hPAK.Ident
    End Function


    Public ReadOnly Property FileOpened() As Boolean
        Get
            FileOpened = mblnFileOpened
        End Get
    End Property

    Public ReadOnly Property NumberOfFiles() As Integer
        Get
            NumberOfFiles = mlngNummtypPakFiles
        End Get
    End Property

    Public ReadOnly Property PAKFile() As String
        Get
            PAKFile = mstrPackedFile
        End Get
    End Property

    Public ReadOnly Property PakIdent() As Integer
        Get
            PakIdent = mtypPakHeader.Ident
        End Get
    End Property


    Public Function GetTempFileName() As String
        Dim TmpFile, TmpPath As String

        TmpPath = Me.GetTempPath
        TmpFile = Space(Len(TmpPath) + MAX_FILENAME_LEN)
        Call GetTempFileNameA(TmpPath, VB6.GetEXEName(), UNIQUE_NAME, TmpFile)
        GetTempFileName = Left(TmpFile, InStr(TmpFile, Chr(0)) - 1)
    End Function

    Public Function GetTempPath() As String
        Dim S As String
        Dim i As Short

        i = GetTempPathA(0, "")
        S = Space(i)
        Call GetTempPathA(i, S)
        GetTempPath = Me.AddSlash(Left(S, i - 1))
    End Function

    Public Function AddSlash(ByVal S As String, Optional ByVal Pth As String = "\") As String
        Dim Tmp As String

        Tmp = Trim(S)
        If Len(Tmp) > 0 Then
            If Right(Tmp, 1) <> Pth Then
                AddSlash = Tmp & Pth
            Else
                AddSlash = Tmp
            End If
        Else
            AddSlash = Pth
        End If
    End Function

    Public Function RemSlash(ByVal S As String, Optional ByVal Pth As String = "\") As String
        Dim Tmp As String

        Tmp = Trim(S)
        If (Len(Tmp) > 0) And (Right(Tmp, 1) = Pth) Then
            RemSlash = Left(Tmp, Len(Tmp) - 1)
        Else
            RemSlash = Tmp
        End If
    End Function


    Public Function ClosePAK() As Integer
        On Error Resume Next

        ' check if already closed
        If Not mblnFileOpened Then Exit Function

        ' close the pak file
        FileClose(mlngFileNum)
        If Err.Number Then
            ClosePAK = 1
            Exit Function
        End If

        ' dispose of pak directory array
        Erase mtypPakFiles
        If Err.Number Then
            ClosePAK = 2
            Exit Function
        End If

        ' file is no longer considerd open
        mblnFileOpened = False
    End Function

    'UPGRADE_NOTE: Class_Initialize was upgraded to Class_Initialize_Renamed. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1061"'
    Private Sub Class_Initialize_Renamed()
        AutoSave = True
    End Sub
    Public Sub New()
        MyBase.New()
        Class_Initialize_Renamed()
    End Sub

    'UPGRADE_NOTE: Class_Terminate was upgraded to Class_Terminate_Renamed. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1061"'
    Private Sub Class_Terminate_Renamed()
        Me.ClosePAK()
    End Sub
    Protected Overrides Sub Finalize()
        Class_Terminate_Renamed()
        MyBase.Finalize()
    End Sub

    Public Function ExtractToTmpFile(ByVal Index As Integer, Optional ByRef KillFile As Boolean = False) As String
        Dim TmpFile As String
        Dim TmpmlngFileNum As Integer
        Dim Ret As Integer

        ' setup error handler
        On Error Resume Next

        ' check inputs
        If Not mblnFileOpened Then Exit Function
        If Index < 0 Then Exit Function
        If Index > mlngNummtypPakFiles - 1 Then Exit Function

        ' get tmp filename
        TmpFile = Me.GetTempFileName
        ' determin weather or not to delete tmp created file
        If KillFile Then
            'UPGRADE_WARNING: Dir has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
            If Dir(TmpFile) <> "" Then Kill(TmpFile)
            If Err.Number Then Exit Function
        End If

        ' get free file number and open file
        TmpmlngFileNum = FreeFile()
        FileOpen(TmpmlngFileNum, TmpFile, OpenMode.Binary, , OpenShare.LockRead)
        If Err.Number Then GoTo AnError

        ' copy data
        Ret = CopyFromFileToFile(mlngFileNum, TmpmlngFileNum, mtypPakFiles(Index).Position + 1, mtypPakFiles(Index).Length)
        ' check if error croping
        If Ret = 0 Then GoTo AnError

CleanUp:
        ' close tmp file
        FileClose(TmpmlngFileNum)
        If Err.Number Then Exit Function

        ' return tmp file name
        ExtractToTmpFile = TmpFile
        Exit Function

AnError:
        ' Prevent recursive errors
        On Error Resume Next

        ' try to del tmp file
        Kill(TmpFile)
        ' return nothing
        ExtractToTmpFile = ""
    End Function

    Public Function GetFileName(ByVal Index As Integer) As String
        If Not mblnFileOpened Then Exit Function
        If Index < 0 Then Exit Function
        If Index > mlngNummtypPakFiles - 1 Then Exit Function

        GetFileName = Trim(mtypPakFiles(Index).Named)
    End Function

    Public Function GetFileSize(ByVal Index As Integer) As Integer
        If Not mblnFileOpened Then Exit Function
        If Index < 0 Then Exit Function
        If Index > mlngNummtypPakFiles - 1 Then Exit Function

        GetFileSize = mtypPakFiles(Index).Length
    End Function

    Public Function GetFile(ByVal fName As String, Optional ByVal Pth As String = "\") As String
        Dim Pos, TmpPos As Short
        Dim TmpName As String

        TmpName = Trim(fName)

        Pos = 0 : TmpPos = 0

        Pos = InStr(TmpName, Pth)
        Do While Pos <> 0
            TmpPos = Pos
            Pos = InStr(TmpPos + 1, TmpName, Pth)
        Loop

        If TmpPos = 0 Then GetFile = "" Else GetFile = Right(TmpName, Len(TmpName) - TmpPos)
    End Function

    Public Function GetPath(ByVal fName As String, Optional ByVal Pth As String = "\") As String
        Dim Pos, TmpPos As Short
        Dim TmpName As String

        TmpName = Trim(fName)

        Pos = 0 : TmpPos = 0

        Pos = InStr(TmpName, Pth)
        Do While Pos <> 0
            TmpPos = Pos
            Pos = InStr(TmpPos + 1, TmpName, Pth)
        Loop

        If TmpPos = 0 Then GetPath = "" Else GetPath = Left(TmpName, TmpPos)
    End Function

    Public Function CreatePAK(ByVal NewFile As String) As Boolean
        Dim Ret, Written As Integer ', Overlap As OVERLAPPED

        On Error Resume Next


        If mblnFileOpened Then Exit Function

        CreatePAK = False

        ' if file exists delete it
        'UPGRADE_WARNING: Dir has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
        If Dir(NewFile) <> "" Then
            Kill(NewFile)
            '        If DeleteFile(NewFile) = 0 Then
            '            ' error
            '            Exit Function
            '        End If
        End If

        ' setup header
        mtypPakHeader.DirLen = 0
        mtypPakHeader.DirOfs = Len(mtypPakHeader)
        mtypPakHeader.Ident = 1262698832 ' 1262698832 = PACK

        ' open new file
        mlngFileNum = FreeFile()
        FileOpen(mlngFileNum, NewFile, OpenMode.Binary)

        '    ' check to see if file is greater then max_path
        '    If Len(NewFile) > MAX_PATH Then
        '        ' change dir
        '        ChDrive Left$(Trim$(NewFile), 1)
        '        ChDir Me.GetPath(NewFile)
        '    End If
        '
        '    Sec.nLength = Len(Sec)
        '    ' create file
        '    mlngFileNum = CreateFile(Trim$(NewFile), GENERIC_READ Or GENERIC_WRITE, _
        ''                         0, Sec, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, 0)
        '    ' check for error
        '
        '
        '    Ret = WriteFile(mlngFileNum, mtypPakHeader, Len(mtypPakHeader), Written, Overlap)
        ' write header
        'UPGRADE_WARNING: Put was upgraded to FilePut and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
        FilePut(mlngFileNum, mtypPakHeader, 1)

        ' remove mtypPakFiles array
        Erase mtypPakFiles

        ' store new file name
        mstrPackedFile = NewFile

        ' new pak started
        mblnFileOpened = True

        CreatePAK = True
    End Function

    Public Function LongToStr(ByVal Lng As Integer) As String
        Dim Tmp As String

        Tmp = ""
        Tmp = Tmp & Chr(Lng And &HFF)
        Tmp = Tmp & Chr((Lng And &HFF00) \ 256)
        Tmp = Tmp & Chr((Lng And &HFF0000) \ 65536)
        Tmp = Tmp & Chr((Lng And &HFF000000) \ 16777216)
        LongToStr = Tmp
    End Function

    Public Function StrToLong(ByVal Char1 As Byte, ByVal Char2 As Byte, ByVal Char3 As Byte, ByVal Char4 As Byte) As Integer
        StrToLong = CShort(CShort(CShort(Char1 And 255) + (CShort(Char2 And 255) * 256)) + (CShort(Char3 And 255) * 65536)) + (CShort(Char4 And 255) * 16777216)
    End Function

    Public Function AddFile(ByVal NewFile As String, Optional ByVal IntPath As String = "") As Boolean
        Dim lFile, TmpmlngFileNum, Pos As Integer
        Dim TmpStr As New VB6.FixedLengthString(56)
        Dim Ret As Integer

        On Error Resume Next

        ' check if pak opened or not
        If Not mblnFileOpened Then Exit Function

        ' open file to add
        TmpmlngFileNum = FreeFile()
        FileOpen(TmpmlngFileNum, NewFile, OpenMode.Binary, , OpenShare.LockRead)
        If Err.Number Then Exit Function

        ' get len of file being added
        lFile = LOF(TmpmlngFileNum)

        ' allocate room for file
        ReDim Preserve mtypPakFiles(mlngNummtypPakFiles)
        If Err.Number Then Exit Function

        ' set file info
        TmpStr.Value = Trim(Me.AddSlash(Trim(IntPath), "/") & Me.GetFile(Trim(NewFile)))
        If Len(Trim(TmpStr.Value)) < 56 Then
            Pos = Len(TmpStr.Value)
            While Mid(TmpStr.Value, Pos, 1) = " "
                Pos = Pos - 1
                If Pos <= 0 Then GoTo ExitWhile
            End While
        End If


ExitWhile:
        If Pos <> 0 Then
            mtypPakFiles(mlngNummtypPakFiles).Named = Left(TmpStr.Value, Pos) & Chr(0)
        Else
            mtypPakFiles(mlngNummtypPakFiles).Named = TmpStr.Value
        End If
        mtypPakFiles(mlngNummtypPakFiles).Length = lFile
        mtypPakFiles(mlngNummtypPakFiles).Position = mtypPakHeader.DirOfs

        ' copy data
        Ret = CopyFromFileToFile(TmpmlngFileNum, mlngFileNum, 1, lFile, mtypPakHeader.DirOfs + 1)
        If Ret <> lFile Then Exit Function

        ' close tmp file
        FileClose(TmpmlngFileNum)
        If Err.Number Then Exit Function

        ' update num files
        mlngNummtypPakFiles = mlngNummtypPakFiles + 1

        ' update mtypPakHeader
        mtypPakHeader.DirOfs = mtypPakHeader.DirOfs + lFile
        mtypPakHeader.DirLen = mtypPakHeader.DirLen + Len(mtypPakFiles(0))

        ' save changes
        If Me.AutoSave Then
            AddFile = Me.SavePAK
            If Not AddFile Then Exit Function
        Else
            AddFile = True
        End If

        RaiseEvent FileAdded(NewFile, Trim(mtypPakFiles(mlngNummtypPakFiles - 1).Named))
    End Function

    Public Function RemFile(ByVal Index As Integer) As Boolean
        Dim mFileName As Object
        Dim TmpmlngFileNum, OldIndex As Integer
        Dim Tmp As clsPAK.FileInPAK
        Dim idx As Integer

        On Error Resume Next

        ' check if pak opened or not
        If Not mblnFileOpened Then Exit Function
        If Index < 0 Then Exit Function
        If Index > mlngNummtypPakFiles - 1 Then Exit Function

        'UPGRADE_WARNING: Couldn't resolve default property of object Tmp. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1037"'
        Tmp = mtypPakFiles(Index)

        ' shift down all dirs and move internal files if index
        ' is not the last pak file in array
        If Index <> mlngNummtypPakFiles - 1 Then
            OldIndex = Index
            For idx = Index + 1 To mlngNummtypPakFiles - 1
                ' seek to begining of internal file
                Seek(mlngFileNum, mtypPakFiles(idx).Position + 1)
                If Err.Number Then Exit Function

                ' read and write internal file to new pos
                'UPGRADE_ISSUE: InputB function is not supported. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1040"'
                'UPGRADE_WARNING: Put was upgraded to FilePut and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
                FilePut(mlngFileNum, InputB(mtypPakFiles(idx).Length, mlngFileNum))
                If Err.Number Then Exit Function

                ' move file type struc
                'UPGRADE_WARNING: Couldn't resolve default property of object mtypPakFiles(OldIndex). Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1037"'
                mtypPakFiles(OldIndex) = mtypPakFiles(idx)
                mtypPakFiles(OldIndex).Position = mtypPakFiles(OldIndex).Position - Tmp.Length
                ' update header
                mtypPakHeader.DirOfs = mtypPakHeader.DirOfs - Tmp.Length

                ' position pointer
                OldIndex = OldIndex + 1
            Next
        End If

        ' reduce num files
        mlngNummtypPakFiles = mlngNummtypPakFiles - 1
        ' reduce pak files structs array
        ReDim Preserve mtypPakFiles(mlngNummtypPakFiles)


        ' modify header
        mtypPakHeader.DirLen = mtypPakHeader.DirLen - Len(mtypPakFiles(0))

        ' close file
        FileClose(mlngFileNum)
        ' crop file
        SetEOF(mtypPakHeader.DirOfs + mtypPakHeader.DirLen)

        ' reopen file
        mlngFileNum = FreeFile()
        'UPGRADE_WARNING: Couldn't resolve default property of object mFileName. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1037"'
        FileOpen(mlngFileNum, mFileName, OpenMode.Binary, , OpenShare.LockRead)

        ' save changes
        If Me.AutoSave Then
            RemFile = Me.SavePAK
            If Not RemFile Then Exit Function
        Else
            RemFile = True
        End If


        RaiseEvent FileRemoved(Trim(Tmp.Named))
    End Function

    Public Function SavePAK() As Boolean
        Dim TmpmlngFileNum, idx As Integer

        On Error Resume Next

        ' check if pak opened or not
        If Not mblnFileOpened Then Exit Function

        ' seek to start of dir structs
        Seek(mlngFileNum, mtypPakHeader.DirOfs + 1)

        ' append new files info to pak
        For idx = 0 To mlngNummtypPakFiles - 1
            'UPGRADE_WARNING: Put was upgraded to FilePut and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
            FilePut(mlngFileNum, mtypPakFiles(idx))
        Next

        ' save PAK Header
        'UPGRADE_WARNING: Put was upgraded to FilePut and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
        FilePut(mlngFileNum, mtypPakHeader, 1)

        ' crop off any data at end of file if it exists
        If LOF(mlngFileNum) > (mtypPakHeader.DirOfs + 1 + mtypPakHeader.DirLen) Then
            FileClose(mlngFileNum)
            SavePAK = SetEOF(mtypPakHeader.DirOfs + mtypPakHeader.DirLen)
            mlngFileNum = FreeFile()
            FileOpen(mlngFileNum, mstrPackedFile, OpenMode.Binary, , OpenShare.LockRead)
            If Not SavePAK Then Exit Function
        End If


        SavePAK = True
    End Function

    Public Function FindPAKFile(ByVal intName As String) As Integer
        Dim idx As Integer

        FindPAKFile = -1

        For idx = 0 To mlngNummtypPakFiles - 1
            If Trim(UCase(intName)) = UCase(Trim(mtypPakFiles(idx).Named)) Then
                FindPAKFile = idx
                Exit Function
            End If
        Next
    End Function

    Private Function SetEOF(ByVal Position As Integer) As Boolean
        'UPGRADE_WARNING: Arrays in structure OF may need to be initialized before they can be used. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1063"'
        Dim Ret As Integer
        Dim OF As clsPAK.OFSTRUCT
        Dim Tmp As Integer

        SetEOF = False
        Ret = OpenFile(mstrPackedFile, OF, OF_WRITE)
        Tmp = SetFilePointer(Ret, Position, 0, FILE_BEGIN)
        If Tmp = 0 Then Exit Function
        Tmp = SetEndOfFile(Ret)
        If Tmp <> 1 Then Exit Function
        Tmp = CloseHandle(Ret)
        If Tmp <> 1 Then Exit Function
        SetEOF = True
    End Function


    Public Function AddDir(ByVal Path As String) As Boolean
        On Error Resume Next

        AddDir = False
        If Len(Trim(Path)) = 0 Then Exit Function

        ' allocate room for path and or file
        ReDim Preserve mtypPakFiles(mlngNummtypPakFiles)
        If Err.Number Then Exit Function

        mtypPakFiles(mlngNummtypPakFiles).Length = 0
        mtypPakFiles(mlngNummtypPakFiles).Position = 0
        mtypPakFiles(mlngNummtypPakFiles).Named = Trim(Path)

        mlngNummtypPakFiles = mlngNummtypPakFiles + 1
        AddDir = True
    End Function

    Private Function CopyFromFileToFile(ByVal SourceFileNum As Integer, ByVal DestFileNum As Integer, Optional ByVal StartPos As Integer = 1, Optional ByVal NumBytesToCopy As Integer = 1, Optional ByVal DestStartPos As Integer = 1) As Integer
        Dim TmpData As String
        Dim DataBlock As Integer
        Dim BytesWritten As Integer
        Dim idx As Integer

        On Error GoTo AnError

        ' set number of bytes to copy at a time
        DataBlock = 32000
        If NumBytesToCopy < DataBlock Then DataBlock = NumBytesToCopy

        ' dimention size of string
        TmpData = New String(Chr(System.Windows.Forms.Keys.Space), DataBlock)


        ' start data copy
        BytesWritten = 0
        While True
            '        ' set start pos for read
            '        Seek SourceFileNum, StartPos + BytesWritten
            '
            '        ' set start pos for write
            '        Seek DestFileNum, DestStartPos + BytesWritten

            ' read data
            'UPGRADE_WARNING: Get was upgraded to FileGet and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
            FileGet(SourceFileNum, TmpData, StartPos + BytesWritten)

            ' write data
            'UPGRADE_WARNING: Put was upgraded to FilePut and has a new behavior. Click for more: 'ms-help://MS.VSCC/commoner/redir/redirect.htm?keyword="vbup1041"'
            FilePut(DestFileNum, TmpData, DestStartPos + BytesWritten)

            ' rec num bytes written
            BytesWritten = BytesWritten + DataBlock

            ' check if need to copy more data
            If BytesWritten >= NumBytesToCopy Then GoTo Skip

            ' check to resize datablock
            If BytesWritten + DataBlock > NumBytesToCopy Then
                DataBlock = NumBytesToCopy - BytesWritten
                TmpData = New String(Chr(System.Windows.Forms.Keys.Space), DataBlock)
                If DataBlock <= 0 Then GoTo Skip
            End If
        End While

Skip:
        ' return a-ok and exit
        CopyFromFileToFile = NumBytesToCopy
        Exit Function

AnError:
        ' Prevent recursive errors
        On Error Resume Next

        ' return bad
        CopyFromFileToFile = 0
    End Function

    Friend Sub PAKItemNameChanged(ByVal Item As CBX_GameTools.clsPAKItem)

    End Sub
End Class