'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Monday, December 23, 2002
'=============================================

Namespace Grouper
    Public Structure GrouperFileHeader
        Public Version, EntryCount As Integer
        Public TOCStartPosition As Long
    End Structure

    Public Structure GrouperTOCEntryHeader
        Public Position, Length As Long
    End Structure

    Namespace Exceptions
        Public Class DataBlockException
            Inherits Exception
            Private mstrMessage As String


            Public Overrides ReadOnly Property Message() As String
                Get
                    Return mstrMessage
                End Get
            End Property

            Public Sub New()
                MyBase.New()
                mstrMessage = ""
            End Sub

            Public Sub New(ByVal Message As String)
                MyBase.New(Message)
                mstrMessage = Message
            End Sub

            Public Sub New(ByVal Message As String, ByVal InnerException As Exception)
                MyBase.New(Message, InnerException)
                mstrMessage = Message
            End Sub
        End Class

        Public Class AddTOCEntryException
            Inherits Exception
            Private mstrMessage As String


            Public Overrides ReadOnly Property Message() As String
                Get
                    Return mstrMessage
                End Get
            End Property

            Public Sub New()
                MyBase.New()
                mstrMessage = ""
            End Sub

            Public Sub New(ByVal Message As String)
                MyBase.New(Message)
                mstrMessage = Message
            End Sub

            Public Sub New(ByVal Message As String, ByVal InnerException As Exception)
                MyBase.New(Message, InnerException)
                mstrMessage = Message
            End Sub
        End Class

        Public Class CloseFileException
            Inherits Exception
            Private mstrMessage As String

            Public Overrides ReadOnly Property Message() As String
                Get
                    Return mstrMessage
                End Get
            End Property

            Public Sub New()
                MyBase.New()
                mstrMessage = ""
            End Sub

            Public Sub New(ByVal Message As String)
                MyBase.New(Message)
                mstrMessage = Message
            End Sub

            Public Sub New(ByVal Message As String, ByVal InnerException As Exception)
                MyBase.New(Message, InnerException)
                mstrMessage = Message
            End Sub
        End Class
    End Namespace

    Public Class GrouperReader
        Private cintNotSet As Integer = -10
        Private mintFileNumber As Integer = cintNotSet
        Private mFileHeader As GrouperFileHeader
        Private mEntries() As GrouperTOCEntryHeader
        Private mblnDataBlock As Boolean = False
        Private mintCurrentDataBlock As Integer = -1

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal Filename As String)
            Me.New()
            Me.Open(Filename)
        End Sub

        Public ReadOnly Property IsOpen() As Boolean
            Get
                Return CBool(mintFileNumber <> cintNotSet)
            End Get
        End Property

        Public Sub Open(ByVal Filename As String)
            Dim idx As Integer


            If IO.File.Exists(Filename) = False Then
                Throw New IO.FileNotFoundException()
            End If

            Try
                If Me.IsOpen Then Me.Close()
            Catch ex As Exceptions.CloseFileException
                Throw New IO.FileLoadException("Error closing opened file.", ex)

            Catch ex As Exception
                Throw New Exception("Unknown Error!", ex)
            End Try

            Try
                mintFileNumber = FreeFile()

            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error getting free file number.", ex)

            Catch ex As Exception
                Throw New Exception("Unknown Error!", ex)
            End Try

            Try
                FileOpen(mintFileNumber, Filename, OpenMode.Binary, OpenAccess.Read, OpenShare.LockReadWrite)
            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error opening file.", ex)

            Catch ex As Exception
                Throw New Exception("Unknown Error!", ex)

            End Try

            Try ' read header 
                FileGet(mintFileNumber, mFileHeader.Version)
                FileGet(mintFileNumber, mFileHeader.TOCStartPosition)
                FileGet(mintFileNumber, mFileHeader.EntryCount)
            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error reading file header.", ex)
            End Try

            If mFileHeader.EntryCount > 0 Then ReDim mEntries(mFileHeader.EntryCount - 1)

            Try
                Seek(mintFileNumber, mFileHeader.TOCStartPosition)

                ' read TOC entries
                For idx = 0 To mFileHeader.EntryCount - 1
                    FileGet(mintFileNumber, mEntries(idx).Position)
                    FileGet(mintFileNumber, mEntries(idx).Length)
                Next

            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error reading TOC entries.", ex)


            Catch ex As OutOfMemoryException
                Throw New IO.FileLoadException("Not enough memory availible.", ex)

            Catch ex As Exception
                Throw New Exception("Unknown Error!", ex)
            End Try
        End Sub

        Public Sub Close()
            ' close file
            FileClose(mintFileNumber)
            mintFileNumber = cintNotSet
            mblnDataBlock = False
            mintCurrentDataBlock = -1
        End Sub

        Public Function GetTOCEntry(ByVal Index As Integer) As GrouperTOCEntryHeader
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")

            Return mEntries(Index)
        End Function

        Public Sub ReadFile(ByVal Index As Integer, ByRef Filename As String)
            Dim intTmpFile As Integer
            Dim Data As String

            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")

            intTmpFile = FreeFile()
            FileOpen(intTmpFile, Filename, OpenMode.Binary, OpenAccess.Write, OpenShare.LockReadWrite)

            Data = Space(CInt(mEntries(Index).Length))

            Seek(mintFileNumber, mEntries(Index).Position)
            FileGet(mintFileNumber, Data)

            FilePut(intTmpFile, Data)

            FileClose(intTmpFile)
        End Sub

        Public Sub BeginDataBlock(ByVal Index As Integer)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")

            Seek(mintFileNumber, mEntries(Index).Position)

            mblnDataBlock = True
            mintCurrentDataBlock = Index
        End Sub

        Public Sub EndDataBlock()
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            mblnDataBlock = False
            mintCurrentDataBlock = -1
        End Sub

        Public ReadOnly Property CurrentDataBlockIndex() As Integer
            Get
                Return mintCurrentDataBlock
            End Get
        End Property

#Region " ReadData overloads ... "
        Public Sub ReadData(ByVal Index As Integer, ByRef Data As String, ByVal Length As Integer, Optional ByVal Position As Long = 0)
            Dim strTmp As String

            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Length > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            strTmp = CStr(IIf(mEntries(Index).Length < Length, CInt(mEntries(Index).Length), Space(Length)))

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, strTmp)

            Data = strTmp
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Byte, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Integer, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Long, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub


        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Short, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Boolean, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Single, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Decimal, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Char, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Date, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub

        Public Sub ReadData(ByVal Index As Integer, ByRef Data As Double, Optional ByVal Position As Long = 0)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")
            If Position < 0 Or Position > mEntries(Index).Length - 1 Then Throw New IndexOutOfRangeException("The position argument is out of the entries bounds.")
            If Position + Len(Data) > mEntries(Index).Length Then Throw New OverflowException("Cannot read a data type with a size that exceeds the entry length.")

            If mblnDataBlock = False Then Seek(mintFileNumber, mEntries(Index).Position + Position)
            FileGet(mintFileNumber, Data)
        End Sub
#End Region

#Region " ReadDataBlock overloads ... "
        Public Sub ReadDataBlock(ByRef Data As String, ByVal Length As Integer)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data, Length)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Byte)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Integer)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Long)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Short)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Boolean)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Single)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Decimal)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Char)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Date)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub

        Public Sub ReadDataBlock(ByRef Data As Double)
            If mblnDataBlock Then
                Me.ReadData(mintCurrentDataBlock, Data)
            Else
                Throw New Exceptions.DataBlockException("A call to 'BeginDataBlock' must occour before reading data.")
            End If
        End Sub
#End Region

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            Try
                Me.Close()
            Catch e As Exception
                'ERR: do nothing
            End Try
        End Sub
    End Class

    Public Class GrouperWriter
        Private cintNotSet As Integer = -10
        Private mintFileNumber As Integer = cintNotSet
        Private mFileHeader As GrouperFileHeader
        Private mEntries() As GrouperTOCEntryHeader
        Private mblnDataBlock As Boolean = False

        Public ReadOnly Property IsOpen() As Boolean
            Get
                Return CBool(mintFileNumber <> cintNotSet)
            End Get
        End Property

        Public Sub New()
            MyBase.New()
        End Sub

        Public Sub New(ByVal Filename As String)
            Me.New()
            Me.Create(Filename)
        End Sub

        Public Sub Create(ByVal Filename As String)
            If Me.IsOpen Then Me.Close()

            mintFileNumber = FreeFile()
            Try
                FileOpen(mintFileNumber, Filename, OpenMode.Binary, OpenAccess.Write, OpenShare.LockReadWrite)

            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error opening the file.", ex)
            End Try

            Try ' write header 
                With mFileHeader
                    .Version = 1
                    .TOCStartPosition = Len(mFileHeader)
                    .EntryCount = 0
                End With
                FilePut(mintFileNumber, mFileHeader.Version)
                FilePut(mintFileNumber, mFileHeader.TOCStartPosition)
                FilePut(mintFileNumber, mFileHeader.EntryCount)
            Catch ex As IO.IOException
                Throw New IO.FileLoadException("Error writing file header.", ex)
            End Try
        End Sub

        Public Sub Close()
            Dim idx As Integer

            mblnDataBlock = False

            Try
                With mFileHeader
                    .EntryCount = mFileHeader.EntryCount
                    .TOCStartPosition = Seek(mintFileNumber)
                End With

                ' store TOC Entries
                For idx = 0 To mFileHeader.EntryCount - 1
                    FilePut(mintFileNumber, mEntries(idx).Position)
                    FilePut(mintFileNumber, mEntries(idx).Length)
                Next

                ' store header
                Seek(mintFileNumber, 1)
                FilePut(mintFileNumber, mFileHeader.Version)
                FilePut(mintFileNumber, mFileHeader.TOCStartPosition)
                FilePut(mintFileNumber, mFileHeader.EntryCount)

            Catch ex As IO.IOException
                Throw New Exceptions.CloseFileException("Error writing TOC entries to end of file.", ex)
            End Try

            Try
                ' close file
                FileClose(mintFileNumber)

            Catch ex As IO.IOException
                Throw New Exceptions.CloseFileException("Error closing file.", ex)

            End Try

            mintFileNumber = cintNotSet
        End Sub

        Public Function GetTOCEntry(ByVal Index As Integer) As GrouperTOCEntryHeader
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")
            If Index < 0 Or Index > mFileHeader.EntryCount - 1 Then Throw New IndexOutOfRangeException("The index argument is out of bounds.")

            Return mEntries(Index)
        End Function

        Friend Sub AddTOCEntry(ByVal Length As Long)
            ' NOTE: throwing an exception is really a user preference?
            If mblnDataBlock Then Throw New Exception("Cannot add new TOC entry if 'IsWritingDataBlock' is 'True'.")

            Try
                mFileHeader.EntryCount += 1
                ReDim Preserve mEntries(mFileHeader.EntryCount - 1)

                With mEntries(mFileHeader.EntryCount - 1)
                    .Position = Seek(mintFileNumber)
                    .Length = Length
                End With

            Catch ex As IO.IOException
                Throw New Exceptions.AddTOCEntryException("An 'IOException' occoured.", ex)

            Catch ex As IndexOutOfRangeException
                Throw New Exceptions.AddTOCEntryException("An 'IndexOutOfRangeException' exception occoured.", ex)

            Catch ex As Exception
                Throw New Exceptions.AddTOCEntryException("An unknown exception occoured.", ex)

            End Try
        End Sub

        Public Sub WriteFile(ByRef Filename As String)
            Dim intTmpFile As Integer
            Dim Data As String

            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            intTmpFile = FreeFile()
            FileOpen(intTmpFile, Filename, OpenMode.Binary, OpenAccess.Read, OpenShare.LockReadWrite)

            Data = Space(CInt(LOF(intTmpFile)))
            FileGet(intTmpFile, Data)

            If mblnDataBlock Then
                ' update entry to include file size
                mEntries(mFileHeader.EntryCount - 1).Length += Data.Length
            Else
                ' create new entry for file
                AddTOCEntry(Data.Length)
            End If

            ' append or write data (depends on mblnDataBlock value)
            FilePut(mintFileNumber, Data)

            FileClose(intTmpFile)
        End Sub

        Public Sub BeginDataBlock()
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            AddTOCEntry(0)
            mblnDataBlock = True
        End Sub

        Public Sub EndDataBlock()
            ' NOTE: is throwing an exception here really needed?
            'If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            mblnDataBlock = False
        End Sub

        Public ReadOnly Property IsWritingDataBlock() As Boolean
            Get
                Return mblnDataBlock
            End Get
        End Property

#Region " WriteData overloads ... "
        Public Sub WriteData(ByVal Data As String)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Data.Length
            Else
                ' create new entry
                AddTOCEntry(Data.Length)
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Byte)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Integer)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Long)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Short)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Single)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Double)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Boolean)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Char)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Decimal)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub

        Public Sub WriteData(ByVal Data As Date)
            If mintFileNumber = cintNotSet Then Throw New IO.FileNotFoundException("No file has been opened.")

            If mblnDataBlock Then
                ' append data
                FilePut(mintFileNumber, Data)
                ' update entry
                mEntries(mFileHeader.EntryCount - 1).Length += Len(Data)
            Else
                ' create new entry
                AddTOCEntry(Len(Data))
                ' write data
                FilePut(mintFileNumber, Data)
            End If
        End Sub
#End Region

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            Try
                Me.Close()
            Catch e As Exception
                'ERR: do nothing
            End Try
        End Sub
    End Class
End Namespace