'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 28, 2003
'=============================================


''' <summary>
''' The D3DGraphicFontAnimated class is used to store and render 2D graphic based fonts with animated characters.
''' </summary>
Public Class D3DGraphicFontAnimated
    Implements Interfaces.ITextureReference

    Private mobjChars(255) As Collections.AnimatedStripCollection
    Private mintTextureIndex As Integer
    Private mblnUseTextureIndex As Boolean = True
    Private mobjTexture As Direct3D.Texture

    ''' <summary>
    ''' If the UseTextureIndex is false the developer should use this reference to the texture containing the font characters.
    ''' </summary>
    ''' <returns>Returns a reference to a Direct3D.BaseTexture object.</returns>
    Public Property Texture() As Microsoft.DirectX.Direct3D.Texture Implements CBX.Libs.DX9ToolsR4.Interfaces.ITextureReference.Texture
        Get
            Return mobjTexture
        End Get
        Set(ByVal Value As Microsoft.DirectX.Direct3D.Texture)
            mobjTexture = Value
        End Set
    End Property

    ''' <summary>
    ''' UseTextureIndex is used to determin weather or not to use a internal texture reference or retrieve the texture 
    ''' using a index into a collection or array.
    ''' </summary>
    ''' <returns>Returns the index in a collection or array where the texture containing the font characters is located.</returns>
    ''' <Remarks>
    ''' If UseTextureIndex true then the developer should use the TextureIndex property to retrieve the proper 
    ''' texture from a collection.
    ''' </Remarks>
    Public Property UseTextureIndex() As Boolean Implements CBX.Libs.DX9ToolsR4.Interfaces.ITextureReference.UseTextureIndex
        Get
            Return mblnUseTextureIndex
        End Get
        Set(ByVal Value As Boolean)
            mblnUseTextureIndex = Value
        End Set
    End Property

    ''' <summary>
    ''' Represents the Index where the font texture is located.
    ''' </summary>
    ''' <returns>Returns/Sets the Index where the font texture is located.</returns>
    Public Property TextureIndex() As Integer Implements CBX.Libs.DX9ToolsR4.Interfaces.ITextureReference.TextureIndex
        Get
            Return mintTextureIndex
        End Get
        Set(ByVal Value As Integer)
            mintTextureIndex = Value
        End Set
    End Property

    ''' <summary>
    ''' Retrieves the source rectange for the character.
    ''' </summary>
    ''' <param name="Index">The Index parameter represents a characters ASCII code. See Example.</param>
    ''' <Example>
    '''Dim msg As String = "Hello!"
    '''Dim Font As DX9ToolsR4.D3DGraphicFont
    '''
    '''Msgbox(Font.Chars(Asc(msg.Chars(0))).ToString  
    ''' </Example>
    ''' <returns>Returns/Sets the source rectangle where the character is located.</returns>
    Public Property Chars(ByVal Index As Integer) As Collections.AnimatedStripCollection
        Get
            Return mobjChars(Index)
        End Get
        Set(ByVal Value As Collections.AnimatedStripCollection)
            mobjChars(Index) = Value
        End Set
    End Property

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    Public Sub New()
        MyBase.new()

#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.D3DGraphicFontAnimated.New")
#End If


        Dim idx As Integer

        For idx = mobjChars.GetLowerBound(0) To mobjChars.GetUpperBound(0)
            mobjChars(idx) = New Collections.AnimatedStripCollection()
        Next

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.D3DGraphicFontAnimated.New")
#End If
    End Sub

        ''' <summary>
    ''' Cleans up and disposes of the fonts internal varibles and references.
    ''' </summary>
    Protected Overrides Sub Finalize()
#If Trace = True Then
        TEnter("CBX.Libs.DX9ToolsR4.D3DGraphicFontAnimated.Finalize")
#End If

        MyBase.Finalize()

        Dim idx As Integer

        For idx = mobjChars.GetLowerBound(0) To mobjChars.GetUpperBound(0)
            mobjChars(idx).Clear()
            mobjChars(idx) = Nothing
        Next
        Erase mobjChars

#If Trace = True Then
        TExit("CBX.Libs.DX9ToolsR4.D3DGraphicFontAnimated.Finalize")
#End If
    End Sub
End Class