'Created by: X
'Copyright: Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: November 15, 2003
'=============================================

Public Enum ActionType
    Move = 1
    Rotate = 2
    Scale = 3
End Enum

Public Interface IActions
    ReadOnly Property Actions() As ActionStackCollection
End Interface

Public MustInherit Class Action
    Public LastActionTime As Long
    Public ActionTime As Long

    Public MustOverride ReadOnly Property ActionType() As ActionType
    Public MustOverride Sub Update(ByVal Time As Long)
    Public MustOverride Function TimeUp(ByVal Time As Long) As Boolean
End Class

Public Class ActionMove
    Inherits Action
    Public MoveFrom As Vector3
    Public MoveTo As Vector3
    Public Value As Vector3

    Public Overrides ReadOnly Property ActionType() As SimpleShooter.ActionType
        Get
            Return ActionType.Move
        End Get
    End Property

    Public Overrides Sub Update(ByVal Time As Long)
        Dim Percent As Single

        Percent = Time / CSng(LastActionTime + ActionTime)
        Value = Vector3.Lerp(MoveFrom, MoveTo, Percent)
    End Sub

    Public Overrides Function TimeUp(ByVal Time As Long) As Boolean
        Return CBool(Time > LastActionTime + ActionTime)
    End Function
End Class

Public Class ActionScale
    Inherits Action
    Public ScaleFrom As Vector3
    Public ScaleTo As Vector3
    Public Value As Vector3

    Public Overrides ReadOnly Property ActionType() As SimpleShooter.ActionType
        Get
            Return ActionType.Scale
        End Get
    End Property

    Public Overrides Sub Update(ByVal Time As Long)
        Dim Percent As Single

        Percent = Time / CSng(LastActionTime + ActionTime)
        Value = Vector3.Lerp(ScaleFrom, ScaleTo, Percent)
    End Sub

    Public Overrides Function TimeUp(ByVal Time As Long) As Boolean
        Return CBool(Time > LastActionTime + ActionTime)
    End Function
End Class

Public Class ActionRotate
    Inherits Action
    Public RotateFrom As Vector3
    Public RotateTo As Vector3
    Public Value As Vector3

    Public Overrides ReadOnly Property ActionType() As SimpleShooter.ActionType
        Get
            Return ActionType.Rotate
        End Get
    End Property

    Public Overrides Sub Update(ByVal Time As Long)
        Dim Percent As Single

        Percent = Time / CSng(LastActionTime + ActionTime)
        Value = Vector3.Lerp(RotateFrom, RotateTo, Percent)
    End Sub

    Public Overrides Function TimeUp(ByVal Time As Long) As Boolean
        Return CBool(Time > LastActionTime + ActionTime)
    End Function
End Class

Public Class ActionStackCollection
    Private mobjActions() As Action

    Public Function Pop() As Action
        Dim Item As Action

        If mobjActions Is Nothing Then
            ' do nothing
            Return Nothing
        Else
            If mobjActions.Length > 0 Then
                Item = mobjActions(mobjActions.Length - 1)
                mobjActions(mobjActions.Length - 1) = Nothing

                ReDim Preserve mobjActions(mobjActions.GetUpperBound(0) - 1)
                If mobjActions.Length = 0 Then Me.Clear()
                Return Item

            Else
                Me.Clear()
                Return Nothing
            End If
        End If
    End Function

    Public Function Peek() As Action
        If mobjActions Is Nothing Then
            ' do nothing
            Return Nothing
        Else
            Return mobjActions(mobjActions.Length - 1)
        End If
    End Function

    Public Sub Push(ByVal Item As Action)
        If mobjActions Is Nothing Then
            ReDim mobjActions(0)
        Else
            ReDim Preserve mobjActions(mobjActions.Length)
        End If

        mobjActions(mobjActions.Length - 1) = Item
    End Sub

    Public ReadOnly Property Count() As Integer
        Get
            If mobjActions Is Nothing Then Return Nothing Else Return mobjActions.Length
        End Get
    End Property

    Public Sub Clear()
        Erase mobjActions
        mobjActions = Nothing
    End Sub
End Class