Public Class GameInput
    Private mobjKB As DirectInput.Device
    Private mobjMouse As DirectInput.Device
    Private mobjKBState, mobjPrevKBState As DirectInput.KeyboardState
    Private mobjMouseState, mobjPrevMouseState As DirectInput.MouseState


    Public ReadOnly Property MouseButtonCount() As Integer
        Get
            Return mobjMouseState.GetMouseButtons.Length
        End Get
    End Property

    Public Function Acquire() As Boolean
        Try
            If Not mobjKB Is Nothing Then mobjKB.Acquire()
            If Not mobjMouse Is Nothing Then mobjMouse.Acquire()
        Catch ex As DirectInput.InputException
            'ERR: handle error?
            Return False
        End Try

        Return True
    End Function

    Public Function Update() As Boolean
        Dim TmpKBState As DirectInput.KeyboardState
        Dim TmpMouseState As DirectInput.MouseState
        Dim Ret As Boolean


        Ret = GetDeviceStates(TmpKBState, TmpMouseState)
        If Ret Then
            mobjPrevKBState = mobjKBState
            mobjPrevMouseState = mobjMouseState
            mobjKBState = TmpKBState
            mobjMouseState = TmpMouseState
            Return True
        Else
            Return False
        End If
    End Function

    Private Function GetDeviceStates(ByRef KBState As DirectInput.KeyboardState, ByRef MouseState As DirectInput.MouseState) As Boolean
        Dim ie As DirectInput.InputException = Nothing

        ' try to get kb state
        Try
            KBState = mobjKB.GetCurrentKeyboardState
        Catch
            ' DirectInput may be telling us that the input stream has been
            ' interrupted.  We aren't tracking any state between polls, so
            ' we don't have any special reset that needs to be done.
            ' We just re-acquire and try again.
            ' If input is lost then acquire and keep trying.
            Try
                mobjKB.Acquire()
            Catch inputException As DirectInput.InputException
                ie = inputException
            End Try

            ' Update the dialog text 
            If TypeOf ie Is DirectInput.OtherApplicationHasPriorityException Or TypeOf ie Is DirectInput.NotAcquiredException Then
                ' Exception may be OtherApplicationHasPriorityException or other exceptions.
                ' This may occur when the app is minimized or in the process of 
                ' switching, so just try again later.
                Return False
            ElseIf Not ie Is Nothing Then
                ' Some other exception occurred where the application is unable
                ' to acquire the device. The app will exit at this point.
                'MessageBox.Show("Unable to acquire the device. Application will now exit.")
                Return False
            End If
        End Try

        '-----------------------------------------------------

        ' try to get mouse state
        Try
            MouseState = mobjMouse.CurrentMouseState
        Catch
            ' DirectInput may be telling us that the input stream has been
            ' interrupted.  We aren't tracking any state between polls, so
            ' we don't have any special reset that needs to be done.
            ' We just re-acquire and try again.
            ' If input is lost then acquire and keep trying.
            Try
                mobjMouse.Acquire()
            Catch inputException As DirectInput.InputException
                ie = inputException
            End Try

            ' Update the dialog text 
            If TypeOf ie Is DirectInput.OtherApplicationHasPriorityException Or TypeOf ie Is DirectInput.NotAcquiredException Then
                ' Exception may be OtherApplicationHasPriorityException or other exceptions.
                ' This may occur when the app is minimized or in the process of 
                ' switching, so just try again later.
                Return False
            ElseIf Not ie Is Nothing Then
                ' Some other exception occurred where the application is unable
                ' to acquire the device. The app will exit at this point.
                'MessageBox.Show("Unable to acquire the device. Application will now exit.")
                Return False
            End If
        End Try

        Return True
    End Function

    Public Sub New(ByVal Ctrl As Control, ByVal FullScreen As Boolean)
        Dim Coop As DirectInput.CooperativeLevelFlags

        mobjKB = New DirectInput.Device(DirectInput.SystemGuid.Keyboard)
        mobjMouse = New DirectInput.Device(DirectInput.SystemGuid.Mouse)

        If FullScreen Then
            Coop = DirectInput.CooperativeLevelFlags.Exclusive Or DirectInput.CooperativeLevelFlags.Foreground
        Else
            Coop = DirectInput.CooperativeLevelFlags.NonExclusive Or DirectInput.CooperativeLevelFlags.Foreground
        End If

        mobjKB.SetDataFormat(DirectInput.DeviceDataFormat.Keyboard)
        mobjMouse.SetDataFormat(DirectInput.DeviceDataFormat.Mouse)

        Try
            mobjKB.SetCooperativeLevel(Ctrl, Coop)
        Catch dx As DirectInput.InputException
            If TypeOf dx Is DirectInput.UnsupportedException Then
                Throw dx
                Exit Sub
            End If
        End Try

        Try
            mobjMouse.SetCooperativeLevel(Ctrl, Coop)
        Catch dx As DirectInput.InputException
            If TypeOf dx Is DirectInput.UnsupportedException Then
                Throw dx
                Exit Sub
            End If
        End Try

        ' get current info
        If Not Me.Update() Then
            Throw New Exceptions.InitInputException()
            Exit Sub
        End If
        ' setup prev an get current info
        If Not Me.Update() Then
            Throw New Exceptions.InitInputException()
            Exit Sub
        End If
    End Sub

    Public ReadOnly Property MouseState() As DirectInput.MouseState
        Get
            Return mobjMouseState
        End Get
    End Property

    Public ReadOnly Property Key(ByVal Index As DirectInput.Key) As Boolean
        Get
            Return mobjKBState(Index)
        End Get
    End Property

    Public ReadOnly Property PreviousMouseState() As DirectInput.MouseState
        Get
            Return mobjPrevMouseState
        End Get
    End Property

    Public ReadOnly Property PreviousKey(ByVal Index As DirectInput.Key) As Boolean
        Get
            Return mobjPrevKBState(Index)
        End Get
    End Property
End Class
