'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Sunday, December 29, 2002
'=============================================

''' <summary>
''' Contains a number of exception classes.
''' </summary>
Namespace Exceptions
    ''' <summary>
    ''' Represents a uniuqe error code that represents an error. 
    ''' </summary>
    Public Enum DX9TErrorCode
        ''' <summary>
        ''' Represents succsess.
        ''' </summary>
        OK = 0
        ''' <summary>
        ''' Display mode is not availible.
        ''' </summary>
        ModeNotAvailible = 1
    End Enum

    ''' <summary>
    ''' DX9ToolsBaseError is the base exception from which all DX9Tool exceptions are derived.
    ''' </summary>
    Public Class DX9ToolsBaseError
        Inherits DirectXException

        ''' <summary>
        ''' The error code of the error.
        ''' </summary>
        Protected menuErrorCode As DX9TErrorCode
        ''' <summary>
        ''' The error message of the error.
        ''' </summary>
        Protected mstrErrorMsg As String


        ''' <summary>
        ''' The uniuqe error code that represents the error.
        ''' </summary>
        ''' <returns>Return a reference to a DX9TErrorCode type.</returns>
        Public Property DX9ErrorCode() As DX9TErrorCode
            Get
                Return menuErrorCode
            End Get
            Set(ByVal Value As DX9TErrorCode)
                menuErrorCode = Value
            End Set
        End Property

        ''' <summary>
        ''' The error message or the error.
        ''' </summary>
        ''' <returns>Returns a String representing the error message.</returns>
        Public Property DX9ErrorMsg() As String
            Get
                Return mstrErrorMsg
            End Get
            Set(ByVal Value As String)
                mstrErrorMsg = Value
            End Set
        End Property

#Region " Constructor overloads ... "
        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.New()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        Public Sub New(ByVal Msg As String)
            MyBase.New(Msg)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception)
            MyBase.New(Msg, Inner)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg, Inner)
            menuErrorCode = ErrorCode
        End Sub
#End Region

    End Class

    ''' <summary>
    ''' This exception is raised if the DirectDraw device is nothing.
    ''' </summary>
    Public Class DX9ToolsDDDeviceIsNothing
        Inherits DX9ToolsBaseError

#Region " Constructor overloads ... "
        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.New()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        Public Sub New(ByVal Msg As String)
            MyBase.New(Msg)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception)
            MyBase.New(Msg, Inner)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg, Inner)
            menuErrorCode = ErrorCode
        End Sub
#End Region
    End Class

    ''' <summary>
    ''' This exception is raised if the Direct3D device is nothing.
    ''' </summary>
    Public Class DX9ToolsD3DDeviceIsNothing
        Inherits DX9ToolsBaseError

#Region " Constructor overloads ... "
        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.New()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        Public Sub New(ByVal Msg As String)
            MyBase.New(Msg)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception)
            MyBase.New(Msg, Inner)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg, Inner)
            menuErrorCode = ErrorCode
        End Sub
#End Region
    End Class

    ''' <summary>
    ''' This exception is raised if the PresentParameters cannot be set.
    ''' </summary>
    Public Class DX9ToolsCanNotSetPresentParameters
        Inherits DX9ToolsBaseError

#Region " Constructor overloads ... "
        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.New()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        Public Sub New(ByVal Msg As String)
            MyBase.New(Msg)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception)
            MyBase.New(Msg, Inner)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg, Inner)
            menuErrorCode = ErrorCode
        End Sub
#End Region
    End Class

    ''' <summary>
    ''' This exception is raised if the display mode is not availible.
    ''' </summary>
    Public Class DX9ToolsModeNotAvailible
        Inherits DX9ToolsBaseError


#Region " Constructor overloads ... "
        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        Public Sub New()
            MyBase.New()
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        Public Sub New(ByVal Msg As String)
            MyBase.New(Msg)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception)
            MyBase.New(Msg, Inner)
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="Inner">The exception that is the cause of the current exception. If the innerException parameter is not a null reference (Nothing in Visual Basic), the current exception is raised in a catch block that handles the inner exception.</param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal Inner As Exception, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg, Inner)
            menuErrorCode = ErrorCode
        End Sub

        ''' <summary>
        ''' Creates a new instance of this class.
        ''' </summary>
        ''' <param name="Msg">The error message that explains the reason for the exception. </param>
        ''' <param name="ErrorCode">The error code of the error.</param>
        Public Sub New(ByVal Msg As String, ByVal ErrorCode As DX9TErrorCode)
            MyBase.New(Msg)
            menuErrorCode = ErrorCode
        End Sub
#End Region
    End Class
End Namespace