'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: January 13, 2003
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry

''' <summary>
''' The gneral helpers module contains a number of helper methods to assist the developer.
''' </summary>
Public Class GeneralHelpers

#If Trace = True Then
    ''' <summary>
    ''' This method enables or disables writing to the debug log ang is only availible if TRACE condition is enabled. 
    ''' </summary>
    ''' <param name="Value">If True setup tracers. If False shuts down tracers.</param>
    Public Shared Sub SetupDX9TTracers(ByVal Value As Boolean)
        If Value Then SetupTracers() Else ShutdownTracers()
    End Sub
#End If

#Region " Save/Load Texture state methods ..."
    ''' <summary>
    ''' Saves the states of a Direct3D.TextureStates type to a file.
    ''' </summary>
    ''' <param name="File">The name of the file to save to.</param>
    ''' <param name="States">A reference to a Direct3D.TextureStates type.</param>
    Public Shared Sub SaveTextureStates(ByVal File As String, ByVal States As Direct3D.TextureStates)
        Dim idx As Integer

        If IO.File.Exists(File) Then IO.File.Delete(File)
        For idx = 0 To 7
            SetINIData(File, "Header", "Stage" & idx.ToString, "True")
            PrivateSaveTextureState(File, idx, States(idx))
        Next
    End Sub

    ''' <summary>
    ''' Saves the states of a Direct3D.TextureStates type to a file.
    ''' </summary>
    ''' <param name="File">The name of the file to save to.</param>
    ''' <param name="Stage">The texture stage to be saved.</param>
    ''' <param name="State">The Direct3D.TextureState type to save.</param>
    Public Shared Sub SaveTextureState(ByVal File As String, ByVal Stage As Integer, ByVal State As Direct3D.TextureState)
        Dim idx As Integer


        If IO.File.Exists(File) Then IO.File.Delete(File)

        For idx = 0 To 7
            If Stage = idx Then
                SetINIData(File, "Header", "Stage" & idx.ToString, "True")
            Else
                SetINIData(File, "Header", "Stage" & idx.ToString, "False")
            End If
        Next

        PrivateSaveTextureState(File, Stage, State)
    End Sub

    ''' <summary>
    ''' Loads the states of a Direct3D.TextureStates type from a file.
    ''' </summary>
    ''' <param name="File">The name of the file to load from.</param>
    ''' <param name="Stage">The texture stage to be loaded.</param>
    ''' <returns>Returns a Direct3D.TextureState type.</returns>
    Public Shared Function LoadTextureState(ByVal File As String, ByVal Stage As Integer) As Direct3D.TextureState
        Dim idx As Integer
        Dim State As Direct3D.TextureState
        Dim A, R, G, B As Integer

        State = New Direct3D.TextureState()

        If Not CBool(GetINIData(File, "Header", "Stage" & Stage.ToString)) Then Return Nothing


        State.AlphaArgument0 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "AlphaArgument0")), Direct3D.TextureArgument)
        State.AlphaArgument1 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "AlphaArgument1")), Direct3D.TextureArgument)
        State.AlphaArgument2 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "AlphaArgument2")), Direct3D.TextureArgument)
        State.AlphaOperation = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "AlphaOperation")), Direct3D.TextureOperation)
        State.BumpEnvironmentLuminanceOffset = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentLuminanceOffset"))
        State.BumpEnvironmentLuminanceScale = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentLuminanceScale"))
        State.BumpEnvironmentMaterial00 = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentMaterial00"))
        State.BumpEnvironmentMaterial01 = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentMaterial01"))
        State.BumpEnvironmentMaterial10 = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentMaterial10"))
        State.BumpEnvironmentMaterial11 = CSng(GetINIData(File, "Header", Stage.ToString & ":" & "BumpEnvironmentMaterial11"))
        State.ColorArgument0 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ColorArgument0")), Direct3D.TextureArgument)
        State.ColorArgument1 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ColorArgument1")), Direct3D.TextureArgument)
        State.ColorArgument2 = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ColorArgument2")), Direct3D.TextureArgument)
        State.ColorOperation = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ColorOperation")), Direct3D.TextureOperation)
        A = CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ConstantColorA"))
        R = CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ConstantColorR"))
        G = CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ConstantColorG"))
        B = CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ConstantColorB"))
        State.ConstantColor = Color.FromArgb(A, R, G, B)
        State.ResultArgument = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "ResultArgument")), Direct3D.TextureArgument)
        State.TextureCoordinateIndex = CInt(GetINIData(File, "Header", Stage.ToString & ":" & "TextureCoordinateIndex"))
        State.TextureTransform = CType(CInt(GetINIData(File, "Header", Stage.ToString & ":" & "TextureTransform")), Direct3D.TextureTransform)

        Return State
    End Function
#End Region

#Region " RenderMeshBruteForce overloads ... "
    '''' <summary>
    '''' Renders a mesh using a brute force method. No optomization. 
    '''' </summary>
    '''' <param name="Item">The mesh to be rendered.</param>
    '''' <param name="Textures">A reference to a collection containing the textures that this mesh uses.</param>
    'Public Shared Sub RenderMeshBruteForce(ByVal Item As MeshEx, ByVal Textures As SimpleTextureCollection)
    '    Dim idx As Integer
    '    Dim Mat As MaterialTextureCombo

    '    With Item
    '        For idx = 0 To .MaterialCount - 1
    '            Mat = .GetMaterial(idx)
    '            Item.Mesh.Device.Material = Mat.Material
    '            If Mat.Textures(TextureStage.Base) <> -1 Then
    '                Item.Mesh.Device.SetTexture(0, Textures(Mat.Textures(TextureStage.Base)))
    '            Else
    '                Item.Mesh.Device.SetTexture(0, Nothing)
    '            End If
    '            .Mesh.DrawSubset(idx)
    '        Next
    '    End With
    'End Sub

    ''' <summary>
    ''' Renders a mesh using a brute force method. No optomization. 
    ''' </summary>
    ''' <param name="Item">The mesh to be rendered.</param>
    ''' <param name="Textures">A reference to a collection containing the textures that this mesh uses.</param>
    Public Shared Sub RenderMeshBruteForce(ByVal Item As MeshEx, ByVal Textures As Collections.SimpleTextureCollection)
        Dim idx As Integer
        Dim Mat As MaterialTextureCombo

        With Item
            For idx = 0 To .MaterialCount - 1
                Mat = .GetMaterial(idx)
                Item.Mesh.Device.Material = Mat.Material
                If Mat.Textures(TextureStage.Base) <> -1 Then
                    Item.Mesh.Device.SetTexture(0, Textures(Mat.Textures(TextureStage.Base)))
                Else
                    Item.Mesh.Device.SetTexture(0, Nothing)
                End If
                .Mesh.DrawSubset(idx)
            Next
        End With
    End Sub
#End Region

#Region " RemoveTextureIndexes overloads ... "
    'TODO: add more RemoveTextureIndexes overloads

    ''' <summary>
    ''' Sets all of the texture indexs for all the texture stages to -1.
    ''' </summary>
    ''' <param name="Mat">A MaterialTextureCombo type representing the material to clear.</param>
    Public Shared Sub RemoveTextureIndexes(ByVal Mat As MaterialTextureCombo)
        With Mat
            .Textures(TextureStage.Texture0) = -1
            .Textures(TextureStage.Texture1) = -1
            .Textures(TextureStage.Texture2) = -1
            .Textures(TextureStage.Texture3) = -1
            .Textures(TextureStage.Texture4) = -1
            .Textures(TextureStage.Texture5) = -1
            .Textures(TextureStage.Texture6) = -1
            .Textures(TextureStage.Texture7) = -1
        End With
    End Sub
#End Region

#Region " UVFromRectangle overloads ... "
    'TODO add more UVFromRectangle oberloads ?
    'ERR possible stack overflow error with these overloads check and tripple check them


    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageWidth">The width of the image.</param>
    ''' <param name="ImageHeight">The height of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageWidth As Single, ByVal ImageHeight As Single, _
                                    ByVal Left As Single, ByVal Top As Single, _
                                    ByVal Right As Single, ByVal Bottom As Single) As Vector2()

        Dim objTmp(3) As Vector2

        objTmp(DX9ToolsR4.GUI.VertexIndex.TopLeft) = UVFromPoint(ImageWidth, ImageHeight, Left, Bottom)
        objTmp(DX9ToolsR4.GUI.VertexIndex.TopRight) = UVFromPoint(ImageWidth, ImageHeight, Right, Bottom)
        objTmp(DX9ToolsR4.GUI.VertexIndex.BottomLeft) = UVFromPoint(ImageWidth, ImageHeight, Left, Top)
        objTmp(DX9ToolsR4.GUI.VertexIndex.BottomRight) = UVFromPoint(ImageWidth, ImageHeight, Right, Top)

        Return objTmp
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageWidth">The width of the image.</param>
    ''' <param name="ImageHeight">The height of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageWidth As Integer, ByVal ImageHeight As Integer, _
                                    ByVal Left As Integer, ByVal Top As Integer, _
                                    ByVal Right As Integer, ByVal Bottom As Integer) As Vector2()

        Return UVFromRectangle(New SizeF(ImageWidth, ImageHeight), Left, Top, Right, Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, _
                                    ByVal Left As Single, ByVal Top As Single, _
                                    ByVal Right As Single, ByVal Bottom As Single) As Vector2()

        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Left, Top, Right, Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, _
                                    ByVal Left As Single, ByVal Top As Single, _
                                    ByVal Right As Single, ByVal Bottom As Single) As Vector2()

        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Left, Top, Right, Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, _
                                    ByVal Left As Integer, ByVal Top As Integer, _
                                    ByVal Right As Integer, ByVal Bottom As Integer) As Vector2()

        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Left, Top, Right, Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Left">The left location inside the image.</param>
    ''' <param name="Top">The top location inside the image.</param>
    ''' <param name="Right">The right location inside the image.</param>
    ''' <param name="Bottom">The bottom location inside the image.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, _
                                 ByVal Left As Integer, ByVal Top As Integer, _
                                 ByVal Right As Integer, ByVal Bottom As Integer) As Vector2()

        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Left, Top, Right, Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="R">The source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal R As Rectangle) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, R.Left, R.Top, R.Right, R.Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="R">The source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal R As Rectangle) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, R.Left, R.Top, R.Right, R.Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="R">The source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal R As RectangleF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, R.Left, R.Top, R.Right, R.Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="R">The source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal R As RectangleF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, R.Left, R.Top, R.Right, R.Bottom)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal Location As Point, ByVal Size As Size) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal Location As PointF, ByVal Size As Size) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal Location As Point, ByVal Size As SizeF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As Size, ByVal Location As PointF, ByVal Size As SizeF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal Location As Point, ByVal Size As Size) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal Location As PointF, ByVal Size As Size) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal Location As Point, ByVal Size As SizeF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

    ''' <summary>
    ''' Converts a rectangle into an array of 4 texture coordanants.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="Location">The location representing the top left of the source rectangle to convert.</param>
    ''' <param name="Size">The location representing the size of the source rectangle to convert.</param>
    ''' <returns>Returns an array of 4 Vector2 types representing the converted values.</returns>
    Public Shared Function UVFromRectangle(ByVal ImageSize As SizeF, ByVal Location As PointF, ByVal Size As SizeF) As Vector2()
        Return UVFromRectangle(ImageSize.Width, ImageSize.Height, Location.X, Location.Y, _
                               Location.X + Size.Width, Location.Y + Size.Height)
    End Function

#End Region

#Region " UVFromPoint overloads ... "
    'TODO: add more UVFromPoint overloads?
    'ERR: possible stack overflow error with these overloads check and tripple check them

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageWidth">The width of the image.</param>
    ''' <param name="ImageHeight">The height of the image.</param>
    ''' <param name="X">The X location inside the image.</param>
    ''' <param name="Y">The Y location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageWidth As Single, ByVal ImageHeight As Single, ByVal X As Single, ByVal Y As Single) As Vector2
        Dim V As Vector2

        If X = 0 Then V.X = 0 Else V.X = X / ImageWidth
        If Y = 0 Then V.Y = 0 Else V.Y = Y / ImageHeight
        Return V
    End Function

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageWidth">The width of the image.</param>
    ''' <param name="ImageHeight">The height of the image.</param>
    ''' <param name="X">The X location inside the image.</param>
    ''' <param name="Y">The Y location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageWidth As Integer, ByVal ImageHeight As Integer, ByVal X As Integer, ByVal Y As Integer) As Vector2
        Dim V As Vector2

        If X = 0 Then V.X = 0 Else V.X = CSng(X / ImageWidth)
        If Y = 0 Then V.Y = 0 Else V.Y = CSng(Y / ImageHeight)
        Return V
    End Function

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="P">The location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageSize As SizeF, ByVal P As PointF) As Vector2
        Return UVFromPoint(ImageSize.Width, ImageSize.Height, P.X, P.Y)
    End Function

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="P">The location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageSize As Size, ByVal P As PointF) As Vector2
        Return UVFromPoint(ImageSize.Width, ImageSize.Height, P.X, P.Y)
    End Function

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="P">The location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageSize As SizeF, ByVal P As Point) As Vector2
        Return UVFromPoint(ImageSize.Width, ImageSize.Height, P.X, P.Y)
    End Function

    ''' <summary>
    ''' Converts a X/Y value into a texture coordanant.
    ''' </summary>
    ''' <param name="ImageSize">The size of the image.</param>
    ''' <param name="P">The location inside the image.</param>
    ''' <returns>Returns a Vector2 type representing the converted X/Y value.</returns>
    Public Shared Function UVFromPoint(ByVal ImageSize As Size, ByVal P As Point) As Vector2
        Return UVFromPoint(ImageSize.Width, ImageSize.Height, P.X, P.Y)
    End Function
#End Region

#Region " RestrictVector2 overloads ... "
    ' TODO: add more RestrictVector2 overloads.

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="R">The rectangle representing the region to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal R As RectangleF) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < R.Left Then Tmp.X = R.Left
        If V.X > R.Right Then Tmp.X = R.Right
        If V.Y < R.Top Then Tmp.Y = R.Top
        If V.Y > R.Bottom Then Tmp.Y = R.Bottom

        Return Tmp
    End Function

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="R">The rectangle representing the region to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal R As Rectangle) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < R.Left Then Tmp.X = R.Left
        If V.X > R.Right Then Tmp.X = R.Right
        If V.Y < R.Top Then Tmp.Y = R.Top
        If V.Y > R.Bottom Then Tmp.Y = R.Bottom

        Return Tmp
    End Function

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="L">The location representing the top left region to restrict the Vector2 type to.</param>
    ''' <param name="S">The size of the area relative to the location parameter to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal L As Point, ByVal S As Size) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < L.X Then Tmp.X = L.X
        If V.X > L.X + S.Width Then Tmp.X = L.X + S.Width
        If V.Y < L.Y Then Tmp.Y = L.Y
        If V.Y > L.Y + S.Height Then Tmp.Y = L.Y + S.Height

        Return Tmp
    End Function

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="L">The location representing the top left region to restrict the Vector2 type to.</param>
    ''' <param name="S">The size of the area relative to the location parameter to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal L As PointF, ByVal S As Size) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < L.X Then Tmp.X = L.X
        If V.X > L.X + S.Width Then Tmp.X = L.X + S.Width
        If V.Y < L.Y Then Tmp.Y = L.Y
        If V.Y > L.Y + S.Height Then Tmp.Y = L.Y + S.Height

        Return Tmp
    End Function

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="L">The location representing the top left region to restrict the Vector2 type to.</param>
    ''' <param name="S">The size of the area relative to the location parameter to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal L As Point, ByVal S As SizeF) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < L.X Then Tmp.X = L.X
        If V.X > L.X + S.Width Then Tmp.X = L.X + S.Width
        If V.Y < L.Y Then Tmp.Y = L.Y
        If V.Y > L.Y + S.Height Then Tmp.Y = L.Y + S.Height

        Return Tmp
    End Function

    ''' <summary>
    ''' Restricts a Vector2 type within a specified region.
    ''' </summary>
    ''' <param name="V">The vector2 type to restrict.</param>
    ''' <param name="L">The location representing the top left region to restrict the Vector2 type to.</param>
    ''' <param name="S">The size of the area relative to the location parameter to restrict the Vector2 type to.</param>
    ''' <returns>Returns the restricted Vector2 type.</returns>
    Public Shared Function RestrictVector2(ByVal V As Vector2, ByVal L As PointF, ByVal S As SizeF) As Vector2
        Dim Tmp As Vector2

        Tmp = V
        If V.X < L.X Then Tmp.X = L.X
        If V.X > L.X + S.Width Then Tmp.X = L.X + S.Width
        If V.Y < L.Y Then Tmp.Y = L.Y
        If V.Y > L.Y + S.Height Then Tmp.Y = L.Y + S.Height

        Return Tmp
    End Function
#End Region


#Region " BuildMatrixStack overloads ... "
    ''' <summary>
    ''' Builds a matrix from a matrix stack.
    ''' </summary>
    ''' <param name="Stack">A reference to a MatrixStackCollection type.</param>
    ''' <param name="InitialState">The initial or starting matrix to use</param>
    ''' <returns>Returns the built matrix.</returns>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a standard ArgumentNullException Exception if the Stack parameter is nothing.
    ''' </exception>
    Public Shared Function BuildMatrixStack(ByVal Stack As Collections.MatrixStackCollection, ByVal InitialState As Matrix) As Matrix
        Dim M As Matrix = Matrix.Identity
        Dim idx As Integer

        If Stack Is Nothing Then Throw New ArgumentNullException("Stack")
        If Stack.ActualCount = 0 Then Return M

        M = InitialState
        For idx = 0 To Stack.Count
            If Not Stack(idx) Is Nothing Then
                With Stack(idx)
                    Select Case .Function
                        Case MatrixFunction.Add
                            M = Matrix.Add(M, .Matrix)

                        Case MatrixFunction.Invert
                            M = Matrix.Invert(.Matrix)

                        Case MatrixFunction.Multiply
                            M = Matrix.Multiply(M, .Matrix)

                        Case MatrixFunction.MultiplyTranspose
                            M = Matrix.MultiplyTranspose(M, .Matrix)

                        Case MatrixFunction.Subtract
                            M = Matrix.Subtract(M, .Matrix)

                    End Select
                End With
            End If
        Next

        Return M
    End Function

    ''' <summary>
    ''' Builds a matrix from a matrix stack.
    ''' </summary>
    ''' <param name="Stack">A reference to a MatrixStackCollection type.</param>
    ''' <returns>Returns the built matrix.</returns>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a standard ArgumentNullException Exception if the Stack parameter is nothing.
    ''' </exception>
    Public Shared Function BuildMatrixStack(ByVal Stack As Collections.MatrixStackCollection) As Matrix
        Return BuildMatrixStack(Stack, Matrix.Identity)
    End Function
#End Region

#Region " ModeAvailible overloads ... "
    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Width">The width of the display mode to test for.</param>
    ''' <param name="Height">The height of the display mode to test for.</param>
    ''' <param name="Format">The display format of the display mode to test for.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Width As Integer, ByVal Height As Integer, ByVal Format As Direct3D.Format) As Boolean
        Dim Mode As Direct3D.DisplayMode

        With Direct3D.Manager.Adapters.Item(AdapterIndex)
            With .SupportedDisplayModes.Item(Format).GetEnumerator
                .Reset()
                While .MoveNext
                    Mode = CType(.Current, Direct3D.DisplayMode)
                    If Mode.Format = Format And _
                    Mode.Width = Width And _
                    Mode.Height = Height Then
                        Return True
                    End If
                End While
            End With
        End With

        Return False
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Size">Represents the width and height to test for.</param>
    ''' <param name="Format">The display format of the display mode to test for.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Size As Size, ByVal Format As Direct3D.Format) As Boolean
        Return ModeAvailible(AdapterIndex, Size.Width, Size.Height, Format)
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Size">Represents the width and height to test for.</param>
    ''' <param name="Format">The display format of the display mode to test for.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Size As SizeF, ByVal Format As Direct3D.Format) As Boolean
        Return ModeAvailible(AdapterIndex, CInt(Size.Width), CInt(Size.Height), Format)
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Width">The width of the display mode to test for.</param>
    ''' <param name="Height">The height of the display mode to test for.</param>
    ''' <param name="BitDepth">The bit depth to test for. Only 16bit or 32bit is allowed otherwise argument exception is raised.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Width As Integer, ByVal Height As Integer, ByVal BitDepth As Integer) As Boolean
        Select Case BitDepth
            Case 16
                Return ModeAvailible(AdapterIndex, Width, Height, Direct3D.Format.R5G6B5)
            Case 32
                Return ModeAvailible(AdapterIndex, Width, Height, Direct3D.Format.X8R8G8B8)

            Case Else
                Throw New ArgumentException("Bad value passed. Only 16 or 32 is acceptable.", "BitDepth")
        End Select
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Size">Represents the width and height to test for.</param>
    ''' <param name="BitDepth">The bit depth to test for. Only 16bit or 32bit is allowed otherwise argument exception is raised.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Size As Size, ByVal BitDepth As Integer) As Boolean
        Select Case BitDepth
            Case 16
                Return ModeAvailible(AdapterIndex, Size.Width, Size.Height, Direct3D.Format.R5G6B5)
            Case 32
                Return ModeAvailible(AdapterIndex, Size.Width, Size.Height, Direct3D.Format.X8R8G8B8)

            Case Else
                Throw New ArgumentException("Bad value passed. Only 16 or 32 is acceptable.", "BitDepth")
        End Select
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Size">Represents the width and height to test for.</param>
    ''' <param name="BitDepth">The bit depth to test for. Only 16bit or 32bit is allowed otherwise argument exception is raised.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Size As SizeF, ByVal BitDepth As Integer) As Boolean
        Select Case BitDepth
            Case 16
                Return ModeAvailible(AdapterIndex, CInt(Size.Width), CInt(Size.Height), Direct3D.Format.R5G6B5)
            Case 32
                Return ModeAvailible(AdapterIndex, CInt(Size.Width), CInt(Size.Height), Direct3D.Format.X8R8G8B8)

            Case Else
                Throw New ArgumentException("Bad value passed. Only 16 or 32 is acceptable.", "BitDepth")
        End Select
    End Function

    ''' <summary>
    ''' Used to help determin what display modes are availible.
    ''' </summary>
    ''' <param name="AdapterIndex">The index of the adapter to test.</param>
    ''' <param name="Mode">A Direct3D.DisplayMode type representing the display mode to test for.</param>
    ''' <returns>Returns True if successfull.</returns>
    Public Shared Function ModeAvailible(ByVal AdapterIndex As Integer, ByVal Mode As Direct3D.DisplayMode) As Boolean
        Return ModeAvailible(AdapterIndex, Mode.Width, Mode.Height, Mode.Format)
    End Function

    ''' <summary>
    ''' Checks to see if a specific direct draw display mode is availible.
    ''' </summary>
    ''' <param name="Width">The width of the display mode.</param>
    ''' <param name="Height">The height of the display mode.</param>
    ''' <param name="BitCount">The bitcount of the display mode. For example you would specify 16 for a 16bit color mode etc. </param>
    ''' <returns>Will return True if the mode is available else False if it is not.</returns>
    Public Shared Function ModeAvailible(ByVal Device As DirectDraw.Device, ByVal Width As Integer, ByVal Height As Integer, ByVal BitCount As Integer) As Boolean
        Dim Desc As DirectDraw.SurfaceDescription
        Dim Modes As DirectDraw.DisplayModesCollection


        If Device Is Nothing Then Throw New Exceptions.DX9ToolsDDDeviceIsNothing()

        Modes = New DirectDraw.DisplayModesCollection(Device)
        For Each Desc In Modes
            If Desc.Width = Width And Desc.Height = Height Then
                If Desc.PixelFormatStructure.RgbBitCount = BitCount Then Return True
            End If
        Next

        Return False
    End Function
#End Region
End Class
