'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Sunday, December 29, 2002
'=============================================

''' <summary>
''' Provides advanced mesh functionality beyond the basic Direct3D.Mesh type.
''' </summary>
Public Class MeshEx
    ' Inherits Direct3D.Mesh

    ''' <summary>
    ''' THe array that is used to store the materials.
    ''' </summary>
    Protected mtypMaterials() As MaterialTextureCombo
     ''' <summary>
    ''' Represents the number of materials that are stored.
    ''' </summary>
    Protected mintMaterialCount As Integer
     ''' <summary>
    ''' The Direct3D.Mesh type that the MeshEX class uses.
    ''' </summary>
    Protected mobjMesh As Direct3D.Mesh

#Region " New overloads ... "
    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Numfaces">Number of faces for the mesh. The valid range is greater than 0 and one less than the maximum integer  (typically 65534), because the last index is reserved.</param>
    ''' <param name="Numverts">Number of vertices for the mesh. This parameter must be greater than 0.</param>
    ''' <param name="Options">One or more flags from the MeshFlags enumeration that specify creation options for the mesh (excepting the Simplify* and Optimize* flags).</param>
    ''' <param name="VertexFormat">One or more flags from the VertexFormats enumeration that specify the format of the mesh vertices.</param>
    ''' <param name="Device">A Device object that represents the device associated with the mesh.</param>
    Public Sub New(ByVal Numfaces As Integer, ByVal Numverts As Integer, _
                   ByVal Options As Direct3D.MeshFlags, _
                   ByVal VertexFormat As Direct3D.VertexFormats, _
                   ByVal Device As Direct3D.Device)
        mobjMesh = New Direct3D.Mesh(Numfaces, Numverts, Options, VertexFormat, Device)
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Numfaces">Number of faces for the mesh. The valid range is greater than 0 and one less than the maximum integer  (typically 65534), because the last index is reserved.</param>
    ''' <param name="Numverts">Number of vertices for the mesh. This parameter must be greater than 0.</param>
    ''' <param name="Options">One or more flags from the MeshFlags enumeration that specify creation options for the mesh (excepting the Simplify* and Optimize* flags).</param>
    ''' <param name="Declaration">A GraphicsStream containing an array of VertexFormats enumeration values that specify the format of the mesh vertices.</param>
    ''' <param name="Device">A Device object that represents the device associated with the mesh.</param>
    Public Sub New(ByVal Numfaces As Integer, ByVal Numverts As Integer, _
                   ByVal Options As Direct3D.MeshFlags, _
                   ByVal Declaration As Direct3D.GraphicsStream, _
                   ByVal Device As Direct3D.Device)
        mobjMesh = New Direct3D.Mesh(Numfaces, Numverts, Options, Declaration, Device)
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Numfaces">Number of faces for the mesh. The valid range is greater than 0 and one less than the maximum integer  (typically 65534), because the last index is reserved.</param>
    ''' <param name="Numverts">Number of vertices for the mesh. This parameter must be greater than 0.</param>
    ''' <param name="Options">One or more flags from the MeshFlags enumeration that specify creation options for the mesh (excepting the Simplify* and Optimize* flags).</param>
    ''' <param name="Declaration">An array of VertexFormats enumeration values that specify the format of the mesh vertices.</param>
    ''' <param name="Device">A Device object that represents the device associated with the mesh.</param>
    Public Sub New(ByVal Numfaces As Integer, ByVal Numverts As Integer, _
                   ByVal Options As Direct3D.MeshFlags, _
                   ByVal Declaration() As Direct3D.VertexElement, _
                   ByVal Device As Direct3D.Device)
        mobjMesh = New Direct3D.Mesh(Numfaces, Numverts, Options, Declaration, Device)
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    ''' <param name="Mesh">Sets the initial value of the Mesh property.</param>
    Public Sub New(ByVal Mesh As Direct3D.Mesh)
        mobjMesh = Mesh
    End Sub

    ''' <summary>
    ''' Creates a new instance of this class.
    ''' </summary>
    Public Sub New()
        MyBase.new()
    End Sub
#End Region

    ''' <summary>
    ''' Gets a stored material from a specified index.
    ''' </summary>
    ''' <param name="Index">The index of the material to retrieve.</param>
    ''' <returns>Gets a stored material from a specified index.</returns>
    Public Function GetMaterial(ByVal Index As Integer) As MaterialTextureCombo
        Return mtypMaterials(Index)
    End Function

#Region " SetMaterial Overloads ... "
    ''' <summary>
    ''' Sets a stored material in a specified index.
    ''' </summary>
    ''' <param name="Index">The index of the material to retrieve.</param>
    ''' <param name="Mat">The material to be stored.</param>
    Public Sub SetMaterial(ByVal Index As Integer, ByVal Mat As Direct3D.Material)
        mtypMaterials(Index).Material = Mat
    End Sub

    ''' <summary>
    ''' Sets a stored material in a specified index.
    ''' </summary>
    ''' <param name="Index">The index of the material to retrieve.</param>
    ''' <param name="Texture">The texture index to be stored.</param>
    ''' <param name="TextureIndex">The stage where the texture index will be stored.</param>
    Public Sub SetMaterial(ByVal Index As Integer, ByVal Texture As TextureStage, ByVal TextureIndex As Integer)
        mtypMaterials(Index).Textures(Texture) = TextureIndex
    End Sub

    ''' <summary>
    ''' Sets a stored material in a specified index.
    ''' </summary>
    ''' <param name="Index">The index of the material to retrieve.</param>
    ''' <param name="Mat">The material to be stored.</param>
    ''' <param name="Texture">The texture index to be stored.</param>
    ''' <param name="TextureIndex">The stage where the texture index will be stored.</param>
    Public Sub SetMaterial(ByVal Index As Integer, ByVal Mat As Direct3D.Material, _
                           ByVal Texture As TextureStage, ByVal TextureIndex As Integer)
        mtypMaterials(Index).Material = Mat
        mtypMaterials(Index).Textures(Texture) = TextureIndex
    End Sub

    ''' <summary>
    ''' Sets a stored material in a specified index.
    ''' </summary>
    ''' <param name="Index">The index of the material to retrieve.</param>
    ''' <param name="Mat">The material to be stored.</param>
    Public Sub SetMaterial(ByVal Index As Integer, ByVal Mat As MaterialTextureCombo)
        'Dim idx As Integer

        mtypMaterials(Index) = Mat ' .Material = Mat.Material

        '        For idx = 0 To 7
        '            mtypMaterials(Index).Textures(idx) = Mat.Textures(idx)
        '        Next
    End Sub
#End Region

    ''' <summary>
    ''' Represents the number of materials that are stored.
    ''' </summary>
    ''' <returns>Returns the number os materials that are stored.</returns>
    Public ReadOnly Property MaterialCount() As Integer
        Get
            Return mintMaterialCount
        End Get
    End Property

    ''' <summary>
    ''' Sets the number os materials that are stored.
    ''' </summary>
    ''' <param name="Count">The number of materials.</param>
    ''' <param name="Preserve">If True will preserve existing materials.</param>
    Public Sub SetMaterialCount(ByVal Count As Integer, Optional ByVal [Preserve] As Boolean = True)
        Dim idx, Old As Integer

        'If Count < mintMaterialCount Then
        '    For idx = Count To mintMaterialCount
        '        If Not mtypMaterials(idx).Texture Is Nothing Then
        '            mtypMaterials(idx).Texture.Dispose()
        '        End If
        '        mtypMaterials(idx).Texture = Nothing
        '    Next
        'End If
        If Count < 1 Then
            Throw New ArgumentOutOfRangeException("Count", Count, "Count must be a value of 1 or heigher.")
        End If

        If [Preserve] Then
            If mtypMaterials Is Nothing Then
                Old = 0
            Else
                Old = mtypMaterials.Length
            End If

            ReDim Preserve mtypMaterials(Count - 1)
            For idx = Old To mtypMaterials.Length - 1
                mtypMaterials(idx) = New MaterialTextureCombo()
                GeneralHelpers.RemoveTextureIndexes(mtypMaterials(idx))
            Next
        Else
            ReDim mtypMaterials(Count - 1)
            For idx = 0 To mtypMaterials.Length - 1
                mtypMaterials(idx) = New MaterialTextureCombo()
                GeneralHelpers.RemoveTextureIndexes(mtypMaterials(idx))
            Next
        End If
        mintMaterialCount = Count
    End Sub


#Region " LoadMesh overloads ... "
    ''' <summary>
    ''' Loads a mesh from a file.
    ''' </summary>
    ''' <param name="Filename">The mesh file to load.</param>
    ''' <param name="Dev">A reference to a Direct3D.Device type.</param>
    ''' <param name="Textures">A reference to a DX9ToolsR4.SimpleTextureCollection type that any textures will be added to.</param>
    ''' <param name="MeshFlags">Defines flags used to specify creation options for a mesh.</param>
    ''' <returns>Returns a reference to a new MeshEx type.</returns>
    Public Shared Function LoadMesh(ByVal Filename As String, ByVal Dev As Direct3D.Device, _
                                    ByRef Textures As DX9ToolsR4.Collections.SimpleTextureCollection, _
                                    Optional ByVal MeshFlags As Direct3D.MeshFlags = Direct3D.MeshFlags.Managed) As MeshEx
        Return GeoHelpers.LoadMesh(Filename, Dev, Textures, MeshFlags)
    End Function
#End Region

    ''' <summary>
    ''' Gets or Sets the Direct3D.Mesh type that the MeshEX class uses.
    ''' </summary>
    ''' <returns>Returns a reference to a Direct3D.Mesh type.</returns>
    Public Property Mesh() As Direct3D.Mesh
        Get
            Return mobjMesh
        End Get
        Set(ByVal Value As Direct3D.Mesh)
            mobjMesh = Value
        End Set
    End Property
End Class