'Created by: X
'E-Mail: mailto:createdbyx@yahoo.com
'Web: http://www.createdbyx.com/
'Date: Sunday, December 29, 2002
'=============================================

Imports Microsoft.DirectX.Direct3D.Geometry

Namespace Modifiers
    Public Class PositionModifier
        Implements IModify
        Private mobjMesh As MeshAdv
        Public Position As Vector3


        Public Property Mesh() As CBX.Libs.DX9ToolsR4.MeshAdv Implements CBX.Libs.DX9ToolsR4.IModify.Mesh
            Get
                Return mobjMesh
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.MeshAdv)
                mobjMesh = Nothing
            End Set
        End Property

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            mobjMesh = Nothing
        End Sub

        Public Overridable Sub ApplyModifier() Implements CBX.Libs.DX9ToolsR4.IModify.ApplyModifier
            mobjMesh.Matrix.Multiply(Matrix.Translation(Me.Position))
        End Sub
    End Class

    Public Class MovmentModifier
        Inherits PositionModifier
        Public Direction As Vector3

        Public Overrides Sub ApplyModifier()
            Me.Position.X += Me.Direction.X
            Me.Position.Y += Me.Direction.Y
            Me.Position.Z += Me.Direction.Z

            Me.Mesh.Matrix.Multiply(Matrix.Translation(Me.Position))
        End Sub
    End Class

    Public Class RotationModifier
        Implements IModify
        Private mobjMesh As MeshAdv
        Public Rotation As Vector3


        Public Property Mesh() As CBX.Libs.DX9ToolsR4.MeshAdv Implements CBX.Libs.DX9ToolsR4.IModify.Mesh
            Get
                Return mobjMesh
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.MeshAdv)
                mobjMesh = Nothing
            End Set
        End Property

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            mobjMesh = Nothing
        End Sub

        Public Overridable Sub ApplyModifier() Implements CBX.Libs.DX9ToolsR4.IModify.ApplyModifier
            Dim Tmp As Matrix

            Tmp.RotateYawPitchRoll(Me.Rotation.Y, Me.Rotation.X, Me.Rotation.Z)
            Me.Mesh.Matrix.Multiply(Tmp)
        End Sub
    End Class

    Public Class RotatingModifier
        Inherits RotationModifier
        Public RotationSpeed As Vector3
        Public Wrap As Integer = 360
        Public DoWrap As Boolean = True


        Public Overrides Sub ApplyModifier()
            Dim Tmp As Matrix
            Dim Radian As Single

            Me.Rotation.X += Me.RotationSpeed.X
            Me.Rotation.Y += Me.RotationSpeed.Y
            Me.Rotation.Z += Me.RotationSpeed.Z
            If Me.DoWrap Then
                Radian = DegreeToRadian(Me.Wrap)
                If Me.Rotation.X > Radian Then Me.Rotation.X = Me.Rotation.X - Radian
                If Me.Rotation.Y > Radian Then Me.Rotation.Y = Me.Rotation.Y - Radian
                If Me.Rotation.Z > Radian Then Me.Rotation.Z = Me.Rotation.Z - Radian
            End If

            Tmp.RotateYawPitchRoll(Me.Rotation.Y, Me.Rotation.X, Me.Rotation.Z)
            Me.Mesh.Matrix.Multiply(Tmp)
        End Sub
    End Class

    Public Class ScaleModifier
        Implements IModify
        Private mobjMesh As MeshAdv
        Public Scale As Vector3


        Public Property Mesh() As CBX.Libs.DX9ToolsR4.MeshAdv Implements CBX.Libs.DX9ToolsR4.IModify.Mesh
            Get
                Return mobjMesh
            End Get
            Set(ByVal Value As CBX.Libs.DX9ToolsR4.MeshAdv)
                mobjMesh = Nothing
            End Set
        End Property

        Protected Overrides Sub Finalize()
            MyBase.Finalize()
            mobjMesh = Nothing
        End Sub

        Public Overridable Sub ApplyModifier() Implements CBX.Libs.DX9ToolsR4.IModify.ApplyModifier
            mobjMesh.Matrix.Multiply(Matrix.Scaling(Me.Scale))
        End Sub
    End Class

    Public Class PRSModifier
        Inherits PositionModifier
        Public Rotation As Vector3
        Public Scale As Vector3

        Public Overrides Sub ApplyModifier()
            ' apply position modifier
            MyBase.ApplyModifier()

            Dim Tmp As Matrix

            ' apply rotation modifier
            Tmp.RotateYawPitchRoll(Me.Rotation.Y, Me.Rotation.X, Me.Rotation.Z)
            Me.Mesh.Matrix.Multiply(Tmp)

            ' apply scaling modifier
            Me.Mesh.Matrix.Multiply(Matrix.Scaling(Me.Scale))
        End Sub

        Public Sub ApplyModifierEx(Optional ByVal ApplyRotation As Boolean = True, Optional ByVal ApplyScale As Boolean = True)
            ' apply position modifier
            MyBase.ApplyModifier()

            Dim Tmp As Matrix

            If ApplyRotation Then
                ' apply rotation modifier
                Tmp.RotateYawPitchRoll(Me.Rotation.Y, Me.Rotation.X, Me.Rotation.Z)
                Me.Mesh.Matrix.Multiply(Tmp)
            End If

            ' apply scaling modifier
            If ApplyScale Then Me.Mesh.Matrix.Multiply(Matrix.Scaling(Me.Scale))
        End Sub
    End Class
End Namespace